/*!
 * Copyright (c) 2024 ENEO Tecnología S.L.
 * This file is part of redBorder.
 * redBorder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * redBorder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with redBorder. If not, see <http://www.gnu.org/licenses/>.
 */

$(document).on("propertychange keyup input paste", "#search-user", function(event){
  var incidentId = $(this).data('incident-id');
  var query = $(this).val();

  if (query.length > 2) {
    $.ajax({
      url: '/incidents/' + incidentId + '/search_user',
      dataType: 'json',
      data: { q: query },
      success: function(data) {
        $('#user-results').empty();
        data.forEach(function(user) {
          $('#user-results').append('<a href="#" class="list-group-item list-group-item-action" data-id="' + user.id + '">' + user.name + '</a>');
        });
      }
      });
  } else {
    $('#user-results').empty();
  }
});

$(document).on("click", "#linked-incidents", function() {
  var url = window.location.href;
  var incidentId = url.match(/\/incidents\/(\d+)/)[1];
  var width = "500px";
  
  openSidebar('<div class="circle-loader"></div>', width);

  $.ajax({
    url: '/incidents/' + incidentId + '/sidebar_linked_incidents_content',
    data: { 
      incident_id: incidentId 
    },
    success: function(data) {
      openSidebar(data, width);
    },
  });
});

showObservableInfo = function (observable) {
  var url = window.location.href;
  var incidentId = url.match(/\/incidents\/(\d+)/)[1];
  var width = "500px";
  
  try {
    openSidebar('<div class="circle-loader"></div>', width);

    $.ajax({
      url: '/incidents/' + incidentId + '/sidebar_observable_content',
      data: { 
        observable: observable, 
        incident_id: incidentId 
      },
      success: function(data) {
        openSidebar(data, width);
      },
    });
  } catch (e) {
    openSidebar('<div class="sidebar-header sidebar-title bold">An unexpected error occurred. Close the sidebar and try again.</div>', width);
    console.error('Error during AJAX call:', e);
  }  
}

$(document).on("click", ".incident-row", function() {
  var incidentId = $(this).data('id');
  var width = "380px";

  openSidebar('<div class="circle-loader"></div>', width);

  $.ajax({
      url: '/incidents/' + incidentId + '/sidebar_content',
      success: function(data) {
          openSidebar(data, width);
      }
  });
});

function openSidebar(content, width) {
  $('#sidebar').css('width', width);
  $('#sidebar-content').html(content);
}

function closeSidebar() {
  $('#sidebar').css('width', '0');
}


$(document).on("click", "#incident-select-all", function(event) {
  $('input[name="selected_incidents[]"]').each(function() {
    this.checked = event.target.checked;
  });
});

$(document).on("click", "#incident-dropdownMenuButton", function(event) {
  event.stopPropagation();
  $(this).closest('.dropdown').toggleClass('show-options');
});

$(document).ready(function() {
  var storedIds = JSON.parse(localStorage.getItem('selected_incidents')) || [];

  storedIds.forEach(function(id) {
    $('input[name="selected_incidents[]"][value="' + id + '"]').prop('checked', true);
  });
});

// Actualizar el localStorage cada vez que se selecciona o deselecciona un checkbox
$(document).on('change', 'input[name="selected_incidents[]"]', function() {
  var selected = JSON.parse(localStorage.getItem('selected_incidents')) || [];

  if (this.checked) {
    // Añadir el ID si no está ya en el array
    if (!selected.includes(parseInt(this.value, 10))) {
      selected.push(parseInt(this.value, 10));
    }
  } else {
    // Remover el ID si se deselecciona
    selected = selected.filter(function(id) {
      return id !== parseInt(this.value, 10);
    }.bind(this));
  }

  localStorage.setItem('selected_incidents', JSON.stringify(selected));
});

$(document).on("click", "#delete-selected", function(event) {
  var bulkDeleteForm = $('#bulk-delete-form');
  var deleteIncidentIdsContainer = $('#bulk-delete-incident-ids');
  var deletePageInput = $('#bulk-delete-page');

  event.preventDefault();

  var selected = JSON.parse(localStorage.getItem('selected_incidents')) || [];

  if (selected.length > 0) {
    if (confirm('Are you sure you want to delete the selected incidents?')) {
      // Limpiar los valores antiguos
      deleteIncidentIdsContainer.empty();
      selected.forEach(function(id) {
        deleteIncidentIdsContainer.append('<input type="hidden" name="incident_ids[]" value="' + id + '">');
      });

      deletePageInput.val(new URLSearchParams(window.location.search).get('page') || 1);
      localStorage.removeItem('selected_incidents');
      bulkDeleteForm.submit();
    }
  } else {
    alert('Please select at least one incident.');
  }
});

$(document).on("click", "#link-selected", function(event) {
  var bulkUpdateLinksForm = $('#bulk-update-link-form');
  var incidentIdsContainer = $('#bulk-update-incident-links-ids');
  var updateLinkPageInput = $('#bulk-update-links-page');

  event.preventDefault();

  var selected = JSON.parse(localStorage.getItem('selected_incidents')) || [];

  if (selected.length > 0) {
    // Limpiar los valores antiguos en el formulario
    incidentIdsContainer.empty();
    selected.forEach(function(id) {
      incidentIdsContainer.append('<input type="hidden" name="incident_ids[]" value="' + id + '">');
    });

    updateLinkPageInput.val(new URLSearchParams(window.location.search).get('page') || 1);
    localStorage.removeItem('selected_incidents');
    bulkUpdateLinksForm.submit();
  } else {
    alert('Please select at least one incident.');
  }
});

$(document).on("click", ".dropdown-item[data-status]", function(event) {
  var bulkUpdateForm = $('#bulk-update-form');
  var statusInput = $('#bulk-update-status');
  var updateIncidentIdsContainer = $('#bulk-update-incident-ids');
  var updatePageInput = $('#bulk-update-page');

  event.preventDefault();

  var status = $(this).data('status');
  var selected = JSON.parse(localStorage.getItem('selected_incidents')) || [];

  if (selected.length > 0) {
    statusInput.val(status);
    // Limpiar los valores antiguos
    updateIncidentIdsContainer.empty();
    selected.forEach(function(id) {
      updateIncidentIdsContainer.append('<input type="hidden" name="incident_ids[]" value="' + id + '">');
    });

    updatePageInput.val(new URLSearchParams(window.location.search).get('page') || 1);
    localStorage.removeItem('selected_incidents');
    bulkUpdateForm.submit();
  } else {
    alert('Please select at least one incident.');
  }
});

// $(document).on("click", "#playbook-sections .nav-pills .disabled").click(function(e) {
//   // Establecer la pestaña activa al cargar la página
//   var activeTab = $('#playbook-sections .nav-pills li.active a').attr('href');
//   $(activeTab).addClass('in active');

//   // Manejar el clic en las pestañas
//   $('#playbook-sections .nav-pills a').click(function(e) {
//     if ($(this).hasClass('disabled')) {
//       e.preventDefault();
//     } else {
//       var target = $(this).attr('href');
//       $('#playbook-sections .tab-pane').removeClass('in active');
//       $(target).addClass('in active');
//     }
//   });
// });