DittoCode::HoldFile.if 'ENTERPRISE'

#######################################################################
# Copyright (c) 2024 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# Provide API methods to work with organizations
#
class Api::V1::IncidentsController < Api::V1::BaseSensorsController
  # ----------------------------------------------
  # Before and after actions
  # ----------------------------------------------
  # skip_before_action :authenticate_user_from_token!, :user_setup_api

  #
  # POST /api/v1/incidents/create
  #
  def create
    return error_422("You don\'t have privileges to do that.") unless current_user.super_admin?

    incident = Incident.new

    incident.name = params[:name] ? params[:name] : 'Unknown Incident'
    incident.priority = Priority.find_by(name: params[:priority]) || Priority.find_by(name: 'unknown')
    incident.uuid = params[:uuid]
    incident.incident_status = IncidentStatus.find_by(name: IncidentStatus::NEW)
    incident.source = params[:source] if params[:source] && !params[:source].empty?
    incident.sensor_id = Sensor.find_by(uuid: params[:domain_uuid]).id rescue Sensor.root.id

    if incident.save
      render json: { success: true, id: incident.id }, status: :created
    else
      render json: { 
        success: false, 
        errors: incident.errors.full_messages 
      }, status: :unprocessable_entity
    end
  end # create

  #
  # POST /api/v1/incidents/link
  #
  def link
    return error_422("You don\'t have privileges to do that.") unless current_user.super_admin?

    parent_incident = Incident.find_by(uuid: params[:parent_incident_uuid])
    child_incident = Incident.find_by(uuid: params[:child_incident_uuid])

    if parent_incident.nil? || child_incident.nil?
      render json: { success: false, errors: "One or both incidents not found." }, status: :not_found
      return
    end

    if IncidentLink.link_exists?(parent_incident, child_incident)
      render json: { success: false, errors: "The link between these incidents already exists." }, status: :unprocessable_entity
      return
    end

    begin
      # Create the bidirectional link
      parent_incident.linked_incidents << child_incident
      render json: { success: true, message: "Incidents linked successfully." }, status: :created
    rescue ActiveRecord::RecordInvalid => e
      render json: { success: false, errors: e.message }, status: :unprocessable_entity
    end
  end
end
