#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class ApplicationObjectsController < ApplicationController
  before_action :set_application_object, only: [:edit, :update, :destroy]

  layout false, only: [:new, :edit, :import]

  # GET /application_objects
  # GET /application_objects.json
  def index
    @application_objects = ApplicationObject.all.order(:name)
                           .paginate(page: 1, per_page: current_user.per_page_count)
  end

  # GET /application_objects/new
  def new
    @application_object = ApplicationObject.new(value: params[:value])
  end

  # POST /application_objects
  # POST /application_objects.json
  def create
    @application_object = ApplicationObject.new(application_object_params)
    return unless @application_object.save && ApplicationObject.all.one?

    # Get objects
    @application_objects = ApplicationObject.all
                           .paginate(page: 1, per_page: current_user.per_page_count)
  end

  # PATCH/PUT /application_objects/1
  # PATCH/PUT /application_objects/1.json
  def update
    @application_object.update(application_object_params)
  end

  # DELETE /application_objects/1
  # DELETE /application_objects/1.json
  def destroy
    @application_object.destroy if @application_object
  end

  #
  # Search in applications
  #
  def search
    filter = ['( LOWER(name) LIKE :search or ' \
              'LOWER(description) LIKE :search or LOWER(value) LIKE :search)',
              search: "%#{params[:search].try(:downcase)}%"]

    @application_objects =
      ApplicationObject.where(filter).order(:name).paginate(
        page: params[:page],
        per_page: current_user.per_page_count
      )
  end

  #
  # Export all applications as CSV
  #
  def export
    filename = in_time_machine { Time.now.strftime('%Y%m%d%H%M%S') + '-Applications.csv' }
    filter = ['( LOWER(name) LIKE :search or ' \
              'LOWER(description) LIKE :search or LOWER(value) LIKE :search)',
              search: "%#{params[:search].try(:downcase)}%"]
    content = ApplicationObject.where(filter).order(:name)
              .map { |o| "#{o.name};#{o.value};#{o.description}" }.join("\n")
    send_data content, filename: filename
  end

  #
  # Import applications from a given CSV file.
  #
  def import_file
    content_file = params[:application_object][:csv_file]

    if params[:add_vars] == 'true'
      count = ApplicationObject.add_apps_from_file(content_file)
      msg = "#{count[0]} #{I18n.t('controllers.added')} #{count[1]} #{I18n.t('controllers.updated')}"
    else
      count = ApplicationObject.replace_apps_from_file(content_file)
      msg = "#{count} #{I18n.t('controllers.added')}"
    end

    redirect_to objects_path, notice: msg
  end

  private

  # Use callbacks to share common setup or constraints between actions.
  def set_application_object
    @application_object = ApplicationObject.find_by(id: params[:id])
    return if @application_object
    # The object doesn't exist!
    rb_flash "#{I18n.t('controllers.not_exist')}", type: :error
  end

  # Never trust parameters from the scary internet, only allow the white list through.
  def application_object_params
    params.require(:application_object).permit(:name, :value, :description)
  end
end
