#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################
class NotificationsController < ApplicationController
  ## ======================================================================
  # => Before actions and filters
  ## ======================================================================

  before_action :set_notification, only: [:show, :destroy]

  ## ======================================================================
  # => Actions
  ## ======================================================================

  # GET /notifications
  # GET /notifications.json
  def index
    @notifications = Notification.where(user: current_user)
                     .order('created_at DESC')
                     .paginate(page: params[:page],
                               per_page: current_user.per_page_count)
  end

  # DELETE /notifications/1
  # DELETE /notifications/1.json
  def destroy
    if params[:id].eql?('all')
      Notification.where(user: current_user).delete_all
    else
      @notification.destroy
    end
    respond_to do |format|
      format.html { redirect_to notifications_url }
      format.json { head :no_content }
    end
  end

  def open
    Notification.where(user: current_user, read_status: false)
      .update_all(read_status: true)

    render nothing: true
  end

  def reload
  end

  private

  #
  # => Use callbacks to share common setup or constraints between actions.
  #
  def set_notification
    return if params[:id].eql?('all')
    @notification = Notification.find_by(user: current_user, id: params[:id])
    return error_404('') unless @notification
  end

  #
  # => Never trust parameters from the scary internet,
  #    only allow the white list through.
  #
  def notification_params
    params.require(:notification).permit(:user_id, :read_status, :comment, :incident_id)
  end
end
