#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################
class SessionsController < Devise::SessionsController
  layout 'login'
  after_filter :log_failed_login, :only => :new
  before_action only: [:new]

  def create
    self.resource = warden.authenticate!(auth_options)
    if resource.pwd_changed_at == nil
      current_user.update(pwd_changed_at: Time.now)
    end
    if ((current_user.pwd_changed_at + 3.months) < Time.now || current_user.force_pwd_change == true)
      raw, new_token = Devise.token_generator.generate(User, :reset_password_token)
      current_user.update_attributes(:reset_password_token => new_token, :reset_password_sent_at => Time.now)
      redirect_to reset_url(current_user, reset_password_token: new_token), alert: "#{I18n.t 'controllers.password_to_old'}"
    else
      sign_in(resource_name, resource)
      yield resource if block_given?
      respond_with resource, location: after_sign_in_path_for(resource)
      last_sign_in_description = Notification.new(user: current_user, comment: "The last login access from the user #{current_user.name} was at: #{current_user.last_sign_in_at.strftime('%a %b, %Y %I:%M %p')}")
      last_sign_in_description.save()
      pwd_update_notice
      # Workarround: Without this is not auditing the first login
      current_user.audit_sign_in if current_user.sign_in_count == 1
    end
  end

  def pwd_update_notice
    if ((@user.pwd_changed_at + 2.months) < Time.now)
      pwd_expiration_notice = Notification.new(user: current_user, comment: "Reset you password before the #{(current_user.pwd_changed_at + 3.months).strftime('%a, %d %b %Y ')} .")
      pwd_expiration_notice.save()
    end
  end

  def destroy
    current_user.update(last_sign_out_at: Time.now.utc)
    # Workarround: Without this is not auditing the first destroy
    current_user.audit_last_sign_out_at if current_user.sign_in_count == 1
    super
  end

  private
  def log_failed_login
    request_ip = (request.headers['X-Forwarded-For'] ||
              request.headers['X-Real-IP'] ||
              request.remote_ip).split(',').first

    Manager.all.first.audit_fail_login("Failed login with #{request.filtered_parameters["user"]["username"]} from IP #{request_ip}") if failed_login?
  end
  def failed_login?
    (options = env["warden.options"]) && options[:action] == "unauthenticated"
  end

end
