#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################
class SettingsController < ApplicationController
  ## ======================================================================
  # => Before actions and filters
  ## ======================================================================

  before_action :redirect_to_domain_edit, only: [:show]
  before_action :require_super_reader_privileges,
                only: [:show, :redirect_to_domain_edit]
  before_action :require_super_administrative_privileges,
                except: [:show, :redirect_to_domain_edit]

  ## ======================================================================
  # => Actions
  ## ======================================================================

  def show
    @manager_role = Chef::Role.manager
    @sensor = Sensor.root
    @settings = {}
    Setting.all.map do |s|
      @settings[s.name.to_sym] = s.value
    end

    if Redborder::AI.is_ai_service_enabled
      @managers = Manager.all.sort_by(&:name)
      # Init ai_allowed_cpus variable
      unless @settings[:ai_allowed_cpus]
        @settings[:ai_allowed_cpus] = {}
        @managers.each do |manager|
          next unless manager.services['redborder-ai'] && manager.services['redborder-ai'][:status] && manager.services['redborder-ai'][:ok]

          @settings[:ai_allowed_cpus][manager.name.to_sym] = ['0']
        end
      end

      @max_cores_cpu = 0
      @managers.each do |manager|
        @max_cores_cpu = manager.node['cpu']['total'] if manager.node['cpu']['total'].to_i > @max_cores_cpu
      end
    end

    @cluster = load_databag('rBglobal', 'cluster') || {}

    @rsa_content = ''
    File.open('config/rsa').each do |line|
      @rsa_content << line
    end
    # Get default rules
    @incident_statuses = IncidentStatus.all
    cluster = Redborder::Cluster.instance
    @default_druid_rules = cluster.default_datasource.rules
    @fixed_hosts = Chef::DataBagItem.load('rBglobal', 'hosts')['hosts'] rescue {}
    DittoCode::Exec.if 'ENTERPRISE' do
      @segments_s3 = load_databag('passwords', 's3') || {}
      @amazon_cloudwatch = load_databag('passwords', 'cloudwatch') || {}
      @tiers = cluster.tiers.map { |tier| Tier.default_tier_name(tier.name) }
    end

    # Models that are enable and downloaded
    @models = ModelSource.where(enable: true).select {|m| m.downloaded? }
  end

  #
  # Update current settings. This method update Manager role, Sensor root role,
  # Databags and settings.
  #
  def update
    settings = (params[:settings] || {}).dup.symbolize_keys
    audit_messages = []

    settings.delete(:role)

    if settings[:rule_update_time].blank?
      settings[:rule_update_time] = '00:00'
    end

    @manager_role = Chef::Role.manager
    audit_messages.push("Smtp relayhost change to #{params['settings']['role']['redborder']['smtp']['relayhost']}") if !@manager_role.override_attributes['redborder']['smtp']['relayhost'] or @manager_role.override_attributes['redborder']['smtp']['relayhost'] != params['settings']['role']['redborder']['smtp']['relayhost']
    @manager_role.override_attributes['redborder']['smtp']['relayhost'] = params['settings']['role']['redborder']['smtp']['relayhost']

    audit_messages.push("Rsyslog mode change to #{params['settings']['role']['redborder']['rsyslog']['mode']}") if !@manager_role.override_attributes['redborder']['rsyslog']['mode'] or @manager_role.override_attributes['redborder']['rsyslog']['mode'] != params['settings']['role']['redborder']['rsyslog']['mode']
    @manager_role.override_attributes['redborder']['rsyslog']['mode'] = params['settings']['role']['redborder']['rsyslog']['mode']

    audit_messages.push("Rsyslog Servers change to #{params['settings']['role']['redborder']['rsyslog']['servers']}") if !@manager_role.override_attributes['redborder']['rsyslog']['servers'] or @manager_role.override_attributes['redborder']['rsyslog']['servers'] != params['settings']['role']['redborder']['rsyslog']['servers'].split(/\s*,\s*/)
    @manager_role.override_attributes['redborder']['rsyslog']['servers'] =  params['settings']['role']['redborder']['rsyslog']['servers'].split(/\s*,\s*/)

    audit_messages.push("Rsyslog Protocol change to #{params['settings']['role']['redborder']['rsyslog']['protocol']}") if !@manager_role.override_attributes['redborder']['rsyslog']['protocol'] or @manager_role.override_attributes['redborder']['rsyslog']['protocol'] != params['settings']['role']['redborder']['rsyslog']['protocol']
    @manager_role.override_attributes['redborder']['rsyslog']['protocol'] = params['settings']['role']['redborder']['rsyslog']['protocol']

    audit_messages.push("Rsyslog NTP servers change to #{params['settings']['role']['redborder']['ntp']['servers']}") if !@manager_role.override_attributes['redborder']['ntp']['servers'] or @manager_role.override_attributes['redborder']['ntp']['servers'] != params['settings']['role']['redborder']['ntp']['servers'].split(/\s*,\s*/)
    @manager_role.override_attributes['redborder']['ntp']['servers'] = params['settings']['role']['redborder']['ntp']['servers'].split(/\s*,\s*/)

    audit_messages.push("Rsyslog Loggly Token changed") if !@manager_role.override_attributes['redborder']['rsyslog']['loggly']['token'] or @manager_role.override_attributes['redborder']['rsyslog']['loggly']['token'] != params['settings']['role']['redborder']['rsyslog']['loggly']['token']
    @manager_role.override_attributes['redborder']['rsyslog']['loggly']['token'] = params['settings']['role']['redborder']['rsyslog']['loggly']['token']

    if settings[:proxy] == '0'
      audit_messages.push("Proxy Settings change from enabled to disabled") if (@manager_role.override_attributes['redborder']['proxy'] and ((@manager_role.override_attributes['redborder']['proxy']['enabled'] and @manager_role.override_attributes['redborder']['proxy']['enabled'] != false) or !@manager_role.override_attributes['redborder']['proxy'].keys.include?'enabled'))
      @manager_role.override_attributes['redborder']['proxy'] = { 'enabled' => false }
    else
      # We need to check if user and password change before we wipe
      user_changed =    (!@manager_role.override_attributes['redborder']['proxy']['user'] or @manager_role.override_attributes['redborder']['proxy']['user'] != settings[:proxy_user]) ? true : false
      password_change = (!@manager_role.override_attributes['redborder']['proxy']['user'] or @manager_role.override_attributes['redborder']['proxy']['user'] != settings[:proxy_user]) ? true : false

      #audit_messages.push("Proxy Settings change from disabled to enabled") if !@manager_role.override_attributes['redborder']['proxy'] or (@manager_role.override_attributes['redborder']['proxy']['enabled'] and @manager_role.override_attributes['redborder']['proxy']['enabled'] == false)
      @manager_role.override_attributes['redborder']['proxy'] ||= {}

      #audit_messages.push("Proxy Address change to #{settings[:proxy_address]}") if !@manager_role.override_attributes['redborder']['proxy']['address'] or @manager_role.override_attributes['redborder']['proxy']['address'] != settings[:proxy_address]
      @manager_role.override_attributes['redborder']['proxy']['address'] = settings[:proxy_address]

      #audit_messages.push("Proxy Port change to #{settings[:proxy_port]}") if !@manager_role.override_attributes['redborder']['proxy']['port'] or @manager_role.override_attributes['redborder']['proxy']['port'] != settings[:proxy_port]
      @manager_role.override_attributes['redborder']['proxy']['port'] = settings[:proxy_port]

      audit_messages.push("Proxy User change to #{settings[:proxy_user]}") if user_changed
      @manager_role.override_attributes['redborder']['proxy']['user'] = settings[:proxy_user]

      audit_messages.push("Proxy Password changed") if password_change
      @manager_role.override_attributes['redborder']['proxy']['password'] = settings[:proxy_password]
    end

    # Save outliers advanced settings
    if @manager_role.override_attributes['redborder']['outliers'].nil?
      @manager_role.override_attributes['redborder']['outliers'] = {}
    end
    @manager_role.override_attributes['redborder']['outliers']['WINDOW_SIZE'] = settings[:WINDOW_SIZE]
    @manager_role.override_attributes['redborder']['outliers']['AUTO_SENSITIVITY_ANOMALY_PCNT'] = settings[:AUTO_SENSITIVITY_ANOMALY_PCNT]
    @manager_role.override_attributes['redborder']['outliers']['AUTO_SENSITIVITY_SD'] = settings[:AUTO_SENSITIVITY_SD]

    # Save weather settings
    if @manager_role.override_attributes['redborder']['weather'].nil?
      @manager_role.override_attributes['redborder']['weather'] = {}
    end
    @manager_role.override_attributes['redborder']['weather']['api_key'] = settings[:weather_api_key]

    # Save SSO / SAML setting
    sp_sso_cert_path_tmp = "#{Rails.root}/tmp/sp-sso-tmp.crt"
    sp_sso_priv_key_path_tmp = "#{Rails.root}/tmp/sp-sso-tmp.key"
    sp_sso_cert = nil
    sp_sso_priv_key = nil
    # Public SSO cert
    begin
      sp_sso_cert = saml_cert_content(sp_sso_cert_path_tmp, 'external_saml_crt', 'saml_crt')
    rescue Net::HTTPFatalError => e
      Rails.logger.info 'INFO: Removing external_saml_crt from databag certs saml'
      delete_sp_sso_cert_file
    end
    # Private SSO key
    begin
      sp_sso_priv_key = saml_cert_content(sp_sso_priv_key_path_tmp, 'external_saml_key', 'saml_key')
    rescue Net::HTTPFatalError => e
      Rails.logger.info 'INFO: Removing external_saml_priv_key from databag certs saml'
      delete_sp_sso_priv_key_file
    end
    # Saving SSO settings
    @manager_role.override_attributes['redborder']['sso_enabled'] = settings[:sso_enabled]
    Devise.saml_configure do |saml_settings|
      saml_settings.assertion_consumer_service_url  = settings[:sp_sso_target]
      saml_settings.sp_entity_id                    = settings[:sp_entity_id]
      saml_settings.idp_sso_target_url              = settings[:idp_sso_target_url]
      saml_settings.idp_slo_target_url              = settings[:idp_slo_target_url]
      saml_settings.idp_cert                        = settings[:idp_certificate]
      if !sp_sso_cert.nil?
        saml_settings.certificate                   = sp_sso_cert
      end
      if  !sp_sso_priv_key.nil?
        saml_settings.private_key                   = sp_sso_priv_key
      end
    end
    # Modify SSO user creation
    Devise.saml_update_resource_hook = ->(user, saml_response, _auth_value) {
      saml_response.attributes.resource_keys.each do |key|
        if key == 'user_role'
          user_role = saml_response.attribute_value_by_resource_key(key)
          # Get the sensor id of the organization of sso_user_sensor_map.yml
          user_sensor_id = Sensor.where(type: 1).find_by(name: RedBorder::SSO_USER_SENSOR_MAP[user_role]).id rescue Sensor.root.id
          user.send 'sensor_id=', user_sensor_id
        else
          user.send "#{key}=", saml_response.attribute_value_by_resource_key(key)
        end
      end
      unless saml_response.attribute_value_by_resource_key('user_role')
        user.sensor_id = Sensor.find_by(name: 'root').id
      end
      user.admin = '0'
      user.locked_at = Time.now unless settings[:sso_lock_new_users] == '0'
      user.created_by_sso = '1'
      user.remember_created_at = Time.now
      user.save!
    }

    # Update the used model
    @manager_role.override_attributes['redborder']['ai_selected_model'] = settings[:ai_selected_model]

    enabled_cores = params[:ai_allowed_cpus]

    modified_enabled_cores = {}

    if enabled_cores
      enabled_cores.each do |host, cores|
        modified_enabled_cores[host.to_sym] = cores.join(',')
      end
    end

    begin
      modified_enabled_cores.each do |host, cores|
        m = Chef::Node.load host.to_s
        next unless m

        m.normal['redborder']['redborder-ai']['cpus'] = cores
        m.save
      end

      # Asignar el hash modificado después de la iteración
      settings[:ai_allowed_cpus] = modified_enabled_cores
    rescue StandardError => e
      Rails.logger.error "Cannot save redborder-ai cpus in node"
      Rails.logger.error e.message
    end

    # Update incidents settings
    @manager_role.override_attributes['redborder']['incidents_priority_filter'] = settings[:incidents_priority_filter]

    # Update cli_banner setting
    cli_banner = settings[:cli_banner] || ''
    # @proxy_sensor_role = Chef::Role.load("proxy-sensor")
    # @proxy_sensor_role.override_attributes['redborder']['cli_banner'] = cli_banner
    # @proxy_sensor_role.save

    @sensor_role = Chef::Role.load("sensor")
    @sensor_role.override_attributes['redborder']['cli_banner'] = cli_banner

    audit_messages.push("CLI Banner changed to #{settings[:cli_banner]}") if @manager_role.override_attributes['redborder']['cli_banner'] and @manager_role.override_attributes['redborder']['cli_banner'] != cli_banner
    @manager_role.override_attributes['redborder']['cli_banner'] = cli_banner

    # Update geoip license setting
    geoip_user = settings[:geoip_user] || ''
    geoip_key = settings[:geoip_key] || ''

    audit_messages.push("GeoIP license key changed to #{settings[:geoip_key]}") if @manager_role.override_attributes['redborder']['geoip_key'] and @manager_role.override_attributes['redborder']['geoip_key'] != geoip_key
    @manager_role.override_attributes['redborder']['geoip_user'] = geoip_user
    @manager_role.override_attributes['redborder']['geoip_key'] = geoip_key

    @sensor_role.override_attributes['redborder']['geoip_user'] = geoip_user
    @sensor_role.override_attributes['redborder']['geoip_key'] = geoip_key
    @sensor_role.save

    @proxy_sensor_role = Chef::Role.load("proxy-sensor")
    @proxy_sensor_role.override_attributes['redborder']['geoip_user'] = geoip_user
    @proxy_sensor_role.override_attributes['redborder']['geoip_key'] = geoip_key
    @proxy_sensor_role.save

    # Update logstash settings
    audit_messages.push("Logstash Traffic MacScrambling setting changed") if @manager_role.override_attributes['redborder']['logstash']['traffic']['macscrambling'] != params['settings']['role']['redborder']['logstash']['traffic']['macscrambling']
    @manager_role.override_attributes['redborder']['logstash']['traffic']['macscrambling'] = params['settings']['role']['redborder']['logstash']['traffic']['macscrambling']

    audit_messages.push("Logstash Traffic GEOIP setting changed") if @manager_role.override_attributes['redborder']['logstash']['traffic']['geoip'] != params['settings']['role']['redborder']['logstash']['traffic']['geoip']
    @manager_role.override_attributes['redborder']['logstash']['traffic']['geoip'] = params['settings']['role']['redborder']['logstash']['traffic']['geoip']

    audit_messages.push("Logstash Traffic MacVendor setting changed") if @manager_role.override_attributes['redborder']['logstash']['traffic']['macvendor'] != params['settings']['role']['redborder']['logstash']['traffic']['macvendor']
    @manager_role.override_attributes['redborder']['logstash']['traffic']['macvendor'] = params['settings']['role']['redborder']['logstash']['traffic']['macvendor']

    audit_messages.push("Logstash Traffic DarkList setting changed") if @manager_role.override_attributes['redborder']['logstash']['traffic']['darklist'] != params['settings']['role']['redborder']['logstash']['traffic']['darklist']
    @manager_role.override_attributes['redborder']['logstash']['traffic']['darklist'] = params['settings']['role']['redborder']['logstash']['traffic']['darklist']

    audit_messages.push("Logstash Traffic Multienrichment setting changed") if @manager_role.override_attributes['redborder']['logstash']['traffic']['multienrichment'] != params['settings']['role']['redborder']['logstash']['traffic']['multienrichment']
    @manager_role.override_attributes['redborder']['logstash']['traffic']['multienrichment'] = params['settings']['role']['redborder']['logstash']['traffic']['multienrichment']

    audit_messages.push("Logstash Intrusion Multienrichment setting changed") if @manager_role.override_attributes['redborder']['logstash']['intrusion']['multienrichment'] != params['settings']['role']['redborder']['logstash']['intrusion']['multienrichment']
    @manager_role.override_attributes['redborder']['logstash']['intrusion']['multienrichment'] = params['settings']['role']['redborder']['logstash']['intrusion']['multienrichment']

    audit_messages.push("Logstash INtrusion GEOIP setting changed") if @manager_role.override_attributes['redborder']['logstash']['intrusion']['geoip'] != params['settings']['role']['redborder']['logstash']['intrusion']['geoip']
    @manager_role.override_attributes['redborder']['logstash']['intrusion']['geoip'] = params['settings']['role']['redborder']['logstash']['intrusion']['geoip']

    audit_messages.push("Logstash Intrusion MacVendor setting changed") if @manager_role.override_attributes['redborder']['logstash']['intrusion']['macvendor'] != params['settings']['role']['redborder']['logstash']['intrusion']['macvendor']
    @manager_role.override_attributes['redborder']['logstash']['intrusion']['macvendor'] = params['settings']['role']['redborder']['logstash']['intrusion']['macvendor']

    audit_messages.push("Logstash Intrusion Darklist setting changed") if @manager_role.override_attributes['redborder']['logstash']['intrusion']['darklist'] != params['settings']['role']['redborder']['logstash']['intrusion']['darklist']
    @manager_role.override_attributes['redborder']['logstash']['intrusion']['darklist'] = params['settings']['role']['redborder']['logstash']['intrusion']['darklist']

    audit_messages.push("Logstash Location MacScrambling setting changed") if @manager_role.override_attributes['redborder']['logstash']['location']['macscrambling'] != params['settings']['role']['redborder']['logstash']['location']['macscrambling']
    @manager_role.override_attributes['redborder']['logstash']['location']['macscrambling'] = params['settings']['role']['redborder']['logstash']['location']['macscrambling']

    audit_messages.push("Logstash Location MacVendor setting changed") if @manager_role.override_attributes['redborder']['logstash']['location']['macvendor'] != params['settings']['role']['redborder']['logstash']['location']['macvendor']
    @manager_role.override_attributes['redborder']['logstash']['location']['macvendor'] = params['settings']['role']['redborder']['logstash']['location']['macvendor']

    @manager_role.save

    # TODO: Check if we ever enter here
    if params[:role]
      params[:role]['redborder']['rsyslog'] ||= {}

      params[:role]['redborder']['rsyslog']['mode'] =
        params['settings']['role']['redborder']['rsyslog']['mode']

      params[:role]['redborder']['rsyslog']['servers'] =
        params['settings']['role']['redborder']['rsyslog']['servers']

      params[:role]['redborder']['rsyslog']['protocol'] =
        params['settings']['role']['redborder']['rsyslog']['protocol']

      params[:role]['redborder']['rsyslog']['is_server'] = false

      if settings[:proxy] == '0'
        params[:role]['redborder']['proxy'] = {
          'enabled' => false
        }
      else
        params[:role]['redborder']['proxy'] = {
          'enabled' => true,
          'address' => settings[:proxy_address],
          'port' => settings[:proxy_port],
          'user' => settings[:proxy_user],
          'password' => settings[:proxy_password]
        }
      end
      # Save role on root
      Sensor.root.update_chef_role(params[:role])
    end

    DittoCode::Exec.if 'ENTERPRISE' do
      segments_s3 = settings.delete(:segments_s3)
      amazon_cloudwatch = settings.delete(:amazon_cloudwatch)

      # Save backup segments in databag
      begin
        @segments_s3 = Chef::DataBagItem.load('passwords', 's3')

        audit_messages.push("Backup Hostname setting changed to #{segments_s3['s3_host']}") if @segments_s3['s3_host'] != segments_s3['s3_host']
        @segments_s3['s3_host'] = segments_s3['s3_host']

        audit_messages.push("Backup Access setting changed") if @segments_s3['s3_access_key_id'] != segments_s3['s3_access_key_id']
        @segments_s3['s3_access_key_id'] = segments_s3['s3_access_key_id']

        audit_messages.push("Backup Secret setting changed") if @segments_s3['s3_secret_key_id'] != segments_s3['s3_secret_key_id']
        @segments_s3['s3_secret_key_id'] = segments_s3['s3_secret_key_id']

        audit_messages.push("Backup Bucket setting changed to #{segments_s3['s3_bucket']}") if @segments_s3['s3_bucket'] != segments_s3['s3_bucket']
        @segments_s3['s3_bucket'] = segments_s3['s3_bucket']

        @segments_s3.save
      rescue StandardError => e
        Rails.logger.error 'Error when update Backup Segments S3 '\
                           'setting values'
        Rails.logger.error e
        @segments_s3 = {}
      end

      # Save amazon cloudwatch
      begin
        @amazon_cloudwatch = Chef::DataBagItem.load('passwords', 'cloudwatch')

        audit_messages.push("Amazon Cloudwatch Access Key setting changed") if @amazon_cloudwatch['AWS_ACCESS_KEY'] != amazon_cloudwatch['AWS_ACCESS_KEY']
        @amazon_cloudwatch['AWS_ACCESS_KEY'] = amazon_cloudwatch['AWS_ACCESS_KEY']

        audit_messages.push("Amazon Cloudwatch Secret Key setting changed") if @amazon_cloudwatch['AWS_SECRET_KEY'] != amazon_cloudwatch['AWS_SECRET_KEY']
        @amazon_cloudwatch['AWS_SECRET_KEY'] = amazon_cloudwatch['AWS_SECRET_KEY']

        audit_messages.push("Amazon Cloudwatch Region setting changed") if @amazon_cloudwatch['REGION'] != amazon_cloudwatch['REGION']
        @amazon_cloudwatch['REGION'] = amazon_cloudwatch['REGION']

        audit_messages.push("Amazon Cloudwatch Logwatch setting changed") if @amazon_cloudwatch['LOGWATCHG'] != amazon_cloudwatch['LOGWATCHG']
        @amazon_cloudwatch['LOGWATCHG'] = amazon_cloudwatch['LOGWATCHG']

        @amazon_cloudwatch.save
      rescue StandardError => e
        Rails.logger.error 'Error when update Amazon CloudWatch '\
                           'setting values'
        Rails.logger.error e
        @amazon_cloudwatch = {}
      end

      # Save backup http certificate in encripted databag
      if settings[:http_key].present? && settings[:http_pem].present? &&
         new_certs = Setting.save_http_files(settings[:http_pem], settings[:http_key])
         audit_messages.push("HTTP SSL Certificates settings changed") if new_certs
        begin
          backup_http_crt =
            Chef::DataBagItem.load('certs', 'webui_external')
          backup_http_crt['id'] = 'webui_external'
          backup_http_crt['webui_external_crt'] = Base64.urlsafe_encode64(Setting.http_pem_content)
          backup_http_crt['webui_external_key'] = Base64.urlsafe_encode64(Setting.http_key_content)
          backup_http_crt.save
        rescue Errno::ECONNREFUSED, ArgumentError
          Rails.logger.error 'ERROR: when access databag: cannot contact erchef server'
        rescue Net::HTTPServerException
          Rails.logger.info "INFO: Databag certs webui_external doesn't exist. Creating..."
          backup_http_crt = Chef::DataBagItem.new
          backup_http_crt.data_bag 'certs'
          backup_http_crt['id'] = 'webui_external'
          backup_http_crt['webui_external_crt'] = Base64.urlsafe_encode64(Setting.http_pem_content)
          backup_http_crt['webui_external_key'] = Base64.urlsafe_encode64(Setting.http_key_content)
          if backup_http_crt.save
            Rails.logger.info 'INFO: Databag certs webui_external created.'
          else
            Rails.logger.error 'ERROR: when access databag, cannot contact erchef server'
            Rails.logger.info 'INFO: Removing databag certs webui_external'
            delete_http_file('key')
            delete_http_file('pem')
          end
        ensure
          Setting.send('http_pem_file=', nil)
          Setting.send('http_key_file=', nil)
        end

      elsif settings[:http_key].present? && settings[:http_pem].present?
        settings.except!(*[:http_pem, :http_key])
        @ssl_alert = "#{I18n.t('controllers.ssl')}"
      end
    end

    # Store DruidRules
    if params['namespace_rules']
      data = JSON.parse(params['namespace_rules']).map { |el| JSON.parse(el) }
      rules = DruidConfig::Entities::RuleCollection.new
      data.each do |el|
        rules << DruidConfig::Entities::Rule.parse(el)
      end
      Redborder::Cluster.instance.default_datasource.update_rules(rules)
    end

    update_settings(settings)

    UpdateWebpageURL.call(settings[:webpage_url])

    # Run Chef if it's not running now
    unless DelayedJob.rb_wakeup_chef_cluster?
      Delayed::Job.enqueue(Redborder::Jobs::RbWakeupChefClusterJob
        .new(true, current_user.id), priority: 5)
    end

    # Delete Stored jobs if they are not running now or if there is and update in days to keep
    unless DelayedJob.delete_stored_jobs?
      Delayed::Job.enqueue(Redborder::Jobs::DeleteStoredJobs
          .new(current_user.id, true), priority: 10, run_at: 1.day.from_now.beginning_of_day)
    end
    
    # Audit changes -> Make really slow the update of General Settings
    #audit_messages.each do |message|
    #   create_audit_with_message(message)
    #end

    if @ssl_alert
      redirect_to settings_path, alert: @ssl_alert
    else
      redirect_to settings_path, notice: "#{I18n.t('controllers.settings_updated')}"
    end
  end

  def update_initial_settings
    settings = (params[:settings] || {}).dup.symbolize_keys

    update_settings(settings)
    UpdateWebpageURL.call(settings[:webpage_url])

    session[:shown_first_login] = true
    flash[:notice] = "#{I18n.t('controllers.settings_updated')}"
  rescue Exception => exception
    Rails.logger.error exception.inspect
    Rails.logger.error exception.backtrace.join("\n")
    flash[:error] = "#{I18n.t('controllers.problem_information')}" \
                    "#{I18n.t('controllers.contact')}"
  end

  def new_trap_server
    render layout: false
  end

  def new_radius_server
    render layout: false
  end

  def new_radius_authentication_server
    render layout: false
  end

  def new_white_network
    render layout: false
  end

  def new_black_network
    render layout: false
  end

  def create_trap_server
    @manager_role = Chef::Role.manager
    trap_server = {
      'ip' => params[:trap][:ip],
      'community' => params[:trap][:community]
    }

    @found = false

    @manager_role.override_attributes['redborder']['snmp']['trap_servers'].to_a.each do |server|
      @found = (server['ip'] == params[:trap][:ip])
      break if @found
    end

    unless @found
      @manager_role.override_attributes['redborder']['snmp']['trap_servers'] ||= []
      @manager_role.override_attributes['redborder']['snmp']['trap_servers'] << trap_server
      if @manager_role.save
        create_audit_with_message("Trap Server added (#{params[:trap][:ip].to_s}) on General Settings")
      end
    end

    respond_to :js
  end

  def create_radius_server
    @manager_role = Chef::Role.manager
    radius_server = {
      'host' => params[:radius][:host],
      'shared_secret' => params[:radius][:shared_secret]
    }

    @error = []

    radius_server.each do |k, v|
      (@error << k) if v.blank?
    end

    @manager_role.override_attributes['redborder']['radius_servers'].to_a.each do |server|
      (@error << 'host') if server['host'] == params[:radius][:host]
      break if @error
    end

    if @error.empty?
      @manager_role.override_attributes['redborder']['radius_servers'] ||= []
      @manager_role.override_attributes['redborder']['radius_servers'] << radius_server
      if @manager_role.save
        create_audit_with_message("Radius server added (#{params[:radius][:host].to_s}) on General Settings")
      end
    end

    respond_to :js
  end

  def redirect_to_domain_edit
    redirect_to edit_sensor_path(current_user.sensor) unless current_user.super_reader?
  end

  def create_radius_authentication_server
    @manager_role = Chef::Role.manager
    radius_authentication_server = {
      'src_host' => params[:radius_authentication][:src_host],
      'src_shared_secret' => params[:radius_authentication][:src_shared_secret],
      'dst_host' => params[:radius_authentication][:dst_host],
      'dst_shared_secret' => params[:radius_authentication][:dst_shared_secret]
    }

    @error = []

    radius_authentication_server.each do |k, v|
      (@error << k) if v.blank?
    end

    @manager_role.override_attributes['redborder']['radius_authentication_servers'].to_a.each do |server|
      if server['src_host'] == params[:radius_authentication][:src_host]
        @error << 'src_host' && break
      end
    end

    @error.flatten!

    if @error.empty?
      @manager_role.override_attributes['redborder']['radius_authentication_servers'] ||= []
      @manager_role.override_attributes['redborder']['radius_authentication_servers'] << radius_authentication_server
      if @manager_role.save
        create_audit_with_message("Radius authentication server added (#{params[:radius_authentication][:src_host].to_s}) on General Settings")
      end
    end

    respond_to :js
  end

  def create_white_network
    @manager_role = Chef::Role.manager
    white_network = {
      'network' => NetAddr::CIDR.create(params[:white_network][:network]).to_s
    }

    @found = false

    @manager_role.override_attributes['redborder']['white_networks'].to_a.each do |server|
      @found = (server['network'] == NetAddr::CIDR.create(params[:white_network][:network]).to_s)
      break if @found
    end

    unless @found
      @manager_role.override_attributes['redborder']['white_networks'] ||= []
      @manager_role.override_attributes['redborder']['white_networks'] << white_network
      if @manager_role.save
        create_audit_with_message("White Network added (#{params[:white_network][:network].to_s}) on General Settings")
      end
    end

    respond_to :js
  end

  def create_black_network
    @manager_role = Chef::Role.manager
    black_network = {
      'network' => NetAddr::CIDR.create(params[:black_network][:network]).to_s
    }

    @found = false

    @manager_role.override_attributes['redborder']['black_networks'].to_a.each do |server|
      @found = (server['network'] == NetAddr::CIDR.create(params[:black_network][:network]).to_s)
      break if @found
    end

    unless @found
      @manager_role.override_attributes['redborder']['black_networks'] ||= []
      @manager_role.override_attributes['redborder']['black_networks'] << black_network
      if @manager_role.save
        create_audit_with_message("Black Network added (#{params[:black_network][:network].to_s}) on General Settings")
      end
    end

    respond_to :js
  end

  def delete_trap_server
    @manager_role = Chef::Role.manager
    if @manager_role.override_attributes['redborder']['snmp']['trap_servers'].count > params[:id].to_i
      server_to_delete = @manager_role.override_attributes['redborder']['snmp']['trap_servers'][params[:id].to_i]['ip'].to_s
    end
    @manager_role.override_attributes['redborder']['snmp']['trap_servers'].delete_at(params[:id].to_i)
    if @manager_role.save
      create_audit_with_message("Trap server deleted (#{server_to_delete}) with id #{params[:id]} on General Settings")
    end
    respond_to :js
  end

  def delete_radius_server
    @manager_role = Chef::Role.manager
    if @manager_role.override_attributes['redborder']['radius_servers'].count > params[:id].to_i
      server_to_delete = @manager_role.override_attributes['redborder']['radius_servers'][params[:id].to_i]['host'].to_s
    end
    @manager_role.override_attributes['redborder']['radius_servers'].delete_at(params[:id].to_i)
    if @manager_role.save
      create_audit_with_message("Radius server deleted (#{server_to_delete}) with id #{params[:id]} on General Settings")
    end
    respond_to :js
  end

  def delete_radius_authentication_server
    @manager_role = Chef::Role.manager
    if @manager_role.override_attributes['redborder']['radius_authentication_servers'].count > params[:id].to_i
      server_to_delete = @manager_role.override_attributes['redborder']['radius_authentication_servers'][params[:id].to_i]['src_host'].to_s
    end
    @manager_role.override_attributes['redborder']['radius_authentication_servers'].delete_at(params[:id].to_i)
    if @manager_role.save
      create_audit_with_message("Radius authentication server deleted (#{server_to_delete}) with id #{params[:id]} on General Settings")
    end
    respond_to :js
  end

  def delete_white_network
    @manager_role = Chef::Role.manager
    if @manager_role.override_attributes['redborder']['white_networks'].count > params[:id].to_i
      network_to_delete = @manager_role.override_attributes['redborder']['white_networks'][params[:id].to_i]['network'].to_s
    end
    @manager_role.override_attributes['redborder']['white_networks'].delete_at(params[:id].to_i)
    if @manager_role.save
      create_audit_with_message("White Network deleted (#{network_to_delete}) with id (#{params[:id].to_s}) on General Settings")
    end
    respond_to :js
  end

  def delete_black_network
    @manager_role = Chef::Role.manager
    if @manager_role.override_attributes['redborder']['black_networks'].count > params[:id].to_i
      network_to_delete = @manager_role.override_attributes['redborder']['black_networks'][params[:id].to_i ]['network'].to_s
    end
    @manager_role.override_attributes['redborder']['black_networks'].delete_at(params[:id].to_i)
    if @manager_role.save
      create_audit_with_message("Black Network deleted (#{network_to_delete}) with id (#{params[:id].to_s}) on General Settings")
    end
    respond_to :js
  end

  def new_manager_route
    render layout: false
  end

  def create_manager_route
    @manager_role = Chef::Role.manager
    @error = []

    # Is the route valid?
    begin
      network =
        IPAddr.new(
          NetAddr::CIDR.create(params[:manager_route][:network]).to_s,
          Socket::AF_INET
        )
      network =
        NetAddr::CIDR.create(
          params[:manager_route][:network]
        ).to_s
    rescue
      @error << "#{I18n.t('controllers.invalid_network')}"
    end

    begin
      gateway = IPAddr.new(params[:manager_route][:gateway], Socket::AF_INET)
    rescue
      @error << "#{I18n.t('controllers.invalid_gateway')}"
    end

    # Create network hash if it dont exist
    if @manager_role.override_attributes['redborder']['network'].nil?
      @manager_role.override_attributes['redborder']['network'] = {}
    end

    # Create routes hash if it dont exist
    if @manager_role.override_attributes['redborder']['network']['routes'].nil?
      @manager_role.override_attributes['redborder']['network']['routes'] = {}
    end

    # Save the route
    if @error.empty?
      @manager_role.override_attributes['redborder']['network']['routes'][network.to_s] = gateway.to_s
      @manager_role.override_attributes['redborder']['network']['configure_routes'] = true
    end

    if @manager_role.save
      create_audit_with_message("Network Route added (#{network.to_s}) on General Settings")
    end

    respond_to :js
  end

  def delete_manager_route
    @manager_role = Chef::Role.manager

    aux = @manager_role.override_attributes['redborder']['network']['routes'].to_a
    if aux.count > params[:id].to_i
      route_to_delete = aux[params[:id].to_i].to_s
    end
    aux.delete_at(params[:id].to_i)
    @manager_role.override_attributes['redborder']['network']['routes'] = Hash[aux]

    if @manager_role.save
      create_audit_with_message("Network Route deleted (#{route_to_delete}) on General Settings")
    end
    respond_to :js
  end

  def new_fixed_host
    render layout: false
  end

  def create_fixed_host
    return false unless (params[:fixed_host][:name].present? && params[:fixed_host][:ip].present?)

    @fixed_host = params[:fixed_host]
    @error = []

    begin
      data = Chef::DataBagItem.load('rBglobal', 'hosts')

      if data['hosts'][@fixed_host[:ip]].present?
        @error << "#{I18n.t('controllers.host_error')}"
      else
        data['hosts'][@fixed_host[:ip]] = @fixed_host[:name]
        if data.save
          create_audit_with_message("Fix host added (#{params[:fixed_host].to_s}) on General Settings")
        end
      end

    rescue Errno::ECONNREFUSED, ArgumentError
      Rails.logger.error 'ERROR: when access databag: cannot contact erchef server'
      @error << "#{I18n.t('controllers.databag_error')}"
    rescue Net::HTTPServerException
      Rails.logger.info "INFO: Databag rBglobal hosts doesn't exist. Creating..."
      data = Chef::DataBagItem.new
      data.data_bag 'rBglobal'
      data['id'] = 'hosts'
      data['description'] = 'available hosts'
      data['hosts'] = {}
      data['hosts'][@fixed_host[:ip]] = @fixed_host[:name]
      if data.save
        Rails.logger.info 'INFO: Databag rBglobal hosts created.'
      else
        Rails.logger.error 'ERROR: when access databag, cannot contact erchef server'
        @error << "#{I18n.t('controllers.databag_error')}"
      end
    ensure
      @fixed_hosts = data['hosts']
      respond_to :js
    end
  end

  def delete_fixed_host
    data = Chef::DataBagItem.load('rBglobal', 'hosts')
    data['hosts'].delete(params[:ip])
    if data.save
      create_audit_with_message("Fix host deleted (#{params[:ip].to_s}) on General Settings")
    end
    @fixed_hosts = data['hosts']
    rescue Errno::ECONNREFUSED, Net::HTTPServerException
      Rails.logger.error 'ERROR: when access databag: cannot contact erchef server'
      @error = "#{I18n.t('controllers.databag_error')}"
    ensure
      respond_to :js
  end

  def rsa
    rsa_setting = Manager.all.first
    out = `sudo /bin/env BOOTUP=none /usr/lib/redborder/bin/rb_create_rsa.sh -f` if File.exist?('/usr/lib/redborder/bin/rb_create_rsa.sh')

    if out.empty?
      redirect_to settings_path, alert: "#{I18n.t('controllers.rsa')}"
    else
      rsa_setting.audit_rsa_change if rsa_setting
      redirect_to settings_path,
                  notice: "#{I18n.t('controllers.rsa_success')}"
    end
  end

  def upload_geoip_country_file
    @type = 'country'
    @name = params[:geoip_country_file].original_filename
    if params[:geoip_country_file].content_type != 'application/octet-stream' || !(@name =~ /^.\.dat$/i)
      @error = "#{I18n.t('controllers.dat_file')}"
      render 'upload_geoip_file'
      return
    end

    path_tmp = "#{Rails.root}/tmp/GeoIP-tmp.dat"
    File.open(path_tmp, 'wb') { |f| f.write(params[:geoip_country_file].read) }

    unless Redborder::Geoip.check_database(path_tmp, 'country')
      @error = "#{I18n.t('controllers.invalid_geoip')}"
      render 'upload_geoip_file' && return
    end

    Setting.geoip_country_file = path_tmp
    Setting.geoip_country_custom_file = @name
    create_audit_with_message("GeoIP Country file changed on General Settings")
    render 'upload_geoip_file'
  end

  def upload_geoip_countryv6_file
    @type = 'countryv6'
    @name = params[:geoip_countryv6_file].original_filename
    if params[:geoip_countryv6_file].content_type != 'application/octet-stream' &&
       !(@name =~ /^.\.dat$/i)
      @error = "#{I18n.t('controllers.dat_file')}"
      render 'upload_geoip_file'
      return
    end

    path_tmp = "#{Rails.root}/tmp/GeoIPv6-tmp.dat"

    File.open(path_tmp, 'wb') do |f|
      f.write(params[:geoip_countryv6_file].read)
    end

    unless Redborder::Geoip.check_database(path_tmp, 'countryv6')
      @error = "#{I18n.t('controllers.invalid_geoipv6')}"
      render 'upload_geoip_file' && return
    end

    Setting.geoip_countryv6_file = path_tmp
    Setting.geoip_countryv6_custom_file = @name
    create_audit_with_message("GeoIP Country IPv6 file changed on General Settings")
    render 'upload_geoip_file'
  end

  def upload_geoip_city_file
    @type = 'city'
    @name = params[:geoip_city_file].original_filename
    if params[:geoip_city_file].content_type != 'application/octet-stream' &&
       !(@name =~ /^.\.dat$/i)
      @error = "#{I18n.t('controllers.dat_file')}"
      render 'upload_geoip_file'
      return
    end

    path_tmp = "#{Rails.root}/tmp/GeoLiteCity-tmp.dat"
    File.open(path_tmp, 'wb') { |f| f.write(params[:geoip_city_file].read) }

    unless Redborder::Geoip.check_database(path_tmp, 'city')
      @error = "#{I18n.t('controllers.invalid_geoip')}"
      render 'upload_geoip_file' && return
    end

    Setting.geoip_city_file = path_tmp
    Setting.geoip_city_custom_file = @name
    create_audit_with_message("GeoIP City file changed on General Settings")
    render 'upload_geoip_file'
  end

  def upload_geoip_cityv6_file
    @type = 'cityv6'
    @name = params[:geoip_cityv6_file].original_filename
    if params[:geoip_cityv6_file].content_type != 'application/octet-stream' &&
       !(@name =~ /^.\.dat$/i)
      @error = "#{I18n.t('controllers.dat_file')}"
      render 'upload_geoip_file'
      return
    end

    path_tmp = "#{Rails.root}/tmp/GeoLiteCityv6-tmp.dat"
    File.open(path_tmp, 'wb') { |f| f.write(params[:geoip_cityv6_file].read) }

    unless Redborder::Geoip.check_database(path_tmp, 'cityv6')
      @error = "#{I18n.t('controllers.invalid_geoipv6')}"
      render 'upload_geoip_file' && return
    end

    Setting.geoip_cityv6_file = path_tmp
    Setting.geoip_cityv6_custom_file = @name
    create_audit_with_message("GeoIP City IPv6 file changed on General Settings")
    render 'upload_geoip_file'
  end

  def upload_geoip_asn_file
    @type = 'asn'
    @name = params[:geoip_asn_file].original_filename
    if params[:geoip_asn_file].content_type != 'application/octet-stream' &&
       !(@name =~ /^.\.dat$/i)
      @error = "#{I18n.t('controllers.dat_file')}"
      render 'upload_geoip_file'
      return
    end

    path_tmp = "#{Rails.root}/tmp/GeoIPASNum-tmp.dat"
    File.open(path_tmp, 'wb') { |f| f.write(params[:geoip_asn_file].read) }

    unless Redborder::Geoip.check_database(path_tmp, 'asn')
      @error = "#{I18n.t('controllers.invalid_geoip')}"
      render 'upload_geoip_file' && return
    end

    Setting.geoip_asn_file = path_tmp
    Setting.geoip_asn_custom_file = @name
    create_audit_with_message("GeoIP ASN file changed on General Settings")
    render 'upload_geoip_file'
  end

  def upload_geoip_asnv6_file
    @type = 'asnv6'
    @name = params[:geoip_asnv6_file].original_filename
    if params[:geoip_asnv6_file].content_type != 'application/octet-stream' &&
       !(@name =~ /^.\.dat$/i)
      @error = "#{I18n.t('controllers.dat_file')}"
      render 'upload_geoip_file'
      return
    end

    path_tmp = "#{Rails.root}/tmp/GeoIPASNumv6-tmp.dat"
    File.open(path_tmp, 'wb') { |f| f.write(params[:geoip_asnv6_file].read) }

    unless Redborder::Geoip.check_database(path_tmp, 'asnv6')
      @error = "#{I18n.t('controllers.invalid_geoipv6')}"
      render 'upload_geoip_file' && return
    end

    Setting.geoip_asnv6_file = path_tmp
    Setting.geoip_asnv6_custom_file = @name
    create_audit_with_message("GeoIP ASN IPv6 file changed on General Settings")
    render 'upload_geoip_file'
  end

DittoCode::Exec.if 'ENTERPRISE' do
  def upload_http_key_file
    @type = 'key'
    @name = params[:http_key_file].original_filename
    if params[:http_key_file].content_type != 'application/octet-stream' &&
       !(@name =~ /^.*\.key$/i)
      @error = "#{I18n.t('controllers.key_file')}"
      render 'upload_http_file'
      return
    end

    Setting.save_http_key_tmp params[:http_key_file]
    render 'upload_http_file'
  end

  def upload_http_pem_file
    @type = 'pem'
    @name = params[:http_pem_file].original_filename
    if params[:http_pem_file].content_type != 'application/pkix-cert' &&
       !(@name =~ /^.*\.crt$/i)
      @error = "#{I18n.t('controllers.crt_file')}"
      render 'upload_http_file'
      return
    end

    Setting.save_http_pem_tmp params[:http_pem_file]
    render 'upload_http_file'
  end

  def delete_http_files
    %w(key pem).each do |type|
      delete_http_file(type)
    end
    create_audit_with_message("Deleted HTTP SSL Certificates")
    File.delete("#{Rails.root}/tmp/HTTPKey-tmp.key") if File.exist?("#{Rails.root}/tmp/HTTPKey-tmp.key")
    File.delete("#{Rails.root}/tmp/HTTPPem-tmp.crt") if File.exist?("#{Rails.root}/tmp/HTTPPem-tmp.crt")
  end
end

  def upload_sp_sso_cert_file
    @name = params[:sp_sso_cert].original_filename
    if params[:sp_sso_cert].content_type != 'application/pkix-cert' && !(@name =~ /^.*\.(crt)|(pub)$/i)
      @error = "#{I18n.t('controllers.crt_file')}"
      render 'upload_http_file'
      return
    end
    Setting.save_sp_sso_cert_tmp params[:sp_sso_cert]
    render 'upload_http_file'
  end

  def delete_sp_sso_cert_file
    file_path = "#{Rails.root}/tmp/sp-sso-tmp.crt"
    File.delete(file_path) if File.exist?(file_path)
  end

  def upload_sp_sso_priv_key_file
    @name = params[:sp_sso_priv_key].original_filename
    if params[:sp_sso_priv_key].content_type != 'application/pkix-cert' && !(@name =~ /^.*\.key$/i)
      @error = "#{I18n.t('controllers.crt_file')}"
      render 'upload_http_file'
      return
    end
    Setting.save_sp_sso_priv_key_tmp params[:sp_sso_priv_key]
    render 'upload_http_file'
  end

  def delete_sp_sso_priv_key_file
    file_path = "#{Rails.root}/tmp/sp-sso-tmp.key"
    File.delete(file_path) if File.exist?(file_path)
  end

  def delete_geoip_file
    if %w(country countryv6 city cityv6 asn asnv6).include?(params[:type])
      Setting.send("geoip_#{params[:type]}_custom=", 0)
      create_audit_with_message("GeoIP deleting file (#{params[:type].to_s}) on General Settings")
      execute_geoip_update
    else
      render js: '', status: :forbidden && return
    end
  end

  def download_ai_model
    model = Setting.find_by(name: 'ai_selected_model').value
    Delayed::Job.enqueue(Redborder::Jobs::DownloadAIModelJob.new(true, model, User.current_user.id),
                         priority: 5, run_at: Time.now)
    redirect_to settings_path, notice: "#{I18n.t('settings.ai_assistant.start_downloading')}"
  end

  private

  #
  # Load an Encrypted Databag. If it doesn't exist, I wil be created.
  #
  # == Parameters:
  # db_name::
  #   String with name of store
  # databag::
  #   String with name of dataBag
  #
  # == Returns:
  # DataBag item or nil
  #
  def load_encrypted_databag(db_name, databag)
    # Try to load it
    return Chef::EncryptedDataBagItem.load(db_name, databag)
  rescue ArgumentError
    Rails.logger.error 'No secret found at /etc/chef/encrypted_data_bag_secret'
    nil
  rescue Errno::ECONNREFUSED
    Rails.logger.error 'ERROR: when access databag: cannot contact erchef server'
    nil
  rescue Net::HTTPServerException
    Rails.logger.info "INFO: Databag #{db_name} #{databag} doesn't exist. Creating..."
    data = Chef::DataBagItem.new
    data.data_bag db_name
    data['id'] = databag
    data['created'] = false
    if data.save
      Rails.logger.info "INFO: Databag #{db_name} #{databag} created."
      data
    else
      Rails.logger.error 'ERROR: when access databag, cannot contact erchef server'
      nil
    end
  end

  #
  # Load a Databag. If it doesn't exist, I wil be created.
  #
  # == Parameters:
  # db_name::
  #   String with name of store
  # databag::
  #   String with name of dataBag
  #
  # == Returns:
  # DataBag item or nil
  #
  def load_databag(db_name, databag)
    # Try to load it
    return Chef::DataBagItem.load(db_name, databag)
  rescue Errno::ECONNREFUSED, ArgumentError
    Rails.logger.error 'ERROR: when access databag: cannot contact erchef server.'
    nil
  rescue Net::HTTPServerException
    Rails.logger.info "INFO: Databag #{db_name} #{databag} doesn't exist. Creating..."
    data = Chef::DataBagItem.new
    data.data_bag db_name
    data['id'] = databag
    data['created'] = false
    if data.save
      Rails.logger.info "INFO: Databag #{db_name} #{databag} created."
      data
    else
      Rails.logger.error 'ERROR: when access databag, cannot contact erchef server.'
      nil
    end
  end

  def execute_geoip_update
    if DelayedJob.geoip_update? && !DelayedJob.geoip_update.running?
      DelayedJob.geoip_update.destroy!
      Delayed::Job.enqueue(
        Redborder::Jobs::GeoipUpdateJob.new(User.current_user.id, true),
        priority: 10, run_at: Time.now + 1.minute
      )
    elsif !DelayedJob.geoip_update?
      Delayed::Job.enqueue(
        Redborder::Jobs::GeoipUpdateJob.new(User.current_user.id, true),
        priority: 10, run_at: Time.now + 1.minute
      )
    end
  end

  def delete_http_file(type)
    Setting.send("http_#{type}=", nil)
    Setting.send("http_#{type}_file=", nil)

    data = Chef::DataBagItem.load('certs', 'webui_external')
    data.raw_data = {
      "id" => "webui_external"
    }
    data.save
  rescue Errno::ECONNREFUSED, Net::HTTPServerException
    Rails.logger.error 'ERROR: when access databag: cannot contact erchef server'
  end

  def update_settings(settings)
    # Save the current settings
    settings_to_save = []
    sql_values = []

    # Save settings
    settings.each do |name, value|
      # remove blank values in array settings
      value.delete_if(&:blank?) if value.is_a?(Array)
      sql_values << '(?, ?)'
      settings_to_save << "#{name}"
      settings_to_save << "#{value}"
    end

    # Audit the ones that changes
    actual_settings = []
    Setting.all.each do |s|
      actual_settings << s.name
      actual_settings << s.value
    end

    changed_settings = Hash[*settings_to_save].to_a - Hash[*actual_settings].to_a

    # Audit changes -> Make really slow the update of General Settings
    #changed_settings.to_h.each do |k,v|
    #  create_audit_with_message("#{k} setting changed to #{v}")
    #end

    # This action is only available to PostgreSQL
    sql_query = 'UPDATE settings AS s SET value = c.value FROM'\
                "(VALUES #{sql_values.join(', ')}) AS "\
                'c(name, value) WHERE c.name = s.name;'
    sql_query = ActiveRecord::Base.send(
                  :sanitize_sql_array, [sql_query] + settings_to_save)
    ActiveRecord::Base.connection.execute(sql_query)
  end

  def create_audit_with_message(message)
    audit_model = Manager.all.first
    audit_model.audit_setting_change(message)
  end

  def saml_cert_content(filename, key, alternative_key)
    cert = nil
    if File.exist?(filename) && !File.zero?(filename)
      # Read the file and saves the new cert in data bag
      cert_content = File.read(filename)
      encoded_cert_content = Base64.urlsafe_encode64(cert_content)
      cert = cert_content
      begin
        data_bag = Chef::DataBagItem.load('certs', 'saml')
        data_bag[key] = encoded_cert_content
        data_bag.save
      rescue Errno::ECONNREFUSED, ArgumentError
        Rails.logger.error 'ERROR: when access databag: cannot contact erchef server'
      rescue Net::HTTPServerException
        Rails.logger.info "INFO: Databag certs saml doesn't exist. Creating..."
        data_bag = Chef::DataBagItem.new
        data_bag.data_bag 'certs'
        data_bag['id'] = 'saml'
        data_bag[key] = encoded_cert_content
        if data_bag.save
          Rails.data_bag.info 'INFO: Databag certs saml created.'
        else
          Rails.logger.error 'ERROR: when access databag, cannot contact erchef server'
          raise Net::HTTPFatalError.new('ERROR: when access databag, cannot contact erchef server', nil)
        end
      end
    else
      begin
        data_bag = Chef::DataBagItem.load('certs', 'saml')
        if data_bag[key]
          data_bag.delete(key)
          data_bag.save
        end
        cert = Base64.urlsafe_decode64(data_bag[alternative_key]) if data_bag[alternative_key]
      rescue Errno::ECONNREFUSED, ArgumentError
        Rails.logger.error 'ERROR: when access databag: cannot contact erchef server'
      rescue Net::HTTPServerException
        Rails.logger.info "INFO: Databag certs saml doesn't exist."
      end
    end
    cert
  end
end
