#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module ManagersHelper
  # TODO: check number_to_human_size not working in ruby 2.7
  # def time_suffixed_to_human(str)
  #   suffix = str.slice!(-2, 2)
  #   number = str.to_i
  #   case suffix
  #   when "kB"
  #     number *= 1024
  #   when "mB"
  #     number *= (1024 * 1024)
  #   when "gB"
  #     number *= (1024 * 1024 * 1024)
  #   end
  #   number_to_human_size number
  # end

  def display_name name
    name_to_ret = name.clone
    name_to_ret.sub!("druid_", "")
    name_to_ret.sub!("riak", "s3")
    name_to_ret
  end

  def show_total_services_by_service(manager)
    n = 0
    manager.services.each{|s,v| n += 1 if Manager::DISPLAYABLE_SERVICES.include?(s) and v[:status] and v[:ok]}
    return n
  end

  def show_total_services_by_manager(managers, services)
    n = 0
    managers.each{|m| n += 1 if m.services.fetch(services, {})[:status]}
    return n
  end

  def get_consumer(info)
    if info[:storm] == "?" and info[:druid_rt] == "?"
      "?"
    elsif info[:druid_rt] != "?"
      "#{info[:druid_rt]} (realtime)"
    else
      "#{info[:storm]} (storm)"
    end
  end

  #
  # Delete the CDOMAIN of a String
  #
  def gsub_cdomain(str)
    str.gsub(".node.#{Redborder::Cluster.cdomain}", '')
  end

  def percent_used(hash)
    used_memory(hash) * 100 / hash["total"].to_i
  end

  def used_memory(hash)
    hash["total"].to_i - hash["free"].to_i - hash["cached"].to_i - hash["buffers"].to_i
  end

  def used_memory_to_human(hash)
    value = used_memory(hash)
    value.to_s + "kB"
  end

  def show_icon var, ok_msg, fail_msg
    (var == ok_msg ? "<i class='fa fa-check icon-green' title='#{ok_msg}'></i>" : "<i class='fa fa-times icon-red' title='#{fail_msg}'></i>").html_safe
  end

  def up_button var, fail_msg, node
    if var == fail_msg
      if DelayedJob.up_pcp?
        link_to "#{I18n.t('managers_helper.repair')}",  "#", class: "btn disabled"
      else
        link_to "#{I18n.t('managers_helper.repair')}", up_pcp_manager_path(node), method: :post, data: { confirm: "#{I18n.t('managers_helper.confirm')}" }, class: "btn repair"
      end
    end
  end

  def recovery_button var, fail_msg, node
    if var == fail_msg
      if DelayedJob.recovery_pcp?
        link_to "#{I18n.t('managers_helper.repair')}",  "#", class: "btn disabled"
      else
        link_to "#{I18n.t('managers_helper.repair')}", recovery_pcp_manager_path(node), method: :post, data: { confirm: "#{I18n.t('managers_helper.confirm')}" }, class: "btn repair"
      end
    end
  end

  def check_active_repair
    ("<span class='help-block'>"+ I18n.t('managers_helper.info') + "</span>").html_safe if (DelayedJob.up_pcp? or DelayedJob.recovery_pcp?)
  end

  def reassign_partitions_button
    link =  if @reasigning_partitions
              link_to "#{I18n.t('managers_helper.reassigning')}",  "#", class: "btn btn-success disabled"
            else
              link_to "#{I18n.t('managers_helper.reassigning')}",  reassign_partitions_managers_path, method: :post, data: { confirm: "#{I18n.t('managers_helper.confirm')}" }, class: "btn btn-success"
            end
    html = content_tag(:div, link, :class => "col-md-12 col-margin-ts right-text")
    content_tag(:div, html, :class => "row")
  end

  def namespace_name_by_uuid(uuid)
    Sensor.find_by_uuid(uuid).name rescue uuid
  end

end
