#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# Helps to render the redborder objects view
#
module ObjectTypesHelper
  #
  # Create a button to add a new object
  #
  # == Parameters:
  # object_class_name::
  #   Name of the object class to show the button
  #
  # == Returns:
  # A link to create an object
  #
  def new_object_type_link(object_class_name)
    object_name = object_class_name.underscore

    link_to("#{I18n.t "object_types.create.new_object_type"}",
            new_object_type_path(class: object_class_name),
            "data-toggle" => "modal",
            "data-target" => "#default_modal",
            "data-backdrop" => "static")
  end

  #
  # Create the options to edit an object
  #
  # == Parameters:
  # object::
  #   RedBorder object to get buttons
  #
  # == Returns:
  # HTML code of buttons to edit and destroy
  #

  def object_type_options(object_type)
  content = content_tag(:td, class: "last row-right") do
    edit_link = link_to_if(object_type.persisted?, content_tag(:i, nil, class: "fa fa-gear"), edit_object_type_path(object_type), class: "btn edit-object", "data-toggle" => "modal", "data-target" => "#default_modal", "data-backdrop" => "static")

    delete_link = link_to_if(object_type.persisted?, content_tag(:i, nil, class: "fa fa-trash"), object_type_path(object_type), method: :delete, data: { confirm: "#{t("confirm", default: "Are you sure?")}" }, class: "btn destroy-object", remote: true)

    edit_link + delete_link
  end

  content
end

  #
  # Return the dropdown menu for redborder objects
  #
  # == Parameters:
  # tab::
  #   Symbol of the tab
  #
  # == Returns:
  # HTML code of buttons to import and export
  #
  def object_types_dropdown_options
    controller = "object_types"

    html =
      content_tag(:li) do
        link_to "#{t "import_csv"}",
                "/#{controller}/import/",
                class: "import_object_types",
                "data-toggle" => "modal",
                "data-target" => "#default_modal",
                "data-backdrop" => "static"
      end

    html +=
      content_tag(:li) do
        link_to "#{t "export_csv"}",
                "/#{controller}/export/",
                class: "export_objects"
      end

    # Return HTML safe string
    html.html_safe
  end
end
