#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# Helps to render the redborder objects view
#
module RedborderObjectsHelper
  #
  # Create a button to add a new object
  #
  # == Parameters:
  # object_class_name::
  #   Name of the object class to show the button
  #
  # == Returns:
  # A link to create an object
  #
  def new_object_link(object_class_name)
    object_name = object_class_name.underscore

    link_to("#{I18n.t '.new2'} #{object_name.humanize}",
            new_object_path(class: object_class_name),
            class: 'btn btn-success pull-right',
            'data-toggle' => 'modal',
            'data-target' => "#default_modal",
            'data-backdrop' => 'static')
  end

  #
  # Create the tabs to navigate between objects
  #
  # == Parameters:
  # tabs::
  #   Array of symbols with tabs to show
  #
  # == Returns:
  # Safe string with HTML code
  #
  def object_tabs(tabs)
    html = ''
    tabs.each do |tab|
      html +=
        content_tag(:li, class: (session[:object] == tab ? 'active' : ''),
                         id: tab) do
          link_to(object_tab_name(tab), "##{tab}", :'data-toggle' => 'tab')
        end
    end
    # Return HTML safe string
    html.html_safe
  end

  #
  # Return the name of the tab for redborder objects
  #
  # == Parameters:
  # tab::
  #   Symbol of the tab
  #
  # == Returns:
  # String with the name
  #
  def object_tab_name(tab)
    return 'Net' if tab == :Net
    tab.to_s.upcase
  end

  #
  # Create the options to edit an object
  #
  # == Parameters:
  # object::
  #   RedBorder object to get buttons
  #
  # == Returns:
  # HTML code of buttons to edit and destroy
  #
  def object_options(object)
    content = content_tag(:td, class: 'last row-right') do
      links = link_to(content_tag(:i, nil, class: 'fa fa-gear'),
                      edit_object_path(id: object, object: { type: object.type }),
                      class: 'btn edit-object', 'data-toggle' => 'modal',
                      'data-target' => "#default_modal",
                      'data-backdrop' => 'static')

      links + link_to(content_tag(:i, nil, class: 'fa fa-trash'),
                      object_path(id: object, object: { type: object.type }), method: :delete,
                      data: { confirm: "#{t 'confirm'}" },
                      class: 'btn destroy-object', remote: true)
    end
    # Return the td
    content
  end

  #
  # Return the dropdown menu for redborder objects
  #
  # == Parameters:
  # tab::
  #   Symbol of the tab
  #
  # == Returns:
  # HTML code of buttons to import and export
  #
  def objects_dropdown_options(tab)
    controller = tab.eql?('app') ? 'application_objects' : 'objects'

    html =
      content_tag(:li) do
        link_to "#{t 'import_csv'}",
                "/#{controller}/import/#{tab}",
                class: 'import_objects',
                'data-toggle' => 'modal',
                'data-target' => '#default_modal',
                'data-backdrop' => 'static'
      end

    html +=
      content_tag(:li) do
        link_to "#{t 'export_csv'}",
                "/#{controller}/export/#{tab}",
                class: 'export_objects'
      end

    # Return HTML safe string
    html.html_safe
  end

  def ip_addresses_dropdown(ip_addresses)
    # Aseguramos que ip_addresses sea siempre un arreglo
    ips = ip_addresses.is_a?(Array) ? ip_addresses : ip_addresses.split(', ').reject(&:empty?)
    return nil if ips.empty?

    if ips.size == 1
      ips.first
    else
      content_tag(:select, class: 'ip-dropdown') do
        ips.each do |ip|
          concat(content_tag(:option, ip, value: ip))
        end
      end
    end
  end
end
