#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class ModelSource < ApplicationRecord

  include Redborder::Audit
  include StorageFile

  validates :name, uniqueness: true
  validates :name, presence: { message: "#{I18n.t('controllers.name_required')}" }

  audited only: [:name, :description, :tgzurl, :md5url, :enable],
          on: [:create, :update, :destroy]

  default_scope { order('created_at DESC') }

  storage_file :cached, path: 'rb-webui/:class/:id/:filename'

  def code=(value)
    super(value.try(:strip))
  end

  def clean_name
    name.gsub(/\s+|\.+/, "_")
  end

  def full_url
    url
  end

  def full_md5url
    md5url
  end

  def cached_uri
    return nil if cached.path.nil?
    StorageFile.get_uri cached.path
  end

  def cached_uri_path
    if StorageFile.s3_enabled?
      "s3://redborder" + cached.path
    else
      cached.path
    end
  end

  def downloaded?
    !cached.path.nil?
  end

end
