#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

require File.expand_path('../boot', __FILE__)
require 'rails/all'
require 'rexml'

# Assets should be precompiled for production (so we don't need the gems loaded then)
Bundler.require(*Rails.groups(assets: %w(development test)))

ENV['DITTOCODE_ENV'] ||= 'ENTERPRISE'
module RedBorder
  redborder_config = File.read('config/redborder_config.yml')
  CONFIG = YAML.load(redborder_config)[Rails.env].symbolize_keys
  COUNTRIES = YAML.load(File.read('config/countries.yml'))
  DRUID_CONFIG = YAML.load(File.read('config/rbdruid_config.yml'))[Rails.env].symbolize_keys
  BLOCKED = false
  VERSION = '3.1'
  ZENDESK_CONFIG = YAML.load(File.read('config/zendesk.yml'))

  USER_CONFIG = if File.exist?('config/defaultuser.yml')
                  YAML.load_file('config/defaultuser.yml')
                else
                  { 'email'    => 'admin@redborder.com',
                    'login'    => 'admin',
                    'password' => 'DefaultPassw0rd!' }
                end

  LICENSE_MAIL       = CONFIG[:license_mail] || 'sales@redborder.com'
  LICENSING_MODE     = CONFIG[:licensing_mode] || 'global'
  LICENSE_PUBLIC_KEY = File.read('config/license.key.pub')
  SSO_USER_SENSOR_MAP = YAML.load(File.read('config/sso_user_sensor_map.yml'))

  class Application < Rails::Application
    # Custom directories with classes and modules you want to be autoloadable.
    config.autoload_paths += %W(#{config.root}/lib)
    config.autoload_paths += Dir["#{config.root}/lib/**/"]
    config.eager_load_paths += %W(#{config.root}/lib/redborder)
    config.assets.paths << Rails.root.join('app', 'assets')

    modules = if File.exist? "#{Rails.root}/config/modules.yml"
                YAML.load(File.read('config/modules.yml'))['modules']
              end

    # Loads the core plugins located in lib/plugins
    unless ENV['NO_MODULES']
      Dir.glob(File.join(Rails.root, 'lib/modules/*')).sort.each do |directory|
        next unless File.directory?(directory)
        module_name = directory.split('/').last

        next if modules && !modules.include?(module_name)

        config.eager_load_paths += Dir["#{config.root}/lib/modules/#{module_name}/app/models/**/"]
        config.eager_load_paths += Dir["#{config.root}/lib/modules/#{module_name}/app/controllers/**/"]
        config.eager_load_paths += Dir["#{config.root}/lib/modules/#{module_name}/app/helpers/**/"]
        config.eager_load_paths += Dir["#{config.root}/lib/modules/#{module_name}/lib/**/"]
        config.i18n.load_path += Dir["#{config.root}/lib/modules/#{module_name}/config/locales/*.yml"]
        I18n.load_path += Dir["#{config.root}/lib/modules/#{module_name}/config/locales/*.yml"]
      end
    end

    # Settings in config/environments/* take precedence over those specified here.
    # Application configuration should go into files in config/initializers
    # -- all .rb files in that directory are automatically loaded.

    # Set Time.zone default to the specified zone and make Active Record auto-convert to this zone.
    # Run "rake -D time" for a list of tasks for finding time zone names. Default is UTC.
    # config.time_zone = 'Central Time (US & Canada)'

    # The default locale is :en and all translations from config/locales/*.rb,yml are auto loaded.
    #config.i18n.load_path += Dir[Rails.root.join('config', 'locales', '*.{rb,yml}').to_s]
    I18n.load_path += Dir[Rails.root.join('config', 'locales', '*.{rb,yml}').to_s]
    config.i18n.default_locale = :en
    config.i18n.enforce_available_locales = false
    config.i18n.available_locales = [:en, :es]

    config.action_view.field_error_proc = proc { |html_tag, _instance|
      "<div class=\"field_with_errors control-group error\">#{html_tag}</div>".html_safe
    }

    config.active_record.raise_in_transactional_callbacks = true

    config.exceptions_app = self.routes
  end
end
