#######################################################################
# Copyright (c) 2024 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

RedBorder::Application.routes.draw do

  DittoCode::Exec.if 'ENTERPRISE' do
    resources :alerts do
      get :dismiss, on: :member
      get :disable, on: :member
      get :enable, on: :member
    end
  end

  if Rails.env.development?
    mount MailPreview => 'mail_view'
  end

  root to: 'dashboards#index'

  resources :severities, only: :index

  # Change namespace
  post 'namespaces', to: 'sensors#namespaces'

  resources :products, only: :index
  get 'results', to: 'products#index', as: :results

  devise_for :users, :path_names => { :sign_in => 'login', :sign_out => 'logout', :sign_up => 'register' },
                     :controllers => { :registrations => "registrations", :sessions => 'sessions', :passwords => "passwords" }

  devise_scope :user do
    get '/users/password/reset' => 'passwords#reset', as: :reset
    put '/users/password/reset' => 'passwords#password_evaluation', as: :evaluation_password
  end
DittoCode::Exec.if 'ENTERPRISE' do

  # resources :correlation_engine_rules do
  #   post :apply, on: :collection
  # end

  # Support Center
  resources :support, only: [:index, :new, :create] do
    collection do
      get 'login', to: 'support#login'
      post 'login', to: 'support#login_to_zendesk'
      delete 'logout', to: 'support#logout'
    end
  end

  # Plugins
  resources :plugins do
    post "updating", action: :updating
    post "force_plugins_update", on: :collection
    get "check_is_applying", action: :check_is_applying
  end

  get :overview, to: 'dashboards#overview'

  resources :dashboards, path: 'dashboard' do
    get :new_sensor, on: :collection
    get :new_user, on: :collection
    get :image, on: :member
    post :set_locked_status, on: :member
    post :set_background_position, on: :member
    get :clone, on: :member
    get :export, on: :member
    get :import, on: :collection
    post :import, on: :collection, to: "dashboards#save_import"
    post :clone, action: :create_clone, on: :member
    get :to_report, on: :member
    post :to_report, on: :member, to: "dashboards#convert_to_report"
    patch :remove_image, on: :member
    get :overview, on: :collection, as: :overview

    post 'overview/store_search_type',
          on: :collection, action: :overview_store_search_type
    get 'sensors/:id/overview', on: :collection, action: :overview, as: :overview_sensor

    resources :widgets, except: [:index, :new] do
      get :list, on: :collection
      get 'graph/:product', action: :graph, as: :graph, on: :collection
      get 'types_graphs/:widget_type', action: :types_graphs, as: :types_graphs, on: :collection
      get :new_module, on: :member
      get :reload, on: :member
      get :image, on: :member
      get :clone, on: :member
      post :clone, action: :create_clone, on: :member
      post :set_absolute_position, on: :member
      get :sensors_map, on: :member
      post :set_position, on: :collection

      get 'new/:type/(:graph)/(:product)', action: :new, as: :new, on: :collection
    end
  end
end

DittoCode::Exec.if 'COMMUNITY' do

  resources :dashboards, only: [:update, :edit, :index, :show, :destroy], path: 'dashboard' do
    get :new_user, on: :collection
    get :image, on: :member
    post :set_locked_status, on: :member
    post :set_background_position, on: :member
    get :export, on: :member
    get :import, on: :collection
    post :import, on: :collection, to: "dashboards#save_import"
    patch :remove_image, on: :member
    get :to_report, on: :member
    post :to_report, on: :member, to: "dashboards#convert_to_report"
    get :overview, on: :collection, as: :overview
    get 'sensors/:id/overview', on: :collection, action: :overview,
                                as: :overview_sensor


    resources :widgets, except: [:index, :new] do
      get :list, on: :collection
      get 'graph/:product', action: :graph, as: :graph, on: :collection
      get 'types_graphs/:widget_type', action: :types_graphs, as: :types_graphs, on: :collection
      get :new_module, on: :member
      get :reload, on: :member
      get :image, on: :member
      get :clone, on: :member
      post :clone, action: :create_clone, on: :member
      post :set_absolute_position, on: :member
      get :sensors_map, on: :member
      post :set_position, on: :collection

      get 'new/:type/(:graph)/(:product)',
        action: :new, as: :new, on: :collection
    end
  end

end

  DittoCode::Exec.if 'ENTERPRISE' do
    # Work with tiers
    resources :tiers, except: [:index] do
      member do
        get :confirm_destroy
        patch :move_to_tier
      end
    end
  end

  resources :model_sources do
    get :check_model_job_status, on: :member
    post :force_update, on: :collection
  end

  resources :incidents do
    get :search, action: :search, as: :search, on: :collection
    get :new_user, on: :member
    post :assign_user, on: :member
    delete :unassign_user, on: :member
    get :search_user, action: :search_user, as: :search_user, on: :member
    get :sidebar_content, on: :member
    get :sidebar_observable_content, on: :member
    get :sidebar_linked_incidents_content, on: :member
    get :edit_description, on: :member

    # Bulk actions
    post :bulk_delete, on: :collection
    post :bulk_update_status, on: :collection
    post :bulk_create_links, on: :collection
    
    # Notes
    get :new_note, on: :member
    get :new_response_action_note, on: :member
    get :edit_note, on: :member
    post :create_note, on: :member
    post :create_response_action_note, on: :member
    patch :update_note, on: :member
    delete 'destroy:note/:note_id', to: 'incidents#destroy_note', as: :destroy_note, on: :member
    
    # Linked incidents
    delete 'unlink', to: 'incidents#unlink'

    # Detection
    get :detection_search, action: :detection_search, as: :detection_search, on: :collection

    # Response
    post :next_phase, on: :member
    post :previous_phase, on: :member
    post :close, on: :member
    get :new_response_action_execute, on: :member
    post :create_response_action_execute, on: :member

    # Worklog
    get :worklog_search, action: :worklog_search, as: :worklog_search, on: :collection
    
    # Reports
    get :create_incident_report, on: :member
    get :edit_report_modal, on: :member
    post :update_report_section, on: :member

    # AI
    get :check_ai_job_status
    post :generate_ai_reports, on: :member
    
    collection do
      get 'asset_details/:asset_id', to: 'incidents#asset_details', as: 'asset_details', constraints: { asset_id: /[^\/]+/ }
    end

  end

  post 'update_order', to: 'task_templates#update_order'

  resources :playbook_templates do
    get :search, action: :search, as: :search, on: :collection
    post :clone, action: :create_clone, on: :member
    patch :set_as_default, on: :member

    # Task Template
    get :new_task_template, on: :member
    get :edit_task_template, on: :member
    post :create_task_template, on: :member
    patch :update_task_template, on: :member
    delete 'destroy_task_template/:task_template_id', to: 'playbook_templates#destroy_task_template', as: :destroy_task_template, on: :member

    # Get integration service actions
    get 'integration_service_actions/:integration_service', to: 'playbook_templates#integration_service_actions', as: :integration_service_actions, on: :collection
  end

  resources :alarms do
    patch :reactivate, on: :member
  end

  resources :captive_portal_sessions do
    get :success, on: :collection
    get :preview, on: :member
    post :logout, on: :collection
  end

   # Reports

DittoCode::Exec.if 'ENTERPRISE' do
  resources :reports do
    get :new_sensor, on: :collection
    get :new_user, on: :collection
    get :composition, on: :member
    get :history_data, on: :collection
    get :history_records, on: :member
    get :export, on: :member
    get :import, on: :collection
    get :generate, on: :member
    post :import, on: :collection, action: :save_import
    post :send_now, on: :member
    post 'add_page/:after', on: :member, action: :add_page
    get 'remove_page/:page', on: :member, action: :remove_page_modal, as: :remove_page_modal
    post 'remove_page/:page', on: :member, action: :remove_page, as: :remove_page

    resources :widgets, except: [:index, :new] do
      get :list, on: :collection
      get 'graph/:product/(:page)', action: :graph, as: :graph, on: :collection
      get 'types_graphs/:widget_type/:page', action: :types_graphs, as: :types_graphs, on: :collection
      get :new_module, on: :member
      get :reload, on: :member
      get :image, on: :member
      get :clone, on: :member
      post :clone, action: :create_clone, on: :member
      get :sensors_map, on: :member
      post :set_position, on: :collection
      post 'move/:page', on: :member, action: :move, as: :move

      get 'new/:type/(:graph)/(:product)/(:page)', action: :new, as: :new, on: :collection
    end
  end
end

DittoCode::Exec.if 'COMMUNITY' do

  resources :reports, only: [:update, :edit, :index, :show, :destroy] do
    get :new_user, on: :collection
    get :composition, on: :member
    get :export, on: :member
    get :import, on: :collection
    post :import, on: :collection, action: :save_import
    post :send_now, on: :member
    post 'add_page/:after', on: :member, action: :add_page
    get 'remove_page/:page', on: :member, action: :remove_page_modal, as: :remove_page_modal
    post 'remove_page/:page', on: :member, action: :remove_page, as: :remove_page

    resources :widgets, except: [:index, :new] do
      get :list, on: :collection
      get 'graph/:product/:page', action: :graph, as: :graph, on: :collection
      get 'types_graphs/:widget_type/:page', action: :types_graphs, as: :types_graphs, on: :collection
      get :new_module, on: :member
      get :reload, on: :member
      get :image, on: :member
      get :clone, on: :member
      post :clone, action: :create_clone, on: :member
      get :sensors_map, on: :member
      post :set_position, on: :collection
      post 'move/:page', on: :member, action: :move, as: :move

      get 'new/:type/(:graph)/(:product)/(:page)', action: :new, as: :new, on: :collection
    end
  end
end

  #ReportsHistoricals

  get '/reports_historicals/:id/visualize_pdf', to: 'reports#visualize_pdf', as: 'visualize_pdf_report_historical'
  post 'reports_historicals', to: 'reports#create_report_historical'
  delete '/reports_historicals/:id', to: 'reports#destroy_report_historical', as: 'delete_report_historical'

  # Works with filters
  resources :filters, only:[] do
    collection do
      patch ':product/:id/del_and_add',
            to: 'filters#delete_and_add', as: 'delete_and_add'
      patch ':product/:id/del',
            to: 'filters#delete', as: 'delete'
      patch ':product/:id/add',
            to: 'filters#add', as: 'add'
      patch ':product/:id/contains',
            to: 'filters#contains', as: 'contains'
      patch ':product/:id/exclude',
            to: 'filters#exclude', as: 'exclude'
      patch 'multi',
            to: 'filters#multi', as: 'multi'
      patch ':product/domain',
            to: 'filters#domain', as: 'domain'
      post  ':product/advanced',
            to: 'filters#advanced', as: 'advanced'
    end

    post :globals, on: :collection
    get :expanded, on: :collection
  end

  resources :lookups do
    collection do
      get :getaddrinfo
    end
  end

  resources :users do
    patch :recreate_api_key, on: :collection
    get :time_selector, on: :collection
    patch :save_columns, on: :collection
    get 'search', action: :letter_search, on: :member
    get 'search', action: :search, on: :collection
    get :load_more, on: :collection
    get :list, on: :collection
  end

  resource :settings do
    post :rsa
    patch :update_initial_settings
    patch :upload_geoip_country_file, :upload_geoip_city_file, :upload_geoip_asn_file, :upload_geoip_countryv6_file, :upload_geoip_cityv6_file, :upload_geoip_asnv6_file
    patch :upload_sp_sso_cert_file
    patch :upload_sp_sso_priv_key_file
    delete :delete_geoip_file
    delete :delete_sp_sso_cert_file
    delete :delete_sp_sso_priv_key_file
    DittoCode::Exec.if 'ENTERPRISE' do
      patch :upload_http_key_file, :upload_http_pem_file
      delete :delete_http_files
    end
    get "trap_server/new", as: :new_trap_server, action: :new_trap_server
    get "radius_server/new", as: :new_radius_server, action: :new_radius_server
    get "radius_authentication_server/new", as: :new_radius_authentication_server, action: :new_radius_authentication_server
    get "white_network/new", as: :new_white_network, action: :new_white_network
    get "black_network/new", as: :new_black_network, action: :new_black_network
    get "manager_route/new", as: :new_manager_route, action: :new_manager_route
    get "fixed_host/new", as: :new_fixed_host, action: :new_fixed_host
    get 'download_ai_model', to: 'settings#download_ai_model', as: 'download_ai_model'
    post :trap_server, action: :create_trap_server
    post :radius_server, action: :create_radius_server
    post :radius_authentication_server, action: :create_radius_authentication_server
    post :white_network, action: :create_white_network
    post :black_network, action: :create_black_network
    post :manager_route, action: :create_manager_route
    post :fixed_host, action: :create_fixed_host
    delete "trap_server/:id", action: :delete_trap_server, as: :delete_trap_server, :constraints => { :id => /\d+/ }
    delete "radius_server/:id", action: :delete_radius_server, as: :delete_radius_server, :constraints => { :id => /\d+/ }
    delete "radius_authentication_server/:id", action: :delete_radius_authentication_server, as: :delete_radius_authentication_server, :constraints => { :id => /\d+/ }
    delete "white_network/:id", action: :delete_white_network, as: :delete_white_network, :constraints => { :id => /\d+/ }
    delete "black_network/:id", action: :delete_black_network, as: :delete_black_network, :constraints => { :id => /\d+/ }
    delete "manager_route/:id", action: :delete_manager_route, as: :delete_manager_route , :constraints => { :id => /\d+/ }
    delete "fixed_host", action: :delete_fixed_host, as: :delete_fixed_host
  end

  resources :sensors, except: :show do
    collection do
      resources :tree_filters, only: [:index, :show, :create], controller: :sensors_tree_filters
    end
    get :tree_search, on: :collection
    get :expand_all_tree, on: :collection
    get :search
    get :search, action: :global_search, on: :collection
    get :list, on: :collection
    get :user_root_domains, on: :collection
    get :domains_children
    get :children, on: :member
    DittoCode::Exec.if 'ENTERPRISE' do
      get :new_claim, on: :member
      put :claim, on: :member
      get :import_tree, on: :member
      post :import_tree, on: :member
      get :export_tree, on: :collection
      get :export_tree, on: :member
    end
    get :indexed_data, on: :member
    get :indexed_data_last_month, on: :member
    get :indexed_data_per_day, on: :member
    get :configuration_update, on: :member
    post :image, on: :member, action: :add_image
    delete :image, on: :member, action: :delete_image
    get :reregister, on: :member
    get :map, on: :member
    get :map, on: :collection
    get :new_trap_server, on: :member
    patch :set_chef_status, on: :member
    post :trap_server, on: :member, to: "sensors#create_trap_server"
    delete :trap_server, on: :member, to: "sensors#delete_trap_server"
    get :tree, on: :collection
    get :client_proxy, on: :collection
    get :exporter, on: :collection
    get :client_proxy, on: :member
    get :exporter, on: :member
    get 'client_proxy/load_more', action: :client_proxy_load_more, on: :collection
    get 'exporter/load_more', action: :exporter_load_more, on: :collection
    get 'client_proxy/:id/search', action: :client_proxy_letter_search, on: :collection, as: :client_proxy_letter_search
    get 'exporter/:id/search', action: :exporter_letter_search, on: :collection, as: :exporter_letter_search
    get 'client_proxy/search', action: :client_proxy_search, on: :collection
    get 'exporter/search', action: :exporter_search, on: :collection
    get :new_location, on: :member
    post :add_location, on: :member
    post :remove_location, on: :member
    get :new_interface, on: :member
    post :add_interface, on: :member
    get :new_observation_id_filter, on: :member
    post :observation_id_filter, on: :member, to: 'sensors#create_observation_id_filter'
    delete :observation_id_filter, on: :member, to: 'sensors#delete_observation_id_filter'

    patch :change_parent, on: :member
    post :save_exporter_data

    get "route/new", action: :new_route, as: :new_sensor_route, on: :member, :constraints => { :id => /\d+/ }
    post :route, action: :create_route, as: :sensor_route, on: :member, :constraints => { :id => /\d+/ }
    delete :route, action: :delete_route, as: :delete_sensor_route, on: :member, :constraints => { :id => /\d+/ }
    post :restart_chef_client, action: :restart_chef_client, on: :member, as: :restart_chef_client

    get 'thresholds' => 'event_filters#index', defaults: { format: 'txt' }

    resources :licenses, except: [:show] do
      post :create_request, on: :collection
      get :new_renew, on: :member
      post :create_renew, on: :collection
    end
  end

  resources :objects, controller: :redborder_objects, except: ['show'] do
    post 'import_file/:tab', action: :import_file, as: :import_file, on: :collection
    get 'import/:tab', action: :import, as: :import, on: :collection
    get 'export/:tab', action: :export, as: :export, on: :collection
    get 'search', action: :search, as: :search, on: :collection
    patch 'change_session_tab/:id', action: :change_session_tab, as: :change_object_session_tab, on: :collection
  end

  resources :object_types, controller: :object_types, except: ['show']  do
    collection do
      get 'search'
      post 'import_file/', action: :import_file, as: :import_file
      get 'import/', action: :import, as: :import
      get 'export/', action: :export, as: :export
    end
  end

  resources :application_objects, controller: :application_objects, except: ['show'] do
    post 'import_file/:tab', action: :import_file, as: :import_file, on: :collection
    get 'import/:tab', action: :import, as: :import, on: :collection
    get 'export/:tab', action: :export, as: :export, on: :collection
    get 'search', action: :search, as: :search, on: :collection
  end

  resources :jobs do
    get :info, on: :member
    collection do
      get :start
      get :stop
      get :restart
      get :status
      get :rule_db_version_status
      post :start_alarm
      post :start_checks
      post :start_report
      post :start_geoip_update
      get :start_rb_wakeup_chef_cluster
      get :start_incident_to_syslog_job
      get :start_glpi_inventory_job
      post :start_rb_vulnerabilities_update_db
      post :start_update_content_incidents
      get :search
      post :start_delete_stored_jobs
    end
  end

  delete 'stored_jobs/:id' => 'jobs#destroy_stored_job', as: 'stored_job'

  get 'domain/new' => 'sensors#new_domain'

  resources :notifications do
    patch :open, on: :collection
    get :reload, on: :collection
  end

  resources :managers, only: [:index] do
    collection do
      get :cluster_status
      get :cluster_status_small
      post :save_table_data
DittoCode::Exec.if 'ENTERPRISE' do
      get :virtual_ip_addresses
      get :cluster_synchronization
      post :save_cluster_synchronization
      post :save_virtual_ip_addresses
      post :save_rules
      get :historical_nodes
      get :namespaces
      get :load_more_namespaces
end
      post :reassign_partitions
    end

    member do
      post :up_pcp
      post :recovery_pcp
DittoCode::Exec.if 'ENTERPRISE' do
        get :namespace_rules
        patch :namespace_rules, action: :update_namespace_rules
end
    end
  end

  resources :audits do
    get :search, on: :collection
  end

  resources :overlay_maps do
    get :image, on: :member
    get :original_image, on: :member
    get :image_small, on: :member
    get :original_image_small, on: :member
  end

  resources :licenses do
    post :create_request, on: :collection
    get :new_renew, on: :member
    post :create_renew, on: :collection
    post :export_consumption_to_file, on: :collection
  end

  resources :integrations, only: [:index] do
    get ':page_name', to: 'integrations#show_page', as: 'page', on: :collection
    patch ':page_name', to: 'integrations#update', as: 'update_page', on: :collection
  end

  resources :mitre, only: [:index, :edit, :update] do
    member do
      get 'edit'
      patch '/', action: :update
      post 'assign_rule_to_mitre'
      post 'assign_tactic_to_rule'
      post 'deassign_tactic_from_rule'
      post 'deassign_rule_from_mitre'
      get 'json_mitre_techniques'
    end
    collection do
      post 'set_matrix_custom_time'
      get 'view_mitre_matrix'
      get 'search'
      get 'download'
      get 'auto_assign_job'
      get 'get_mitre_widget_data'
    end
  end

  # ------------------------------------------------------------------
  # API routes
  # ------------------------------------------------------------------
  namespace :api do
    namespace :v1 do
      devise_for :users, path_names: { sign_in: 'login', sign_out: 'logout' }, only: [:sessions]
      # Push notifications
      patch '/push/update', to: 'push#update'
      put '/push/update', to: 'push#update'
      # Fetch Objects
      RedborderObject::OBJECT_CLASSES.each do |obj_class|
        get obj_class.underscore.pluralize.to_sym,
            to: 'redborder_objects#index',
            type: obj_class
      end
      # Send data to druid
      post '/data', to: "data#index"
      # Rules
      get 'rules/active_rules', to: 'rules#active_rules'

      get 'cluster/cdomain', to: 'cluster#cdomain'

      DittoCode::Exec.if 'ENTERPRISE' do
        # Organization
        put '/organizations/:uuid/reach_bytes_limit', to: 'organizations#reach_bytes_limit'
      end

      # Users
      resources :users, only: [:index] do
        patch 'recreate_api_key', on: :member
      end

      # Sensors
      resources :sensors, only: [:index, :create, :destroy, :update],
                          param: :sensor_uuid do
        # Tree
        get :tree, on: :collection
        get :tree, on: :member, action: :tree
        # Override a tree
        post :override_all, on: :collection
        post :override_all, on: :member, action: :override_all
        # DEPRECATED
        post :create_domain, on: :collection
        # NEW
        post :domain, on: :collection, action: :create_domain
        # END
        put :claim, on: :collection
        # DEPRECATED
        get :delete, on: :member, to: 'sensors#destroy'
        # We must use DELETE /api/v1/sensors/:sensor_uuid
        patch :move, on: :member
        DittoCode::Exec.if 'ENTERPRISE' do
          get :nmsp_info, on: :member
        end
        get :licenses, on: :member
        get 'licenses/:id', on: :member, to: 'sensors#license'
      end

      # Monitor
      get 'monitor/', to: 'monitor#index'
      get 'monitor/:id/info', to: 'monitor#info'
      # Module queries
      get 'modules/:module/', to: 'modules#index'
      get 'modules/:module/raw_serie', to: 'modules#raw_serie'
      get 'modules/:module/:id/tops', to: 'modules#tops'
      get 'modules/:module/:id/top_serie', to: 'modules#top_serie'
      get 'modules/:module/info', to: 'modules#info'

      # License
      resources :licenses, only: [:index, :show]

      # Incidents
      resources :incidents, only: [:create] do
        collection do
          post 'link'
        end
      end
    end
    # TODO: This is an alias but we must delete this.
    post 'data', to: 'v1/data#index'
  end

  get '/nodes', to: 'api/v1/chef#nodes'
  get '/roles', to: 'api/v1/chef#roles'
  get '/nodes/:id', to: 'api/v1/chef#node'
  put '/nodes/:id', to: 'api/v1/chef#update_node'
  get '/roles/:id', to: 'api/v1/chef#role'
  get '/clients/:id', to: 'api/v1/chef#client'
  get '/environments', to: 'api/v1/chef#environments'
  get '/environments/:id', to: 'api/v1/chef#environment'
  post '/environments/:id/cookbook_versions', to: 'api/v1/chef#cookbook_versions'
  get '/data/passwords/:id', to: 'api/v1/chef#data_passwords'
  get '/data/rBglobal/:id', to: 'api/v1/chef#data_rBglobal'

  if Rails.env.production?
    get '/404', to: 'application#not_found'
    get '/422', to: 'application#internal_server_error'
    get '/500', to: 'application#internal_server_error'
  end

  if ENV['BLOCKED'] != 'true' && !ENV['NO_MODULES']
    Dir.glob File.expand_path('lib/modules/*', Rails.root) do |plugin_dir|
      module_name = plugin_dir.split('/').last

      # Check if the module is activated
      if Redborder::Module.available?(module_name)
        file = File.join(plugin_dir, 'config/routes.rb')
        if File.exist?(file)
          begin
            instance_eval File.read(file)
          rescue StandardError => e
            puts "An error occurred while loading the routes definition of #{File.basename(plugin_dir)} plugin (#{file}): #{e.message}."
            exit 1
          end
        end
      end
    end
  end
end
