/*!    
 * Copyright (c) 2014 ENEO Tecnología S.L.
 * This file is part of redBorder.
 * redBorder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * redBorder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with redBorder. If not, see <http://www.gnu.org/licenses/>.
 */

if (redBorder.malware === undefined){
  redBorder.malware = {}
}

/*
 * Infections page. We need to initialize the graph based on emails and
 * networks.
 */
redBorder.malware.Infections = function(data, options){
  // Properties
  this.emails = this._prepareData(data.emails);
  this.ips = this._prepareData(data.ips);
  this.maxBalls = options.maxBalls;
  this.graph = {};
  this.active = 'emails';

  // Initialize
  this.init();
}

redBorder.malware.Infections.prototype = {
  // Initialize elements and graphs
  init: function() {
    // Load graph
    this.graph.emails = new RB.Balls({
      container: "infections-graph-emails",
      elements: this.emails,
      callback: this._changeTab,
      maxBalls: this.maxBalls,
      object: this
    });

    this.graph.ips = new RB.Balls({
      container: "infections-graph-ips",
      elements: this.ips,
      callback: this._changeTab,
      maxBalls: this.maxBalls,
      object: this
    });

    // Add table sorter to tables
    $('table.table-sorter').tablesorter();
    
    // Load events
    this._bindEvents();
  },
  // Change the tab when click on a ball
  _changeTab: function(name, group){
    $('.nav-tabs.malware-infections[data-type="' + this.active + '"] li[data-name="' + name + '"] a').tab('show');
  },
  // Parse data of emails to calculate totals
  _prepareData: function(data) {
    var parsedData = [];

    // Prepare the data
    for(element in data){
      var total = 0;
      // Calc total
      data[element].map(function(el){ total += el.count });

      parsedData.push({ 
        name: element, 
        value: total
      });
    }

    return parsedData;
  },
  // Bind UI JS events
  _bindEvents: function() {
    var that = this;
    $(document).on('click', '.nav-tabs.malware-infections li', function(e){
      if (that.active == 'emails'){
        that.graph.emails.setActive($(this).data('name'));
      } else {
        that.graph.ips.setActive($(this).data('name'));
      }
    });

    // Change emails or ips view
    $(document).on('click', '.infections-selector button', function(e){
      if($(this).hasClass('btn-success')){
        return;
      }
      // Show the elements
      $('.infections-selector button.btn-success').removeClass('btn-success');
      $(this).addClass('btn-success');
      var show = $(this).data('type');
      // Store active view
      that.active = show;     
      var hide;

      if (show == 'emails'){
        hide = 'ips';
      } else {
        hide = 'emails';
      }
      // Show another graphs
      $("#infections-graph-" + hide + ", #infections-table-" + hide).fadeOut(200, function(){
        $("#infections-graph-" + show + ", #infections-table-" + show).fadeIn(200);
      });
    });
  },
  // Destroy events
  destroy: function() {
    // Disable click on nav tabs
    $(document).off('click', '.nav-tabs.malware-infections');
  }
};