/*!
 * Copyright (c) 2014 ENEO Tecnología S.L.
 * This file is part of redBorder.
 * redBorder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * redBorder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with redBorder. If not, see <http://www.gnu.org/licenses/>.
 */

redBorder.widgets.graphs.iocs_alerts = function (container, widget) {
  var graph,
      wtype,
      $container = $('#' + container),
      rendered = false,
      path = '.widget[data-widget-id="' + widget.id + '"] ',
      $table = $(path + 'table tbody');

  // The widget manager calls this method when the graph should start
  function init(_wtype) {
    wtype = _wtype;
  }

  function render(){
    // Bind events of the widget
    bindEvents();
    // This widget only need to call draw!
    wtype.queryRecent().then(draw);
  }

  // Draws the widget. This data comes directly from the widget
  // type selected. You should prepare the data to be used if needed.
  function draw(data) {
    var $table,
      uuid;

    $table = $(path + "table");
    $tbody = $table.find("tbody");

    if (data.iocs_alerts.length !== 0) {
      for(var i = 0; i < data.iocs_alerts.length; i++) {
        uuid = setUuid(data.iocs_alerts[i]);
        if(!existIocAlert($tbody, uuid)) {
          prependIocAlert($tbody, data.iocs_alerts[i]);
          if (rendered){
            alertWidget();
            if ($tbody.find("tr").length > 15) {
              $tbody.find("tr").last().remove();
            }
          }
        }
      }
      $table.fadeIn(50, function() {
        $(path + ".iocs-alerts-empty").fadeOut(50);
      });
    } else {
      $table.fadeOut(50, function() {
        $(path + ".iocs-alerts-empty").fadeIn(50);
      });
    }

    // Store first render
    if (!rendered) {
      rendered = true;
    }
    initStickyHeader(widget.id);
    // Finish load
    redBorder.widgets.util.reloadDone($container.parents('.widget'));
  }

  function setUuid(data) {
    return data.ioc + data.endpoint_uuid + data.action + data.timestamp;
  }

  function alertWidget() {
    $(path + ".alert-widget-bar").addClass("fade-pulse");
  }

  function existIocAlert($tbody, uuid) {
    return $tbody.find("tr[ioc-alert-uuid='" + uuid +"']").length !== 0;
  }

  function prependIocAlert($tbody, iocAlert) {
    var uuid = setUuid(iocAlert);
    $tbody.prepend(
      "<tr ioc-alert-uuid='" + uuid + "'>" +
        "<td>" +
          actionIcon(iocAlert.action) + " " +
          formatTime(new Date(iocAlert.timestamp)) +
        "</td>" +
        "<td>" + endPointName(iocAlert) + "</td>" +
        "<td>" + iocAlert.endpoint_hostname + "</td>" +
        "<td>" + iocName(iocAlert) + "</td>" +
      "</tr>");
  }

  function iocName(element) {
    // return "<a class='btn btn-small' href='endpoints/" +
    //   element.endpoint_id + "/details' data-toggle='modal' data-target='#default_modal'\
    //    data-backdrop: 'static'>" + element.ioc + "</a>";
    return element.ioc;
  }

  function endPointName(element) {
    return "<a target='blank' class='btn btn-small' href='endpoints/" + element.endpoint_id + "'>" +
      element.endpoint_name + "</a>";
  }

  function actionIcon(action) {
    if (action == "up")
      return "<i class='fa fa-arrow-circle-up red'></i>";
    else if (action == "down")
      return "<i class='fa fa-arrow-circle-down green'></i>";
  }

  // Format time Output
  function formatTime(time){
    var hours = time.getHours() < 10 ? "0" + time.getHours() : time.getHours();
    var minutes = time.getMinutes() < 10 ? "0" + time.getMinutes() : time.getMinutes();
    return hours + ":" + minutes;
  }

  // Reload function
  function reload(){
    // Get data
    $(path + ".alert-widget-bar").removeClass("fade-pulse");
    wtype.queryRecent().then(draw);
  }

  function bindEvents(){
    // Todo...
  }

  function resize() {
    var header = $(path + ".widget-header").height();
    $container.height($container.closest(".widget").height() - header);
  }

  // Hide the main element when resizing
  function hide() {
    $container.fadeOut(100);
  }

  // Show the main element
  function show() {
    $container.fadeIn(100, function() { initStickyHeader(widget.id) });
  }

  // Returns the graph object for debug purposes
  function getGraph() {
    return graph;
  }

  // Returns the widget
  function getWidget() {
    return widget;
  }

  return {
    init: init,
    resize: resize,
    getGraph: getGraph,
    getWidget: getWidget,
    hide: hide,
    show: show,
    reload: reload,
    draw: draw,
    render: render
  };
}
