/*!
 * Copyright (c) 2014 ENEO Tecnología S.L.
 * This file is part of redBorder.
 * redBorder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * redBorder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with redBorder. If not, see <http://www.gnu.org/licenses/>.
 */

redBorder.widgets.graphs.state_changes_alerts = function (container, widget) {
  var graph,
      wtype,
      $container = $('#' + container),
      rendered = false,
      path = '.widget[data-widget-id="' + widget.id + '"] ',
      $table = $(path + 'table tbody');

  // The widget manager calls this method when the graph should start
  function init(_wtype) {
    wtype = _wtype;
  }

  function render(){
    // Bind events of the widget
    bindEvents();
    // This widget only need to call draw!
    wtype.queryRecent().then(draw);
  }

  // Draws the widget. This data comes directly from the widget
  // type selected. You should prepare the data to be used if needed.
  function draw(data) {
    var $table,
      uuid;

    $table = $(path + "table");
    $tbody = $table.find("tbody");

    if (data.state_changes.length !== 0) {
      for(var i = 0; i < data.state_changes.length; i++) {
        uuid = setUuid(data.state_changes[i]);
        if(!existStateChange($tbody, uuid)) {
          appendStateChange($tbody, data.state_changes[i]);
          if (rendered){
            alertWidget();
            if ($tbody.find("tr").length > 15) {
              $tbody.find("tr").last().remove();
            }
          }
        }
      }
      $table.fadeIn(50, function() {
        $(path + ".state-changes-alerts-empty").fadeOut(50);
      });
    } else {
      $table.fadeOut(50, function() {
        $(path + ".state-changes-alerts-empty").fadeIn(50);
      });
    }

    // Store first render
    if (!rendered) {
      rendered = true;
      setReviewListener();
    };
    initStickyHeader(widget.id);
    // Finish load
    redBorder.widgets.util.reloadDone($container.parents('.widget'));
  }

  function setUuid(element) {
    return element.s3_old + element.s3_new;
  }

  function setReviewListener() {
    $(document).on("click", path + ".mark-as-reviewed", function() {
      event.preventDefault();
      var $row = $(this).closest("tr"),
        $table = $(path + "table");
      $(this).find("i").removeClass("fa-eye").addClass("fa-spinner fa-pulse");
      $.ajax({
        url: '/endpoints/state_changes/review',
        type: 'PATCH',
        data: {
          s3_old: $(this).data('s3-old'),
          s3_new: $(this).data('s3-new'),
          reviewed: "true"
        },
        dataType: 'script'
      }).done(function(){
        $row.remove();
        if ($table.find("tbody tr").length === 0) {
          $table.fadeOut(50, function() {
            $(path + ".state-changes-alerts-empty").fadeIn(50);
          });
        }
      });
    });
  }

  function alertWidget() {
    $(path + ".alert-widget-bar").addClass("fade-pulse");
  }

  function existStateChange($tbody, uuid) {
    return $tbody.find("tr[data-state-change-uuid='" + uuid +"']").length !== 0;
  }

  function appendStateChange($tbody, stateChange) {
    var uuid = setUuid(stateChange.s3_old);
    $tbody.append(
      "<tr data-state-change-uuid='" + uuid + "'>" +
        "<td>" + formatTime(new Date(stateChange.timestamp)) + "</td>" +
        "<td>" + endPointName(stateChange) + "</td>" +
        "<td>" + stateChange.ip_address + "</td>" +
        "<td>" + stateChange.flow_type + "</td>" +
        "<td class='row-center'>" + csvDownloadButton(stateChange.s3_old) +"</td>" +
        "<td class='row-center'>" + csvDownloadButton(stateChange.s3_new) +"</td>" +
        "<td class='last row-center'>" +
          "<a class='btn btn-small mark-as-reviewed' href='#' data-s3-new='" + stateChange.s3_new + "' data-s3-old='" + stateChange.s3_old + "'>" +
            "<i class='fa fa-eye'></i>" +
          "</a>" +
        "</td>" +
      "</tr>");
  }

  function endPointName(element) {
    return "<a target='blank' class='btn btn-small' href='endpoints/" + element.endpoint_id + "'>" +
      element.name + "</a>";
  };

  // Format time Output
  function formatTime(time){
    var months = ["Jan", "Feb", "Mar", "Apr", "May", "Jun",
      "Jul", "Aug", "Sept", "Oct", "Nov", "Dec"];

    var day = time.getDate();
    var month = months[time.getMonth()];

    var hours = time.getHours() < 10 ? "0" + time.getHours() : time.getHours();
    var minutes = time.getMinutes() < 10 ? "0" + time.getMinutes() : time.getMinutes();
    return day + ' ' + month + ' at ' + hours + ":" + minutes;
  }

  function csvDownloadButton(url) {
    return "<a class='btn btn-small' target='blank' " +
      "href='/endpoints/state_changes/download?object=" + url + "'>" +
      "<i class='fa fa-download'></i></a>";
  };

  // Reload function
  function reload(){
    // Get data
    $container.find(".alert-widget-bar").removeClass("fade-pulse");
    wtype.queryRecent().then(draw);
  }

  function bindEvents(){
    // Todo...
  }

  function resize() {
    var header = $(path + ".widget-header").height();
    $container.height($container.closest(".widget").height() - header);
  }

  // Hide the main element when resizing
  function hide() {
    $container.fadeOut(100);
  }

  // Show the main element
  function show() {
    $container.fadeIn(100, function() { initStickyHeader(widget.id) });
  }

  // Returns the graph object for debug purposes
  function getGraph() {
    return graph;
  }

  // Returns the widget
  function getWidget() {
    return widget;
  }

  return {
    init: init,
    resize: resize,
    getGraph: getGraph,
    getWidget: getWidget,
    hide: hide,
    show: show,
    reload: reload,
    draw: draw,
    render: render
  };
}
