#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

Rails.logger.info '[RB] Loading Flow module'

Redborder::Module.register :flow do

  name 'tr.flow.traffic'
  path 'flow'
  session_id 'traffic'
  data_source 'Traffic'
  customizable_tabs true
  selectable_widget true
  cep_topic :rb_flow_post

  menu :product_menu, :flow, { controller: :traffics, action: :tops },
       caption: "tr.flow.traffic", module: 'flow',
       html: { class: 'block-ui-but' }

  widgets_section :infrastructure, [{
       id: :hardwareinfo,
       name: 'Hardware Info',
       icon: 'fa-info'
     },{
       id: :hardwarestatus,
       name: 'Hardware Status',
       icon: 'fa-check-circle-o'
     }
  ]

  overview_widgets [
    {
      name: "tr.flow.bandwidth_line",
      partial: 'bandwidth',
      position: 4,
      info: "tr.flow.info"
    },
    {
      name: "tr.flow.clients",
      partial: 'clients',
      position: 5,
      info: "tr.flow.info2"
    },
    {
      name: "tr.flow.flow_quality",
      partial: 'flow_quality',
      position: 3
    },
    {
      name: "tr.flow.access_points",
      partial: 'access_points',
      position: 1,
      info: "tr.flow.info3"
    },
    {
      name: "tr.flow.channel_load",
      partial: 'channels_load',
      position: 2,
      info: ''
    }
  ]
  widgets [
    {
      name: 'tr.flow.widgets.bandwidth_line',
      icon: 'fa-random',
      fields: {
        graph_type: 'serie',
        serie_type: 'line',
        widget_type: 'raw',
        aggregation: 'bps'
      },
      report: false
    }, {
      id: 'clients',
      name: 'tr.flow.widgets.clients',
      icon: 'fa-users',
      report: false
    }, {
      id: 'floor_map',
      name: 'tr.flow.widgets.floor_map',
      icon: 'icons/floor_grey.svg',
      auto: false,
      report: false
    }, {
      id: 'bandwidth',
      name: 'tr.flow.widgets.bandwidth',
      icon: 'fa-random',
      report: false
    }, {
      id: 'heatmap',
      name: 'tr.flow.widgets.heatmap',
      icon: 'icons/heatmap.svg'
    }, {
      id: 'map',
      name: 'tr.flow.widgets.map',
      icon: 'fa-map-marker'
    }, {
      id: 'clients_map',
      name: 'tr.flow.widgets.clients_map',
      icon: 'icons/clientmap.png',
      report: false,
      module: :ap_state
    }, {
      id: 'user_experience',
      name: 'tr.flow.widgets.user',
      icon: 'fa-user',
      auto: false,
      fields: {
        graph_type: 'load'
      }
    }, {
      id: 'access_points',
      name: 'tr.flow.widgets.ap',
      icon: 'fa-wifi',
      auto: false,
      select_graph_type: true,
      module: :ap_state
    }, {
      id: 'channel_load',
      name: 'tr.flow.widgets.channel',
      icon: 'fa-wifi',
      auto: false,
      select_graph_type: true,
      module: :ap_state
    }
  ]

  columns ({
    # Application
    application_id_name: { enabled: false, size: 1, columns: { application_id_name: 'tr.flow.app' }, category: 'Application' },
    engine_id_name:      { enabled: false, size: 1, columns: { engine_id_name: 'tr.flow.engine' }, category: 'Application' },
    http_user_agent:     { enabled: false, size: 1, columns: { http_user_agent: 'tr.flow.http' }, category: 'Application' },
    host:                { enabled: false, size: 1, columns: { host: "tr.flow.host" }, category: 'Application' },
    host_l2_domain:      { enabled: false, size: 1, columns: { host_l2_domain: "tr.flow.host_l2" }, category: 'Application' },
    referer:             { enabled: false, size: 1, columns: { referer: 'Referer' }, category: 'Application' },
    referer_l2:          { enabled: false, size: 1, columns: { referer_l2: 'Referer L2' }, category: 'Application' },
    selector_name:       { enabled: false, size: 1, columns: { selector_name: 'Selector' }, category: 'Application' },

    # Interface
    lan_interface_name:        { enabled: false, size: 1, columns: { lan_interface_name: 'tr.flow.lan_interface' }, category: 'Interface' },
    lan_interface_description: { enabled: false, size: 1, columns: { lan_interface_description: 'tr.flow.lan_description' }, category: 'Interface' },
    wan_interface_name:        { enabled: false, size: 1, columns: { wan_interface_name: 'tr.flow.wan_interface' }, category: 'Interface' },
    wan_interface_description: { enabled: false, size: 1, columns: { wan_interface_description: 'tr.flow.wan_description' }, category: 'Interface' },
    wireless_station:          { enabled: false, size: 1, columns: { wireless_station: "tr.flow.wireless2" }, complementary_columns: { wireless_station_name: 'Wireless Station Name' }, category: 'Interface' },

    # Network
    sensor_name:         { enabled: true,  size: 2, columns: { sensor_name: 'Sensor' }, category: 'Network', shared: false },
    conversation:        { enabled: false, size: 1, columns: { lan_ip: 'LAN IP', wan_ip: 'WAN IP' }, name: 'Conversation', hide_raw: true, category: 'Network' },
    lan_ip:              { enabled: true,  size: 2, columns: { lan_ip: 'LAN IP' }, complementary_columns: { lan_ip_name: 'LAN IP Name' }, object_type: 'NetObject', category: 'Network' },
    lan_ip_name:         { enabled: false, size: 1, columns: { lan_ip_name: 'tr.flow.lan_name' }, category: 'Network' },
    lan_ip_net_name:     { enabled: false, size: 2, columns: { lan_ip_net_name: "tr.flow.lan_address" }, category: 'Network' },
    wan_ip:              { enabled: true , size: 2, columns: { wan_ip: 'WAN IP' }, complementary_columns: { wan_ip_name: 'WAN IP Name' }, object_type: 'NetObject', category: 'Network' },
    wan_ip_name:         { enabled: false, size: 1, columns: { wan_ip_name: 'tr.flow.wan_name' }, category: 'Network' },
    public_ip:           { enabled: true , size: 2, columns: { public_ip: 'tr.flow.public' }, complementary_columns: { public_ip_name: 'Public IP Name' }, object_type: 'NetObject', category: 'Network' },
    l4_proto:            { enabled: false, size: 1, columns: { l4_proto: 'tr.flow.protocol' }, complementary_columns: { l4_proto_name: 'Protocol Name' }, object_type: 'ProtocolObject', category: 'Network' },
    ip_protocol_version: { enabled: false, size: 1, columns: { ip_protocol_version: 'tr.flow.protocol_version' }, category: 'Network' },
    tos:                 { enabled: false, size: 1, columns: { tos: 'TOS' }, category: 'Network' },
    type:                { enabled: false, size: 1, columns: { type: 'tr.flow.sensor_type' }, category: 'Network', shared: false },
    scatterplot:         { enabled: false, size: 1, columns: { lan_ip: 'LAN IP', wan_ip: 'WAN IP' }, name: 'Scatterplot', hide_raw: true, category: 'Network' },
    trafficmap:         { enabled: false, size: 1, columns: { lan_ip: 'LAN IP', wan_ip: 'WAN IP' }, name: 'Traffic Map', hide_raw: true, category: 'Network' },
     
    # Flow
    direction: { enabled: false, size: 1, columns: { direction: 'tr.flow.direction' }, category: 'Flow' },

    # Location
    # wan_ip_country_code:   { enabled: false, size: 1, columns: { wan_ip_country_code: 'WAN IP Country' }, complementary_columns: { wan_ip_country: 'WAN IP Country Name' }, category: 'Location' },
    ip_country_code:       { enabled: false, size: 1, columns: { ip_country_code: 'tr.flow.country_code' }, complementary_columns: { ip_country: 'Country Name' }, category: 'Location' },
    wan_ip_as_name:        { enabled: false, size: 1, columns: { wan_ip_as_name: "tr.flow.wan_ip_as" }, category: 'Location' },
    # wan_ip_map:            { enabled: false, size: 1, columns: { wan_ip: 'WAN IP' }, complementary_columns: { wan_ip_name: 'WAN IP Name' }, name: 'WAN IP MAP', ips: :wan_ip, hide_raw: true, category: 'Location' },
    public_ip_map:         { enabled: false, size: 1, columns: { public_ip: 'tr.flow.public' }, complementary_columns: { public_ip_name: 'Public IP Name' }, name: 'Public IP MAP', ips: :public_ip, hide_raw: true, category: 'Location' },
    client_latlong:        { enabled: false, size: 1, columns: { client_latlong: 'tr.flow.coordinates' }, category: 'Location' },
    coordinates_map:       { enabled: false, size: 1, columns: { client_latlong: 'tr.flow.coord_map' }, hide_raw: true, category: 'Location' },
    service_provider:      { enabled: false, size: 1, columns: { service_provider: 'tr.flow.sp' }, category: 'Location', hide_global: true },
    service_provider_uuid: { enabled: false, size: 1, columns: { service_provider_uuid: 'tr.flow.sp_uuid' }, category: 'UUIDs', hide_global: true },
    namespace:             { enabled: false, size: 1, columns: { namespace: 'tr.flow.namespace' }, category: 'Location', hide_global: true },
    namespace_uuid:        { enabled: false, size: 1, columns: { namespace_uuid: 'tr.flow.namespace_uuid' }, category: 'UUIDs', hide_global: true },
    deployment:            { enabled: false, size: 1, columns: { deployment: 'tr.flow.deployment' }, category: 'Location', hide_global: true },
    deployment_uuid:       { enabled: false, size: 1, columns: { deployment_uuid: 'tr.flow.deployment_uuid' }, category: 'UUIDs', hide_global: true },
    market:                { enabled: false, size: 1, columns: { market: 'tr.flow.market' }, category: 'Location', hide_global: true },
    market_uuid:           { enabled: false, size: 1, columns: { market_uuid: 'tr.flow.market_uuid' }, category: 'UUIDs', hide_global: true },
    organization:          { enabled: false, size: 1, columns: { organization: 'tr.flow.org' }, category: 'Location', hide_global: true },
    organization_uuid:     { enabled: false, size: 1, columns: { organization_uuid: 'tr.flow.org_uuid' }, category: 'UUIDs', hide_global: true },
    campus:                { enabled: false, size: 1, columns: { campus: 'Campus' }, category: 'Location', hide_global: true },
    campus_uuid:           { enabled: false, size: 1, columns: { campus_uuid: 'tr.flow.campus_uuid' }, category: 'UUIDs', hide_global: true },
    building:              { enabled: false, size: 1, columns: { building: 'tr.flow.building' }, category: 'Location', hide_global: true },
    building_uuid:         { enabled: false, size: 1, columns: { building_uuid: 'tr.flow.building_uuid' }, category: 'UUIDs', hide_global: true },
    floor:                 { enabled: false, size: 1, columns: { floor: 'tr.flow.floor' }, category: 'Location', hide_global: true },
    floor_uuid:            { enabled: false, size: 1, columns: { floor_uuid: 'tr.flow.floor_uuid' }, category: 'UUIDs', hide_global: true },
    zone:                  { enabled: false, size: 1, columns: { zone: 'tr.flow.zone' }, category: 'Location' },

    # Transport
    lan_l4_port: { enabled: false, size: 1, columns: { lan_l4_port: 'LAN L4 Port' }, complementary_columns: { lan_l4_port_name: 'LAN L4 Port Name' }, object_type: 'ServiceObject', category: 'Transport' },
    lan_l4_port_name:         { enabled: false, size: 1, columns: { lan_l4_port_name: 'LAN L4 Port Name' }, category: 'Network' },
    wan_l4_port: { enabled: false, size: 1, columns: { wan_l4_port: 'WAN L4 Port' }, complementary_columns: { wan_l4_port_name: 'WAN L4 Port Name' }, object_type: 'ServiceObject', category: 'Transport' },
    wan_l4_port_name:         { enabled: false, size: 1, columns: { wan_l4_port_name: 'WAN L4 Port Name' }, category: 'Network' },
    tcp_flags:   { enabled: false, size: 1, columns: { tcp_flags: 'TCP flags' }, category: 'Transport' },

    # Datalink
    lan_vlan:    { enabled: false, size: 1, columns: { lan_vlan: 'LAN VLAN' }, complementary_columns: { lan_vlan_name: 'LAN VLAN Name' }, object_type: 'VlanObject', category: 'Datalink' },
    wan_vlan:    { enabled: false, size: 1, columns: { wan_vlan: 'WAN VLAN' }, complementary_columns: { wan_vlan_name: 'WAN VLAN Name' }, object_type: 'VlanObject', category: 'Datalink' },
    wireless_id: { enabled: false,  size: 1, columns: { wireless_id: 'Wireless ID' }, category: 'Datalink' },

    # Client Data
    dot11_protocol:         { enabled: false, size: 1, columns: { dot11_protocol: 'Band' }, category: 'Client Data' },
    client_accounting_type: { enabled: false, size: 1, columns: { client_accounting_type: "tr.flow.client_type" }, category: 'Client Data' },
    client_id:              { enabled: false, size: 1, columns: { client_id: "tr.flow.client_id" }, category: 'Client Data' },
    client_mac:             { enabled: false, size: 1, columns: { client_mac: "tr.flow.client_mac" }, complementary_columns: { client_mac_name: 'Client Name', client_mac_name_two: 'Vendor', client_mac_type: 'Object Type', client_mac_inventored: 'Inventored' }, name: 'Client MAC', object_type: 'MacObject', category: 'Client Data' },
    client_rssi:            { enabled: false, size: 1, columns: { client_rssi: "tr.flow.client_rssi" }, complementary_columns: { client_rssi_value: 'Value' }, category: 'Client Data' },
    dot11_status:           { enabled: false, size: 1, columns: { dot11_status: "tr.flow.client_status" },category: 'Client Data' },
    client_mac_vendor:      { enabled: false, size: 1, columns: { client_mac_vendor: "tr.flow.client_vendor" }, category: 'Client Data' },

    # Darklist
    darklist_score_name:    { enabled: false,  size: 1, columns: { darklist_score_name: "Darklist Score" }, category: "Darklist" },
    darklist_category:      { enabled: false,  size: 1, columns: { darklist_category: "Darklist Category" }, category: "Darklist" },
    darklist_direction:     { enabled: false,  size: 1, columns: { darklist_direction: "Darklist Direction" }, category: "Darklist"},

  })

  aggregations ({
    bps: {
      name: 'bps', unit: 'bps', value: :bps, size: 1,
      aggregations: [
        { type: 'longSum', name: 'bytes', fieldName: 'sum_bytes' }
      ],
      post_aggregations: [
        {
          type: 'arithmetic',
          name: 'bps',
          fn: '/',
          fields: [
            {
              type: 'arithmetic',
              name: 'bits',
              fn: '*',
              fields: [
                {
                  type: 'fieldAccess',
                  fieldName: 'bytes'
                },
                {
                  type: 'constant',
                  value: 8
                }
              ]
            },
            {
              type: 'constant',
              value: proc { |t| t }
            }
          ]
        }
      ],
      description: "tr.flow.number"
    },
    bytes: {
      name: 'bytes', unit: 'B', value: :bytes, size: 1,
      aggregations: [
        { type: 'longSum', name: 'bytes', fieldName: 'sum_bytes' }
      ],
      description: "tr.flow.total"
    },
    pkts: {
      name: 'tr.flow.packets', unit: 'pkts', value: :pkts, size: 1,
      aggregations: [
        { type: 'longSum', name: 'pkts', fieldName: 'sum_pkts' }
      ],
      description: "tr.flow.total2"
    },
    pps: {
      name: 'tr.flow.packets_s', unit: 'pps', value: :pps, size: 1,
      aggregations: [
        { type: 'longSum', name: 'pkts', fieldName: 'sum_pkts' }
      ],
      post_aggregations: [
        {
          type: 'arithmetic',
          name: 'pps',
          fn: '/',
          fields: [
            {
              type: 'fieldAccess',
              fieldName: 'pkts'
            },
            {
              type: 'constant',
              value: proc { |t| t }
            }
          ]
        }
      ],
      description: "tr.flow.number2"
    },
    flows: {
      name: 'flows', unit: 'flows', value: :flows, size: 1,
      aggregations: [
        { type: 'longSum', name: 'flows', fieldName: 'events' }
      ],
      description: 'tr.flow.exported'
    },
    fps: {
      name: 'flows/s', unit: 'fps', value: :fps, size: 1,
      aggregations: [
        { type: 'longSum', name: 'events', fieldName: 'events' }
      ],
      post_aggregations: [
        {
          type: 'arithmetic',
          name: 'fps',
          fn: '/',
          fields: [
            {
              type: 'fieldAccess',
              fieldName: 'events'
            },
            {
              type: 'constant',
              value: proc { |t| t }
            }
          ]
        }
      ],
      description: "tr.flow.number3"
    },
    clients: {
      name: 'tr.flow.client', unit: 'clients', value: :clients, size: 1, round: true,
      aggregations: [
        { type: 'hyperUnique', name: 'clients', fieldName: 'clients' }
      ],
      description: "tr.flow.number4"
    },
    quality: {
      name: 'tr.flow.quality', value: :quality,
      size: 1, round: true, unit: '%',
      aggregations: [
        { type: 'longSum', name: 'flows',    fieldName: 'events' },
        { type: 'longSum', name: 'sum_rssi', fieldName: 'sum_rssi' }
      ],
      post_aggregations: [{
        name: 'quality',
        type: 'javascript',
        fieldNames: [:sum_rssi, :flows],
        function: 'function (sum_rssi, flows) { var quality = sum_rssi / flows; if (quality == 0 || quality < -80) { return 0; } else if (quality > -40) { return 100; } else { return (2.5 * (quality + 80)); } }'
      }],
      description: "tr.flow.percentage"
    },
    risk: {
      name: 'tr.flow.risk', value: :risk, size: 1, unit: '',
      aggregations: [
        { type: 'longSum', name: 'flows', fieldName: 'events' },
        { type: 'longSum', name: 'sum_dl_score', fieldName: 'sum_dl_score' }
      ],
      post_aggregations: [
        {
          type: 'arithmetic',
          name: 'risk',
          fn: '/',
          fields: [
            { type: 'fieldAccess', fieldName: 'sum_dl_score' },
            { type: 'fieldAccess', fieldName: 'flows' }
          ]
        }
      ],
      description: "tr.flow.percentage2"
    },
    wireless_stations: {
      name: 'tr.flow.stations', unit: 'stations',
      value: :wireless_stations, size: 1, round: true,
      aggregations: [
        { type: 'hyperUnique', name: 'wireless_stations',
          fieldName: 'wireless_stations' }
      ],
      description: "tr.flow.wireless"
    },
    clients_per_station: {
      name: 'tr.flow.clients_station', unit: 'clients/station',
      value: :clients_per_station, size: 1, round: true,
      aggregations: [
        { type: 'hyperUnique', name: 'wireless_stations',
          fieldName: 'wireless_stations' },
        { type: 'hyperUnique', name: 'clients', fieldName: 'clients'  }
      ],
      post_aggregations: [
        {
          type: 'arithmetic',
          name: 'clients_per_station',
          fn: '/',
          fields: [
            {
              type: 'hyperUniqueCardinality',
              fieldName: 'clients'
            },
            {
              type: 'hyperUniqueCardinality',
              fieldName: 'wireless_stations'
            }
          ]
        }
      ],
      description: "tr.flow.clients"
    },
    bits_per_sec_per_client: {
      name: 'tr.flow.bps_client', unit: 'bps/client',
      value: :bits_per_sec_per_client, size: 1,
      aggregations: [
        { type: 'longSum', name: 'bytes', fieldName: 'sum_bytes' },
        { type: 'hyperUnique', name: 'clients', fieldName: 'clients' }
      ],
      post_aggregations: [
        {
          type: 'arithmetic',
          name: 'bits_per_sec_per_client',
          fn: '/',
          fields: [
            {
              type: 'arithmetic',
              name: 'bps',
              fn: '/',
              fields: [
                {
                  type: 'arithmetic',
                  name: 'bits',
                  fn: '*',
                  fields: [
                    {
                      type: 'fieldAccess',
                      fieldName: 'bytes'
                    },
                    {
                      type: 'constant',
                      value: 8
                    }
                  ]
                },
                {
                  type: 'constant',
                  value: proc { |t| t }
                }
              ]
            },
            {
              type: 'hyperUniqueCardinality',
              fieldName: 'clients'
            }
          ]
        }
      ],
      description: "tr.flow.average"
    },
    bytes_per_client: {
      name: 'tr.flow.bytes_client', unit: 'B/client',
      value: :bytes_per_client, size: 1,
      aggregations: [
        { type: 'longSum', name: 'bytes', fieldName: 'sum_bytes' },
        { type: 'hyperUnique', name: 'clients', fieldName: 'clients'  }
      ],
      post_aggregations: [
        {
          type: 'arithmetic',
          name: 'bytes_per_client',
          fn: '/',
          fields: [
            {
              type: 'fieldAccess',
              name: 'bytes',
              fieldName: 'bytes'
            },
            {
              type: 'hyperUniqueCardinality',
              fieldName: 'clients'
            }
          ]
        }
      ],
      description: "tr.flow.average2"
    },
    flows_per_client: {
      name: 'tr.flow.flows_client', unit: 'flows/client',
      value: :flows_per_client, size: 1,
      aggregations: [
        { type: 'longSum'    , name: 'flows'  , fieldName: 'events' },
        { type: 'hyperUnique', name: 'clients', fieldName: 'clients'  }
      ],
      post_aggregations: [
        {
          type: 'arithmetic',
          name: 'flows_per_client',
          fn: '/',
          fields: [
            {
              type: 'fieldAccess',
              name: 'flows',
              fieldName: 'flows'
            },
            {
              type: 'hyperUniqueCardinality',
              fieldName: 'clients'
            }
          ]
        }
      ],
      description: "tr.flow.flows"
    },
    flows_per_sec_per_client: {
      name: 'tr.flow.fps_client', unit: 'fps/client',
      value: :flows_per_sec_per_client, size: 1,
      aggregations: [
        { type: 'longSum', name: 'events', fieldName: 'events' },
        { type: 'hyperUnique', name: 'clients', fieldName: 'clients'  }
      ],
      post_aggregations: [
        {
          type: 'arithmetic',
          name: 'flows_per_sec_per_client',
          fn: '/',
          fields: [
            {
              type: 'arithmetic',
              name: 'fps',
              fn: '/',
              fields: [
                {
                  type: 'fieldAccess',
                  fieldName: 'events'
                },
                {
                  type: 'constant',
                  value: proc { |t| t }
                }
              ]
            },
            {
              type: 'hyperUniqueCardinality',
              fieldName: 'clients'
            }
          ]
        }
      ],
      description: "tr.flow.flows2"
    },
    bytes_per_station: {
      name: 'tr.flow.bytes_station', unit: 'B/station',
      value: :bytes_per_station, size: 1,
      aggregations: [
        { type: 'longSum', name: 'bytes', fieldName: 'sum_bytes' },
        { type: 'hyperUnique', name: 'wireless_stations',
          fieldName: 'wireless_stations' }
      ],
      post_aggregations: [
        {
          type: 'arithmetic',
          name: 'bytes_per_station',
          fn: '/',
          fields: [
            {
              type: 'fieldAccess',
              name: 'bytes',
              fieldName: 'bytes'
            },
            {
              type: 'hyperUniqueCardinality',
              fieldName: 'wireless_stations'
            }
          ]
        }
      ],
      description: "tr.flow.average3"
    },
    bits_per_sec_per_station: {
      name: 'tr.flow.bps_station', unit: 'bps/station',
      value: :bits_per_sec_per_station, size: 1,
      aggregations: [
        { type: 'longSum', name: 'bytes', fieldName: 'sum_bytes' },
        { type: 'hyperUnique', name: 'wireless_stations',
          fieldName: 'wireless_stations' }
      ],
      post_aggregations: [
        {
          type: 'arithmetic',
          name: 'bits_per_sec_per_station',
          fn: '/',
          fields: [
            {
              type: 'arithmetic',
              name: 'bps',
              fn: '/',
              fields: [
                {
                  type: 'arithmetic',
                  name: 'bits',
                  fn: '*',
                  fields: [
                    {
                      type: 'fieldAccess',
                      fieldName: 'bytes'
                    },
                    {
                      type: 'constant',
                      value: 8
                    }
                  ]
                },
                {
                  type: 'constant',
                  value: proc { |t| t }
                }
              ]
            },
            {
              type: 'hyperUniqueCardinality',
              fieldName: 'wireless_stations'
            }
          ]
        }
      ],
      description: "tr.flow.network"
    }
  })

  menu :sensor_tree_menu, :new_flow_sensor, { controller: :sensors, action: :new_flow },
        caption: "<i class='sensor-menu-icon flow'></i> Flow".html_safe,
        html: {'data-backdrop' => "static", 'data-target' => "#new_sensor_modal", 'data-toggle' => "modal"}

  DittoCode::Exec.if 'ENTERPRISE' do
  menu :sensor_tree_menu, :new_mse_sensor, { controller: :sensors, action: :new_mse },
        caption: "<i class='sensor-menu-icon mse'></i> MSE".html_safe,
        html: {'data-backdrop' => "static", 'data-target' => "#new_sensor_modal", 'data-toggle' => "modal"}
  menu :sensor_tree_menu, :new_meraki_sensor, { controller: :sensors, action: :new_meraki },
        caption: "<i class='sensor-menu-icon meraki'></i> Meraki".html_safe,
        html: {'data-backdrop' => "static", 'data-target' => "#new_sensor_modal", 'data-toggle' => "modal"}
  menu :sensor_tree_menu, :new_ale_sensor, { controller: :sensors, action: :new_ale },
        caption: "<i class='sensor-menu-icon mse'></i> Aruba ALE".html_safe,
        html: {'data-backdrop' => "static", 'data-target' => "#new_sensor_modal", 'data-toggle' => "modal"}
  menu :sensor_tree_menu, :new_arubacentral_sensor, { controller: :sensors, action: :new_arubacentral },
        caption: "<i class='sensor-menu-icon mse'></i> Aruba Central".html_safe,
        html: {'data-backdrop' => "static", 'data-target' => "#new_sensor_modal", 'data-toggle' => "modal"}
  menu :sensor_tree_menu, :new_device_sensor, { controller: :sensors, action: :new_device },
        caption: "<i class='sensor-menu-icon device'></i> Monitor Device".html_safe,
        html: {'data-backdrop' => "static", 'data-target' => "#new_sensor_modal", 'data-toggle' => "modal"}
  end

  # Menu for Flow sensors
  menu :flow_menu, :edit, {controller: "sensors", action: "edit"}, caption: "tr.flow.edit", first: true, html: {class: "block-ui-but"}, admin: true
  menu :flow_menu, :view, {controller: 'sensors', action: 'view'}, caption: "tr.flow.view", html: {class: 'block-ui-but'}
  menu :flow_menu, :monitors, {controller: "monitor_categories", action: "index"}, param: :sensor_id, caption: "tr.flow.monitors", admin: true, if: Proc.new { |s| User.current_user.modules.include?('monitor') }

  DittoCode::Exec.if 'ENTERPRISE' do
  # Menu for MSE sensors
  menu :mse_menu, :edit, {controller: "sensors", action: "edit"}, caption: "tr.flow.edit", first: true, html: {class: "block-ui-but"}, admin: true
  menu :mse_menu, :monitors, {controller: "monitor_categories", action: "index"}, param: :sensor_id, caption: "tr.flow.monitors", admin: true, if: Proc.new { |s| User.current_user.modules.include?('monitor') }

  # Menu for Meraki sensors
  menu :meraki_menu, :edit, { controller: 'sensors', action: 'edit' },
                     caption: "tr.flow.edit", first: true,
                     html: {class: 'block-ui-but'}, admin: true
  menu :meraki_menu, :view,
                     { controller: 'sensors', action: 'view' },
                     caption: "tr.flow.view", html: {class: 'block-ui-but'}

  # Menu for Aruba ALE sensors
  menu :ale_menu, :edit, {controller: "sensors", action: "edit"}, caption: "tr.flow.edit", first: true, html: {class: "block-ui-but"}, admin: true
  # menu :ale_menu, :monitors, {controller: "monitor_categories", action: "index"}, param: :sensor_id, caption: "Monitors", admin: true, if: Proc.new { |s| User.current_user.modules.include?('monitor') }
  
  # Menu for Aruba Central
  menu :arubacentral_menu, :edit, {controller: "sensors", action: "edit"}, caption: "tr.flow.edit", first: true, html: {class: "block-ui-but"}, admin: true

  # Menu for DEVICE sensors
  menu :device_menu, :edit, {controller: "sensors", action: "edit"}, caption: "tr.flow.edit", first: true, html: {class: "block-ui-but"}, admin: true
  menu :device_menu, :monitors, {controller: "monitor_categories", action: "index"}, param: :sensor_id, caption: "tr.flow.monitors", admin: true, if: Proc.new { |s| User.current_user.modules.include?('monitor') }
  menu :device_menu, :hardware, {controller: "monitor", action: "device"}, caption: "tr.flow.hardware_info", admin: true
  end

  Redborder::MenuManager.map :domain_menu do |menu|
    menu.push :view, "#", caption: "tr.flow.view", after: :edit,
              module: 'flow'
    menu.push :add_sensor, "#", caption: "tr.flow.add", after: :add_domain,
              admin: true,
              if: proc { |s| (RedBorder::LICENSING_MODE == 'organization' &&
                                  [Sensor::ORGANIZATION, Sensor::MARKET,
                                   Sensor::CAMPUS, Sensor::BUILDING,
                                   Sensor::DEPLOYMENT].include?(s.domain_type)) ||
                                 (RedBorder::LICENSING_MODE == 'global' &&
                                  [Sensor::ORGANIZATION, Sensor::MARKET,
                                   Sensor::CAMPUS, Sensor::BUILDING, Sensor::DEPLOYMENT,
                                   Sensor::NAMESPACE].include?(s.domain_type))
                           },
              module: 'flow'
    menu.push :tiles_view, {controller: 'sensors', action: 'view'},
              caption: "tr.flow.ap", parent: :view, after: :map,
              if: Proc.new { |s| [Sensor::ORGANIZATION,
                                  Sensor::MARKET, Sensor::CAMPUS,
                                  Sensor::BUILDING, Sensor::DEPLOYMENT,
                                  Sensor::SERVICE_PROVIDER,
                                  Sensor::NAMESPACE].include?(s.domain_type) },
              html: { class: 'block-ui-but' },
              module: 'flow'

    menu.push :floor_access_points_view, {controller: "sensors", action: "edit"},
              caption: "tr.flow.ap", parent: :view, after: :map,
              if: Proc.new { |s| s.floor? },
              html: { class: 'block-ui-but' },
              module: 'flow'

    menu.push :flow_view, {controller: 'sensors', action: 'flow'},
              caption: 'tr.flow.flow_sensors', parent: :view, after: :mosaic_view,
              html: { class: 'block-ui-but' },
              module: 'flow'

    DittoCode::Exec.if 'ENTERPRISE' do
    menu.push :mse_view, {controller: 'sensors', action: 'mse'},
              caption: 'tr.flow.mse_sensors', parent: :view, after: :flow_view,
              html: { class: 'block-ui-but' },
              module: 'flow'
    menu.push :meraki_view, {controller: 'sensors', action: 'meraki'},
              caption: 'tr.flow.meraki_sensors', parent: :view, after: :flow_view,
              html: { class: 'block-ui-but' },
              module: 'flow'
    menu.push :ale_view, {controller: 'sensors', action: 'ale'},
              caption: 'tr.flow.ale_sensors', parent: :view, after: :flow_view,
              html: { class: 'block-ui-but' },
              module: 'flow'
    menu.push :arubacentral_view, {controller: 'sensors', action: 'arubacentral'},
              caption: 'tr.flow.arubacentral_sensors', parent: :view, after: :flow_view,
              html: { class: 'block-ui-but' },
              module: 'flow'
    menu.push :device_view, {controller: 'sensors', action: 'device'},
              caption: 'tr.flow.monitor_device', parent: :view, after: :flow_view,
              html: { class: 'block-ui-but' },
              module: 'flow'
    end

    menu.push :add_flow_sensor, { controller: :sensors, action: :new_flow },
              caption: 'Flow',
              icon: 'sensor-menu-icon flow',
              parent: :add_sensor,
              html: {'data-backdrop' => 'static', 'data-target' => '#new_sensor_modal',
                     'data-toggle' => 'modal'},
              admin: true,
              if: proc { |s| (RedBorder::LICENSING_MODE == 'organization' &&
                                  [Sensor::ORGANIZATION, Sensor::MARKET,
                                   Sensor::CAMPUS, Sensor::BUILDING,
                                   Sensor::DEPLOYMENT].include?(s.domain_type)) ||
                                 (RedBorder::LICENSING_MODE == 'global' &&
                                  [Sensor::ORGANIZATION, Sensor::MARKET,
                                   Sensor::CAMPUS, Sensor::BUILDING, Sensor::DEPLOYMENT,
                                   Sensor::NAMESPACE].include?(s.domain_type))
                           },
              module: 'flow'

    DittoCode::Exec.if 'ENTERPRISE' do
    menu.push :add_mse_sensor, { controller: :sensors, action: :new_mse },
              caption: 'MSE',
              parent: :add_sensor,
              icon: 'sensor-menu-icon mse',
              html: {'data-backdrop' => 'static', 'data-target' => '#new_sensor_modal',
                     'data-toggle' => 'modal'},
              admin: true,
              if: proc { |s| (RedBorder::LICENSING_MODE == 'organization' &&
                                  [Sensor::ORGANIZATION, Sensor::MARKET,
                                   Sensor::CAMPUS, Sensor::BUILDING,
                                   Sensor::DEPLOYMENT].include?(s.domain_type)) ||
                                 (RedBorder::LICENSING_MODE == 'global' &&
                                  [Sensor::ORGANIZATION, Sensor::MARKET,
                                   Sensor::CAMPUS, Sensor::BUILDING, Sensor::DEPLOYMENT,
                                   Sensor::NAMESPACE].include?(s.domain_type))
                           },
              module: 'flow'
    menu.push :add_meraki_sensor, { controller: :sensors, action: :new_meraki },
              caption: 'Meraki',
              parent: :add_sensor,
              icon: 'sensor-menu-icon meraki',
              html: {'data-backdrop' => 'static',
                     'data-target' => '#new_sensor_modal',
                     'data-toggle' => 'modal'},
              admin: true,
              if: proc { |s| (RedBorder::LICENSING_MODE == 'organization' &&
                                  [Sensor::ORGANIZATION, Sensor::MARKET,
                                   Sensor::CAMPUS, Sensor::BUILDING,
                                   Sensor::DEPLOYMENT].include?(s.domain_type)) ||
                                 (RedBorder::LICENSING_MODE == 'global' &&
                                  [Sensor::ORGANIZATION, Sensor::MARKET,
                                   Sensor::CAMPUS, Sensor::BUILDING, Sensor::DEPLOYMENT,
                                   Sensor::NAMESPACE].include?(s.domain_type))
                           },
              module: 'flow'

      menu.push :add_ale_sensor, { controller: :sensors, action: :new_ale },
                caption: 'ALE',
                parent: :add_sensor,
                icon: 'sensor-menu-icon mse',
                html: {'data-backdrop' => 'static', 'data-target' => '#new_sensor_modal',
                       'data-toggle' => 'modal'},
                admin: true,
                if: proc { |s| (RedBorder::LICENSING_MODE == 'organization' &&
                                    [Sensor::ORGANIZATION, Sensor::MARKET,
                                     Sensor::CAMPUS, Sensor::BUILDING,
                                     Sensor::DEPLOYMENT].include?(s.domain_type)) ||
                                   (RedBorder::LICENSING_MODE == 'global' &&
                                    [Sensor::ORGANIZATION, Sensor::MARKET,
                                     Sensor::CAMPUS, Sensor::BUILDING, Sensor::DEPLOYMENT,
                                     Sensor::NAMESPACE].include?(s.domain_type))
                             },
                module: 'flow'

      menu.push :add_arubacentral_sensor, { controller: :sensors, action: :new_arubacentral },
                caption: 'Aruba Central',
                parent: :add_sensor,
                icon: 'sensor-menu-icon mse',
                html: {'data-backdrop' => 'static', 'data-target' => '#new_sensor_modal',
                        'data-toggle' => 'modal'},
                admin: true,
                if: proc { |s| (RedBorder::LICENSING_MODE == 'organization' &&
                                    [Sensor::ORGANIZATION, Sensor::MARKET,
                                      Sensor::CAMPUS, Sensor::BUILDING,
                                      Sensor::DEPLOYMENT].include?(s.domain_type)) ||
                                    (RedBorder::LICENSING_MODE == 'global' &&
                                    [Sensor::ORGANIZATION, Sensor::MARKET,
                                      Sensor::CAMPUS, Sensor::BUILDING, Sensor::DEPLOYMENT,
                                      Sensor::NAMESPACE].include?(s.domain_type))
                              },
                module: 'flow'

      menu.push :add_device_sensor, { controller: :sensors, action: :new_device },
                caption: 'tr.flow.monitor_device',
                parent: :add_sensor,
                icon: 'sensor-menu-icon device',
                html: {'data-backdrop' => 'static', 'data-target' => '#new_sensor_modal',
                       'data-toggle' => 'modal'},
                admin: true,
                if: proc { |s| (RedBorder::LICENSING_MODE == 'organization' &&
                                    [Sensor::ORGANIZATION, Sensor::MARKET,
                                     Sensor::CAMPUS, Sensor::BUILDING,
                                     Sensor::DEPLOYMENT].include?(s.domain_type)) ||
                                   (RedBorder::LICENSING_MODE == 'global' &&
                                    [Sensor::ORGANIZATION, Sensor::MARKET,
                                     Sensor::CAMPUS, Sensor::BUILDING, Sensor::DEPLOYMENT,
                                     Sensor::NAMESPACE].include?(s.domain_type))
                             },
                module: 'flow'

  end

  Redborder::MenuManager.map :client_proxy_menu do |menu|
    menu.push  :add_flow_sensor, { controller: :sensors, action: :new_flow },
              caption: 'Flow',
              icon: 'sensor-menu-icon flow',
              parent: :add_sensor,
              html: {'data-backdrop' => 'static', 'data-target' => '#new_sensor_modal',
                     'data-toggle' => 'modal'},
              admin: true,
              module: 'flow'
    DittoCode::Exec.if 'ENTERPRISE' do
    menu.push :add_mse_sensor, { controller: :sensors, action: :new_mse },
              caption: 'MSE',
              parent: :add_sensor,
              icon: 'sensor-menu-icon mse',
              html: {'data-backdrop' => 'static', 'data-target' => '#new_sensor_modal',
                     'data-toggle' => 'modal'},
              admin: true,
              module: 'flow'

    menu.push :add_device_sensor, { controller: :sensors, action: :new_device },
              caption: 'tr.flow.monitor_device',
              parent: :add_sensor,
              icon: 'sensor-menu-icon device',
              html: {'data-backdrop' => 'static', 'data-target' => '#new_sensor_modal',
                     'data-toggle' => 'modal'},
              admin: true,
              module: 'flow'
    end
    
    menu.push :add_ale_sensor, { controller: :sensors, action: :new_ale },
                caption: 'ALE',
                parent: :add_sensor,
                icon: 'sensor-menu-icon mse',
                html: {'data-backdrop' => 'static', 'data-target' => '#new_sensor_modal',
                       'data-toggle' => 'modal'},
                admin: true,
                module: 'flow'

    menu.push :add_arubacentral_sensor, { controller: :sensors, action: :new_arubacentral },
                caption: 'Aruba Central',
                parent: :add_sensor,
                icon: 'sensor-menu-icon mse',
                html: {'data-backdrop' => 'static', 'data-target' => '#new_sensor_modal',
                       'data-toggle' => 'modal'},
                admin: true,
                module: 'flow'
                  
    end
  end

  menu :sensors_view_menu,
       :tiles_view, {controller: 'sensors', action: 'view'},
       caption: "tr.flow.ap", after: :tree_view,
       html: {class: 'block-ui-but'}
  menu :sensors_view_menu, :flow_view,
       {controller: "sensors", action: "flow"},
       caption: "tr.flow.flow_sensors", after: :client_proxy_view,
       html: {class: "block-ui-but"}

  DittoCode::Exec.if 'ENTERPRISE' do
  menu :sensors_view_menu, :mse_view,
       {controller: "sensors", action: "mse"},
       caption: "tr.flow.mse_sensors", html: {class: "block-ui-but"}
  menu :sensors_view_menu, :ale_view,
       {controller: "sensors", action: "ale"},
       caption: "tr.flow.ale_sensors", html: {class: "block-ui-but"}
  menu :sensors_view_menu, :arubacentral_view,
       {controller: "sensors", action: "arubacentral"},
       caption: "tr.flow.arubacentral_sensors", html: {class: "block-ui-but"}
  menu :sensors_view_menu, :device_view,
      {controller: "sensors", action: "device"},
      caption: "tr.flow.device_sensors", html: {class: "block-ui-but"}
  menu :sensors_view_menu, :meraki_view,
       {controller: 'sensors', action: 'meraki'},
       caption: 'tr.flow.meraki_sensors', html: { class: 'block-ui-but' }
  end

  menu :user_home_page_collection_menu, :Traffic, {}, caption: "/flow"
  menu :widget_sensor_table_type_menu, :table_flow, {}, caption: "Table Flow"
  menu :segments_menu, :flow, { controller: :monitor, action: :more_segments, datasource: "rb_flow"}, caption: "Flow", html: {"data-remote" => true, "data-source" => "rb_flow"}
  menu :segments_menu,
       :state,
       { controller: :monitor,
         action: :more_segments,
         datasource: 'rb_state' },
       caption: "tr.flow.state",
       html: { 'data-remote' => true,
               'data-source' => 'rb_state' }

  menu :worker_menu, :mac_finder,
       { controller: :jobs, action: :start_mac_finder },
       html: { method: :post },
       caption: "tr.flow.start",
       if: proc { !DelayedJob.mac_finder? }
  menu :worker_menu, :assets_inventory,
     { controller: :jobs, action: :start_assets_inventory },
     html: { method: :post },
     caption: "tr.flow.inventory",
     if: proc { !DelayedJob.assets_inventory? }
  menu :worker_menu, :zabbix_inventory_job,
      { controller: :jobs, action: :start_zabbix_inventory_job },
      html: { method: :post },
      caption: "tr.flow.zabbix",
      if: proc { !DelayedJob.zabbix_inventory_job? && Integration.zabbix_params_present? }

end

# extend DRUID_CONFIG
druid_flow_config = YAML.load(File.read("#{Rails.root}/lib/modules/flow/config/rbdruid_config.yml"))[Rails.env].symbolize_keys
RedBorder::DRUID_CONFIG.merge! druid_flow_config

require_dependency 'PrimeInfrastructure/functions'

require_dependency 'sensor_flow'

require_dependency 'sensor_mse'
require_dependency 'sensor_meraki'
require_dependency 'sensor_ale'
require_dependency 'sensor_arubacentral'
require_dependency 'sensor_device'

require_dependency 'widget_flow'
require_dependency 'report_block_flow'
require_dependency 'alarm_flow'
require_dependency 'delayed_job_flow'
require_dependency 'check_corrupt_data_job_flow'

DittoCode::Exec.if 'ENTERPRISE' do
require_dependency 'license_flow'
end

# Require controllers
require_dependency 'sensors_controller_flow'
require_dependency 'widgets_controller_flow'
require_dependency 'application_controller_flow'
require_dependency 'jobs_controller_flow'

# Require hooks
require_dependency 'hooks/sensors_edit_links_hook'
require_dependency 'hooks/alarms_flow_hook'
require_dependency 'hooks/widget_data_flow_hook'

# Require helpers
require_dependency 'sensors_helper_flow'
require_dependency 'widgets_helper_flow'
require_dependency 'access_points_helper'
require_dependency 'application_helper_flow'

# Require jobs
require_dependency 'check_access_points_status_job'
require_dependency 'mac_finder_job'
require_dependency 'assets_inventory_job'
require_dependency 'zabbix_inventory_job'

to_prepare = Proc.new do
  unless JobsController.include?(RedBorder::JobsControllerFlow)
    JobsController.send(:include, RedBorder::JobsControllerFlow)
  end
  unless Redborder::Jobs::CheckCorruptDataJob.include?(RedBorder::CheckCorruptDataJobFlow)
    Redborder::Jobs::CheckCorruptDataJob.send(:prepend, RedBorder::CheckCorruptDataJobFlow)
  end
  unless SensorsController.include?(RedBorder::SensorsControllerFlow)
    SensorsController.send(:include, RedBorder::SensorsControllerFlow)
  end
  unless WidgetsController.include?(RedBorder::WidgetsControllerFlow)
    WidgetsController.send(:include, RedBorder::WidgetsControllerFlow)
  end
  unless ApplicationController.include?(RedBorder::ApplicationControllerFlow)
    ApplicationController.send(:include, RedBorder::ApplicationControllerFlow)
  end
  unless SensorsHelper.include?(RedBorder::SensorsHelperFlow)
    SensorsHelper.send(:include, RedBorder::SensorsHelperFlow)
  end
  unless WidgetsHelper.include?(RedBorder::WidgetsHelperFlow)
    WidgetsHelper.send(:include, RedBorder::WidgetsHelperFlow)
  end
  unless ApplicationHelper.include?(RedBorder::ApplicationHelperFlow)
    ApplicationHelper.send(:include, RedBorder::ApplicationHelperFlow)
  end
end

Rails.configuration.to_prepare(&to_prepare)
