#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class VaultController < Redborder::Commons::CommonController
  # Storage of the filters
  include Redborder::Filters::Storage::Session
  # Includes
  include Redborder::Rbdruid

  self.view_paths = 'lib/modules/vault/app/views'
  layout "#{Rails.root}/app/views/layouts/application"
  prepend_view_path "#{Rails.root}/app/views"

  helper_method :sort_column, :sort_direction

  ## ======================================================================
  # => Before actions and filters
  ## ======================================================================

  before_action :set_saved_vault_filters,
                only: [:index, :tops, :comparative,
                       :cardinality, :export_raw_to_file]

  # -------------------------------------------------------------
  # Actions
  # -------------------------------------------------------------

  def export_info
    filename = in_time_machine { Time.now.strftime('%Y%m%d%H%M%S') +"-Details.json" }

    opts = params[:element].except(:timestamp, :events)
                           .delete_if { |k, v| v.blank? }
                           .map { |k, v| " -b " + Base64.urlsafe_encode64("#{k.to_s}=#{v.to_s}") }.join

    time = params[:element][:timestamp].to_time.utc
    comd = "sudo /opt/rb/bin/rb_get_raw_vault.sh -Y #{time.strftime('%Y')} "\
           "-M #{time.strftime('%m')} -D #{time.strftime('%d')} "\
           "-H #{time.strftime('%H')} -m #{time.strftime('%M')} #{opts}"

    content = `#{comd}`

    send_data content, filename: filename
  end

  # -------------------------------------------------------------
  # Private methods
  # -------------------------------------------------------------

  private

  #
  # Set the filters. Change current filters if params[:search] is defined.
  #
  def set_source
    @source = Redborder::DruidSource.get(source: 'vault')
    @filter = Redborder::Filter.new(@source, load_filters)
  end

  def set_saved_vault_filters
    @filters = SavedVaultFilter.where(user_id: current_user.id)
    @filters_public = SavedVaultFilter.where
                      .not(user_id: current_user.id).select(&:public)
  end
end
