#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder

    module Jobs
  
      class DeleteStoredJobs < Struct.new(:user_id, :verbose)
        include Redborder::Jobs::JobHelper
  
        def perform
          begin
            cutoff_date = calculate_date
            content = ("Auto delete Stored jobs older than '#{cutoff_date.to_s}':\n")
            content += ("Auto delete stored jobs daily.\n")
            content += ("To change how many days of stored jobs to keep:\n")
            content += ("Modify General Settings > General Parameters > Days to keep Stored jobs")
            autodelete(cutoff_date)
          rescue Exception => e
            content += "#{e.class}: #{e.message}\n"
            e.backtrace.each do |line|
              content += "#{line}\n"
            end
            logit "#{e.class}: #{e.message}" if verbose
          ensure
            StoredDelayedJob.create(job: 'DeleteStoredJobs', user_id: user_id, content: content)
          end
  
        end
  
        private
  
        def calculate_date
          duration = Setting.find_by(name: "autodelete_stored_jobs_duration").value.to_i
          cutoff_date = Time.current - duration.days - 5.minutes
          cutoff_date
        end
  
        def autodelete(cutoff_date)
          jobs_to_delete = StoredDelayedJob.where("created_at <= '#{cutoff_date}'")
          if jobs_to_delete.exists?
            jobs_to_delete.destroy_all
          end
          Delayed::Job.enqueue(self.class.new(user_id, true), 
            priority: 10, run_at: 1.day.from_now.beginning_of_day)        
        end
      end
  
    end
  
  end
  