;(function (window, undefined) { "strict mode";
window.RB = window.RB || {};
RB.Common = RB.Common || {};

RB.Common.palette = [
  "#638bdf", "#f26e57", "#89a54e", "#e89c4f", "#946acc",
  "#1758b5", "#c74531", "#3e812d", "#d87e34", "#734ca2",
  "#63a0e7", "#f4988a", "#a2bb77", "#edbc6c", "#be90fd",
  "#0f3742", "#783234", "#55633f", "#fdc52b", "#371c39",
  "#38606b", "#996062", "#6f795f", "#878787", "#8a3b8f",
  "#5d8d9a", "#ae9596", "#a8b298", "#bcbcbc", "#eb80f1"
];

RB.Common.color = d3.scale.ordinal().range(RB.Common.palette);

RB.Serie = function (options) {
  var self = this, brushGroup,
    restview = 0, w, h;

  self.options = {
    container:           options.container,
    width:               options.width,
    height:              options.height,
    margin: {
      top:               RB.util.default((options.margin || {}).top, 5),
      bottom:            RB.util.default((options.margin || {}).bottom, 5),
      right:             RB.util.default((options.margin || {}).right, 5),
      left:              RB.util.default((options.margin || {}).left, 5)
    },
    yLimits:             RB.util.default(options.yLimits, []),
    navchart:            RB.util.default(options.navchart, false),
    interpolation:       RB.util.default(options.interpolation, "linear"),
    autoscaling:         RB.util.default(options.autoscaling, false),
    units:               RB.util.default(options.units, ""),
    step:                RB.util.default(options.step, -1),
    genericXAxis:        RB.util.default(options.genericXAxis, false),
    yzoom:               RB.util.default(options.yzoom, true),
    fillWith:            RB.util.default(options.fillWith, 0),
    maxPoints:           RB.util.default(options.maxPoints, 300),
    timezone:            RB.util.default(options.timezone, "auto"),
    tooltip:             RB.util.default(options.tooltip, true),
    show_value_always:   RB.util.default(options.show_value_always, false),
    legend:              RB.util.default(options.legend, false),
    legend_position:     RB.util.default(options.legend_position, 'bottom'),
    zoomable:            RB.util.default(options.zoomable, true),
    loading:             RB.util.default(options.loading, true),
    hide_axises:         RB.util.default(options.hide_axises, false),
    hide_y_axis:         RB.util.default(options.hide_y_axis, false),
    discrete_y_axis:     RB.util.default(options.discrete_y_axis, false),
    tick_y_count:        RB.util.default(options.tick_y_count, 5),
    hide_x_axis:         RB.util.default(options.hide_x_axis, false),
    autorender:          RB.util.default(options.autorender, true),
    predict_area:        RB.util.default(options.predict_area, false),
    shared_colors:       RB.util.default(options.shared_colors, false),
    palette:             RB.util.default(options.palette, undefined),
    stroke:              RB.util.default(options.stroke, 3),
    dataFunction:        options.dataFunction,
    onZoomEvent:         options.onZoomEvent,
    onZoomResetEvent:    options.onZoomResetEvent,
    tooltip_date_format: d3.time.format(options.tooltip_date_format || "%a, %H:%M - %d/%m/%Y"),
    multiplier:          options.multiplier
  };

  self.elements         = [];
  self.bisect           = d3.bisector(function (d) { return d; }).right;
  self.stack            = d3.layout.stack().values(function (d) { return d.values; });
  self.multiplier       = 1;
  self.maxDomain        = [-1, -1];
  self.chartid          = Math.random().toString().substring(2, 7);
  self.render           = self.options.autorender;
  self.multiplierPrefix = "";
  self.maximized        = "";
  self.margin           = {};
  self.color            = d3.scale.ordinal().range(RB.Common.palette);
  self.outliers         = false;
  self.tickYFormatter   = function (d) {
                            if (self.options.discrete_y_axis === false ||
                              (self.options.discrete_y_axis && d % 1 === 0)) {
                                return d + " " + self.multiplierPrefix + self.options.units;
                            }
                          };

  // Visible domain
  if (options.domain) {
    self.domain = [];
    if (self.options.timezone === "auto") {
      self.domain[0] = new Date(options.domain[0] * 1000);
      self.domain[1] = new Date(options.domain[1] * 1000);
    } else {
      self.domain[0] = new Date(new Date(options.domain[0] * 1000).getTime() +
        self.options.timezone + new Date().getTimezoneOffset() * 60000);
      self.domain[1] = new Date(new Date(options.domain[1] * 1000).getTime() +
        self.options.timezone + new Date().getTimezoneOffset() * 60000);
    }
  } else {
    self.domain = [-1, -1];
  }

  // Update the created time of this chart, since we just created it
  self.reloadCreatedTime();

  // Scale functions
  self.x = d3.time.scale();
  self.y = d3.scale.linear();

  // Axises
  if (self.options.hide_axises === false) {

    if(self.options.hide_x_axis === false){
      self.xAxis = d3.svg.axis().scale(self.x)
        .orient("bottom")
        .tickFormat(function(d) {
          var diff = self.x.domain()[1] - self.x.domain()[0],
          using = d3.time.format,
          format;

          if (diff < 259200000) {
            format = using("%H:%M")(d);
          } else if (diff < 1036800000) {
            format = using("%a")(d);
          } else if (diff < 3456000000) {
            if (self.options.genericXAxis) {
              format = using("Day %d")(d);
            } else {
              format = using("%d %b")(d);
            }
          } else {
            format = using("%b")(d);
          }

          return format;
        });
    }

    if(self.options.hide_y_axis === false){
      self.yAxis = d3.svg.axis().scale(self.y)
        .tickFormat(self.tickYFormatter)
        .ticks(self.options.tick_y_count)
        .orient("left");
    }
  }

  // Lines and areas
  self.area = d3.svg.area()
    .interpolate(self.options.interpolation)
    .x(function (d) { return self.x(d.timestamp); })
    .y0(function (d) {
      var ret = 0;
      if (d.y0 !== undefined) {
        ret = d.y0;
      }
      return self.y(self.multiplier * ret) + 1;
    })
    .y1(function (d) {
      var scale = d.y;
      if (d.y0 !== undefined) {
        scale += d.y0;
      }
      return self.y(self.multiplier * scale) + 1;
    })
    .defined(function (d) {
      return d.y !== undefined;
    });

  self.line = d3.svg.area()
    .interpolate(self.options.interpolation)
    .x(function (d) {
      return self.x(d.timestamp);
    })
    .y(function (d) {
      var scale = d.y;
      if (d.y0 !== undefined) {
        scale += d.y0;
      }
      return self.y(self.multiplier * scale) + 1;
    })
    .defined(function (d) {
      return d.y !== undefined;
    });

  // Brush
  self.brush = d3.svg.brush()
    .x(self.x)
    .on("brushend", function () {
      self._onMainBrushEnd(this);
    });

  if (self.options.navchart) {
    // Navigation chart scale functions
    self.x2 = d3.time.scale();
    self.y2 = d3.scale.linear();

    // Navigation chart brush
    if (self.options.zoomable) {
      self.navChartBrush = d3.svg.brush()
        .x(self.x2)
        .on("brushend", function () {
          self._onNavChartBrushChange();
        });
    }

    // Navigation chart axis
    if (self.options.hide_axises === false && self.options.hide_x_axis === false) {
      self.xAxis2 = d3.svg.axis().scale(self.x2).orient("bottom");
    }

    // Navigation chart areas
    self.area2 = d3.svg.area()
      .interpolate(self.options.interpolation)
      .x(function (d) {
        return self.x2(d.timestamp);
      })
      .y0(function () {
        return self.y2(0);
      })
      .y1(function (d) {
        return self.y2(d.y);
      });

    self.line2 = d3.svg.line()
      .interpolate(self.options.interpolation)
      .x(function (d) {
        return self.x2(d.timestamp);
      })
      .y(function (d) {
        return self.y2(d.y);
      });
  }

  if (self.render) {
    self._initialize();
  }
};

RB.Serie.prototype = {
  /* Public general methods */
  // Adds a new serie
  add: function (options) {
    var self = this,
      calculatedStep = 0,
      converted = {};

    // Multiple clicks can call multiple adds. Ignore after the first one = do it only once.
    if (!self.addedSeries) self.addedSeries = new Set();  //This set registers each Serie
  
    if (self.addedSeries.has(options.name)) return;       //Series has been already added

    self.addedSeries.add(options.name);                   //Register the Serie
  
    // Hide loading msg automatically
    self.options.loading = false;

    // Add only if there is 1+ points
    if (options.data !== undefined && options.data.length > 0) {
      // Read, parse and format the data
      converted = self._buildData(options);

      // Save the point step if necessary
      calculatedStep = RB.util.findStep(converted);
      if ((calculatedStep !== -1 && calculatedStep < self.options.step) ||
        (calculatedStep !== -1 && self.options.step === -1)) {
        self.options.step = calculatedStep;
      }

      if (self.options.step > 0) {
        // Append to the series array
        if(options.visible === false){
          // Don't show it
          converted.visible = false;
          self.elements.push(converted);
          return;
        } else {
          self.elements.push(converted);
        }

        // Unmaximize if there is a selected area
        self._unmaximize();

        // Update domains
        self._updateXDomain();

        // Append zeros to all the series
        self._fixMissingDataToAll();

        // Stack if necessary
        if (converted.type === "stacked" || converted.type === "sbars") {
          self._stackAll();
        }

        // Update Y Domain
        self._updateYDomain();

        // Draw it
        self._drawAll();
        self._updateAxis();
        self._drawLegend();
      } else {
        throw new Error("Invalid step");
      }
    }
  },
  // Add points to a serie, from a given array of timestamps
  add_points: function (options) {
    var self = this;
    if (options.data !== undefined && options.data.length > 0) {
      var start, stop, d, fecha, xcoord, circles, name = RB.util.cleanName(options.name);

      // Fast exit. We cant build a range if we havent got a domain
      if (self.maxDomain[0] === -1 || self.maxDomain[1] === -1) return;

      $.each(options.data, function( index, value ) {
        // Date where the selected index is. Make timestamp compatible with different javascript parsers
        date = new Date(value.timestamp.replace(" ", "T").replace(" ", ""));
        fecha = new Date(date.toISOString());

        // X coord of the selected point
        xcoord = self.x(fecha.getTime());

        // Which of the elements must be drawn?
        drawableSeries = self.elements.filter(function (d) {
          var index = d.values.map(function (x) {
            return x.timestamp.getTime();
          }).indexOf(fecha.getTime());

          if (index !== -1 && d.name === name && (d.values[index].y !== 0 ||
            (d.type !== "stacked" && d.type !== "sbars")) &&
            !d.hidden && d.values[index].y !== undefined) {
            return true;
          } else return false;
        });

        // Draw circles
        circles = self.semitop.selectAll("circle.rb-point-" + name + "-" + index.toString())
          .data(drawableSeries, function (d) {
            return d.name;
          });

        circles.enter()
          .append("circle")
          .attr("r", "4.5")
          .attr("stroke-width", 3.0)
          .attr("stroke", function (d) { return d3.rgb(d.color || self._getColor(d.realName)).darker(0.7); })
          .attr("fill", "#E17171")
          .attr("timestamp", value.timestamp)
          .attr("expected", value.expected)
          .attr("class", "rb-point-" + name + "-" + index.toString());

        circles
          .attr("cx", xcoord)
          .attr("cy", function (d) {
            var index, ret;

            index = d.values.map(function (x) {
              return x.timestamp.getTime();
            }).indexOf(fecha.getTime());

            ret = self.y(self.multiplier * d.values[index].y);

            if ((d.type === "stacked" || d.type === "sbars") && d.values[index].y0 !== undefined) {
              ret = self.y(self.multiplier * (d.values[index].y + d.values[index].y0));
            }

            return ret;
          });

        if (drawableSeries[0] !== undefined && !drawableSeries[0].visible) {
          $('circle[class^="rb-point-' + name + '"]').hide();
        }
        var activeTab = d3.select('.tab-content .tab-pane.active');
        var tooltips = activeTab.selectAll('.tooltip');
        tooltips.node().parentNode.insertBefore(circles.node(), tooltips.node().nextSibling);
      });

      self.outliers = true;
    }
  },
  // Removes a serie
  remove: function (name) {
    var self = this,
      selected;

    // Make serie addable again
    if(self.addedSeries) self.addedSeries.delete(name);

    // Clean the name
    name = RB.util.cleanName(name);

    // Look for it and remove it from the elements array
    self.elements.forEach(function (d, i) {
      if (d.name === name) {
        selected = d;
        self.elements.splice(i, 1);
      }
    });

    // If self serie doesnt exists, throw an error
    if (selected === undefined) {
      //throw("That serie doesn't exists in this chart");
    } else {

      // Remove it from the DOM
      self.svg.selectAll("g[data-name='" + name + "']")
        .remove();

      // If it was an stacked serie, recompute the stacks
      if (self.elements.length > 0 && (selected.type === "stacked" || selected.type === "sbars")) {
        self._stackAll();
      }

      // Draw all again
      self._unmaximize();
      self._updateXDomain();
      self._updateYDomain();
      self._drawAll();
      self._updateAxis();
    }
  },
  // Hides a serie
  hide: function (name) {
    var self = this,
      selected, selected_predicted;

    // Clean the name
    name = RB.util.cleanName(name);

    // Search it
    self.elements.forEach(function (d) {
      if (d.name === name) {
        selected = d;
      }
      if (d.name === name+"_predicted") {
        selected_predicted = d;
      }
    });

    // If self serie doesnt exists, throw an error
    if (selected === undefined) {
      //throw("That serie doesn't exists in this chart");
    } else {

      // Hide it
      selected.visible = false;
      if (selected_predicted !== undefined) {
        selected_predicted.visible = false;
      }
      $('circle[class^="rb-point-' + name + '"]').hide(1000);

      // If it was a stacked serie, recompute the stack
      if (selected.type === "stacked" || selected.type === "sbars") {
        self._stackAll();
      }

      self._unmaximize();
      self._updateXDomain();
      self._updateYDomain();
      self._updateAxis();
      self._drawAll();
    }
  },
  // Unhides a serie
  unhide: function (name) {
    var self = this,
      selected, selected_predicted;

    // Clean the name
    name = RB.util.cleanName(name);

    // Search it
    self.elements.forEach(function (d) {
      if (d.name === name) {
        selected = d;
      }
      if (d.name === name+"_predicted") {
        selected_predicted = d;
      }
    });

    // If self serie doesnt exists, throw an error
    if (selected === undefined) {
      //throw("That serie doesn't exists in this chart");
    } else {

      // Make it hide
      selected.visible = true;
      if ( selected_predicted !== undefined) {
        selected_predicted.visible = true;
      }
      $('circle[class^="rb-point-' + name + '"]').show(1000);

      // If it was a stacked serie, recompute the stack
      if (selected.type === "stacked" || selected.type === "sbars") {
        self._stackAll();
      }

      self._unmaximize();
      self._updateXDomain();
      self._updateYDomain();
      self._updateAxis();
      self._drawAll();
    }
  },
  // Appends data to a serie.
  append: function (options, remove_from, onFinish) {
    var newValues = {},
      self = this,
      flag = false,
      selected, temp;

    // Read, parse and format the data
    options.forEach(function (element) {
      newValues = self._buildData(element).values;

      // Search it and save it for later
      self.elements.forEach(function (d) {
        if (d.name === RB.util.cleanName(element.name)) {
          selected = d;
        }
      });

      if (selected !== undefined) {
        // Middle vars
        temp = newValues.map(function (element) {
          return element.timestamp.getTime();
        });

        selected.values = selected.values.filter(function (element) {
          return (temp.indexOf(element.timestamp.getTime()) === -1);
        });

        // Append the new values
        selected.values = selected.values.concat(newValues);

        // Sort it again
        selected.values.sort(function (a, b) {
          return d3.ascending(a.timestamp, b.timestamp);
        });

        // Stack if necessary
        if (selected.type === "stacked" || selected.type === "sbars") {
          flag = true;
        }
      }
    });

    // Unmaximize to avoid drawing errors
    self._unmaximize();

    // Stack it
    self._fixMissingDataToAll();
    if (flag === true) {
      self._stackAll();
    }

    // Draw it
    self._drawAll(true, function () {
      var to_remove = self._updateXDomain(remove_from);

      self._updateYDomain();
      self._updateAxis();
      self._drawAll(false, function () {
        if (to_remove.length === 2) {
          self._removeRange(to_remove[0], to_remove[1]);
        }

        self._drawAll(true, onFinish);
      });
    });
  },
  // Resizes the width and height of the chart
  resize: function (width, height) {
    var self = this, w, h, brushGroup;

    // Hide the SVG
    if (self.svg !== undefined) {
      self.svg.style("display", "none");
    }

    // Get width and height defaults
    w = width || d3.select("#" + self.options.container).style("width");
    h = height || d3.select("#" + self.options.container).style("height");
    w = parseInt(w, 10);
    h = parseInt(h, 10);

    // Set default margin
    self.margin.top = RB.util.default((self.options.margin || {}).top, 10);
    self.margin.bottom = RB.util.default((self.options.margin || {}).bottom, 10);
    self.margin.right = RB.util.default((self.options.margin || {}).right, 10);
    self.margin.left = RB.util.default((self.options.margin || {}).left, 10);

    // If we are showing axises, add extra bottom margin
    if (self.options.hide_axises === false && self.options.hide_x_axis === false) {
      self.margin.bottom += 16;
    }

    // If there is a navigation chart, add margin to the main chart
    if (self.options.navchart) {
      self.margin.bottom += 90;
    }

    // Add margin to the bottom if we display the legend
    if (self.options.legend) { self.margin.bottom += 25; }

    // Default height is width/5, if not specified in CSS
    if (isNaN(h) || h === 0) {
      if (self.options.navchart) {
        h = (w / 5) + 60 + self.margin.top;
      } else {
        h = (w / 5);
      }

      if (h < 200) {
        h = 200;
      }
    }

    // Calculate width and height
    self.width = w - self.margin.left - self.margin.right;
    self.height = h - self.margin.top - self.margin.bottom;

    // Resize scale functions
    self.x.range([0, self.width]);
    self.y.range([self.height, 12]);

    // Check if we have a SVG defined before continue
    if (self.svg === undefined) { return; }

    // Update y axis ticks
    if (self.options.hide_axises === false && self.options.hide_y_axis === false) {
      self.yAxis.tickSize(-this.width, 0, 0);
    }

    if (self.options.navchart) {
      // Navigation chart height
      self.height2 = 60;

      // Navigation chart scale functions
      self.x2.range([0, self.width]);
      self.y2.range([self.height2, 0]);
    }

    // Resize SVG elements
    // Main elements
    self.svg
      .attr("width", self.width + self.margin.left + self.margin.right)
      .attr("height", self.height + self.margin.top + self.margin.bottom)
      .style("display", "block");

    d3.select("#rbcharts-clip-" + self.chartid + " rect")
      .attr("width", self.width)
      .attr("height", self.height + 1);

    // Charts group
    self.focus
      .attr("transform", "translate(" + self.margin.left + "," + self.margin.top + ")");

    // Sub chart elements
    if (self.options.navchart) {
      // Navigation chart group
      self.context
        .attr("transform", "translate(" + self.margin.left + "," + (self.height + 35) + ")");
    }

    // Semi-top group (tooltip, etc)
    self.semitop
      .attr("transform", "translate(" + self.margin.left + "," + self.margin.top + ")");

    // Main chart axises
    if (self.options.hide_axises === false) {
      if (self.options.hide_x_axis === false){
        self.semitop.select(".x.axis")
          .attr("transform", "translate(0," + self.height + ")")
          .call(self.xAxis);
      }

      if (self.options.hide_y_axis === false) {
        self.semitop.select(".y.axis")
          .call(self.yAxis);
      }
    }

    if (self.options.navchart && self.options.hide_axises === false && self.options.hide_x_axis === false) {
      // Draw navigation chart x axis
      self.semitop.select(".x2.axis")
        .attr("transform", "translate(0," + (self.height + self.margin.top + self.height2 + 25) + ")")
        .call(self.xAxis2);
    }

    if (self.options.predict_area === true) {
      self.predict.select("rect")
        .attr("height", self.height + 1);

      self.predict.select("line")
        .attr("y2", self.height + 1);
    }

    if(self.options.tooltip){
      // Always-on-top group
      self.onTop
        .attr("transform", "translate(" + self.margin.left + "," + self.margin.top + ")");

      brushGroup = self.onTop.select("g .x.brush")
        .call(self.brush);

      if (self.options.zoomable === false) {
        brushGroup.selectAll("rect").style("cursor", "pointer", "important");
        brushGroup.on('touchstart.brush', null);
        brushGroup.on('mousedown.brush', null);
      }

      brushGroup.selectAll("rect")
        .attr("height", self.height + 1);

      if (self.options.navchart) {
        brushGroup = self.onTop.selectAll(".x2.brush")
          .call(self.navChartBrush);

        brushGroup.selectAll("rect")
          .attr("y", self.height + 26)
          .attr("height", self.height2 + 5);

        if (self.options.zoomable === false) {
          brushGroup.selectAll("rect").style("cursor", "pointer", "important");
          brushGroup.on('touchstart.brush', null);
          brushGroup.on('mousedown.brush', null);
        }
      }

      // Restart view rectangle
      self.onTop.select("g.reset-button")
        .attr("transform", "translate(" + (self.width - 40) + ", 1)");
    }

    // Draw the chart
    self._updateYDomain();
    self._drawAll(true);
    self._updateAxis();
    self._drawLegend();
  },
  // Returns the color of a serie
  getColor: function (name) {
    var self = this,
      cleanedName = RB.util.cleanName(name),
      ret;

    self.elements.forEach(function (d) {
      if (d.name === cleanedName) {
        ret = d.color || self._getColor(d.realName);
      }
    });

    // If self serie doesnt exists, throw an error
    if (ret === undefined) {
      throw("That serie doesn't exists in this chart");
    }

    return ret;
  },
  // Returns the domain of a serie in miliseconds since unix epoch
  currentDomain: function () {
    var self = this,
      domain = self.x.domain();

    domain[0] = domain[0].getTime();
    domain[1] = domain[1].getTime();

    return domain;
  },
  // Forces a X Domain
  forceDomain: function (domain, redraw) {
    var self = this;

    // Round domain to step
    var coeff = self.options.step;
    domain[0] = new Date(Math.ceil(domain[0] / coeff) * coeff);
    domain[1] = new Date(Math.floor(domain[1] / coeff) * coeff);

    self.domain = domain;
    self.x.domain(domain);

    if (redraw || redraw === undefined){
      self._updateYDomain();
      self._updateAxis();
      self._drawAll();
    }
  },
  // Changes the type of a serie
  changeType: function (name, newType) {
    var self = this,
      selected, prevType;

    // Check if the newType is acceptable
    if(newType === "area" || newType === "line" ||
      newType === "stacked" || newType === "bars" ||
      newType === "sbars") {

      // Look for it and save the reference for later
      self.elements.forEach(function (d) {
        if (d.name === name) {
          selected = d;
        }
      });

      // If self serie doesnt exists, throw an error
      if (selected === undefined) {
        throw("That serie doesn't exists in this chart");
      }

      // Change the type
      prevType = selected.type;
      selected.type = newType;

      // Remove the stack baseline if it was an stacked area
      if(prevType === "stacked" || prevType === "sbars") {
        selected.values.forEach(function (d) {
          delete d.y0;
        });
        self._stackAll();
      }

      // Stack if it will be an stacked area
      if(newType === "stacked" || newType === "sbars") {
        self._stackAll();
      }

      // Update the chart
      self._updateYDomain();
      self._updateAxis();
      self._drawAll();
    } else {
      throw("Unacceptable type");
    }
  },
  // Changes the step of the whole chart
  changeStep: function (newStep) {
    // Change the step
    var self = this;

    self.options.step = newStep;

    // Update the chart
    self._updateXDomain();
    self._updateYDomain();
    self._updateAxis();
    self._drawAll();
  },
  // Render if autorender was specified
  startRender: function () {
    var self = this;

    if (!self.options.autorender && self.render === false) {
      self.render = true;
      self._initialize();
    }
  },
  reloadCreatedTime: function () {
    var self = this;

    if (self.options.timezone === "auto") {
      self.created_time = new Date();
    } else {
      self.created_time = new Date(new Date().getTime() + self.options.timezone + new Date().getTimezoneOffset() * 60000);
    }
  },
  /* Drawing methods */
  // Draw the svg and initialize the group elements
  _initialize: function () {
    var self = this,
      brushGroup, restview;

    // Main elements
    self.svg = d3.select("#" + self.options.container).append("svg")
      .attr("class", "rbcharts");

    self.svg.append("defs").append("clipPath")
      .attr("id", "rbcharts-clip-" + self.chartid)
      .append("rect");

    // Charts group
    self.focus = self.svg.append("g")
      .attr("id", "rbcharts-" + self.chartid + "-main");

    // Sub chart elements
    if (self.options.navchart) {
      // Create navigation chart group
      self.context = self.svg.append("g")
        .attr("clip-path", "url(#rbcharts-clip-" + self.chartid + ")");
    }

    // Semi-top group (tooltip, etc)
    self.semitop = self.svg.append("g")
      .attr("id", "rbcharts-" + self.chartid + "-semitop");

    // Main chart axises
    if (self.options.hide_axises === false) {
      if (self.options.hide_x_axis === false){
        self.semitop.append("g")
          .attr("class", "x axis");
      }

      if (self.options.hide_y_axis === false){
        self.semitop.append("g")
          .attr("class", "y axis");
      }
    }

    if (self.options.navchart && self.options.hide_axises === false && self.options.hide_x_axis === false) {
      // Draw navigation chart x axis
      self.semitop.append("g")
        .attr("class", "x2 axis");
    }

    if (self.options.predict_area === true) {
      self.predict = self.semitop.append("g")
        .attr("clip-path", "url(#rbcharts-clip-" + self.chartid + ")");

      self.predict.append("rect")
        .attr("class", "predict")
        .attr("y", 0)
        .attr("width", 9999);

      self.predict.append("line")
        .attr("class", "predict")
        .attr("y1", 0);

      self.predict.append("text")
        .attr("class", "predict")
        .attr("y", 5)
        .text("Predicted");
    }

    // Line where the point is
    self.semitop.append("line")
      .attr("x1", 0)
      .attr("y1", -99999)
      .attr("x2", 0)
      .attr("y2", -99999)
      .attr("class", "tooltip");

    // Tooltip group
    if(self.options.tooltip){

      self.tooltip = self.semitop.append("g")
        .attr("class", "tooltip");

      // Append the container rectangle
      self.tooltip.append("rect")
        .attr("x", 0)
        .attr("y", 0)
        .attr("class", "rect-container");

      // Append the date where the mouse is
      self.tooltip.append("text")
        .attr("x", 12)
        .attr("y", 20)
        .attr("class", "date");

      // Always-on-top group
      self.onTop = self.svg.append("g")
        .attr("id", self.options.container + "-ontop");

      // Draw chart brush
      brushGroup = self.onTop.append("g")
        .attr("class", "x brush")
        .call(self.brush)
        .on("mousemove", function () {
          self._onMouseMove(d3.mouse(this));
        })
        .on("mouseout", function () {
            self._onMouseOut(d3.mouse(this));
        });
     

      brushGroup.selectAll("rect")
        .attr("y", 0);

      if (self.options.zoomable === false) {
        brushGroup.selectAll("rect").style("cursor", "pointer", "important");
        brushGroup.on('touchstart.brush', null);
        brushGroup.on('mousedown.brush', null);
      }

      if (self.options.navchart) {
        // Draw navigation chart brush
        brushGroup = self.onTop.append("g")
          .attr("class", "x2 brush")
          .call(self.navChartBrush);

        brushGroup.selectAll("rect")
          .attr("height", self.height2 + 5);

        if (self.options.zoomable === false) {
          brushGroup.selectAll("rect").style("cursor", "pointer", "important");
          brushGroup.on('touchstart.brush', null);
          brushGroup.on('mousedown.brush', null);
        }
      }

      // Restart view rectangle
      restview = self.onTop.append("g")
        .attr("class", "reset-button")
        .style("display", "none")
        .on("click", function () {
          // Restart domain
          if (self.maxDomain[0] !== -1 && self.maxDomain[1] !== -1) {
            self.x.domain(self.maxDomain);
          } else {
            self.x.domain(self.domain);
          }
          // Update the chart
          self._updateYDomain();
          self._updateMain();
          self._updateAxis();
          // Update the navigation chart brush
          if (self.options.navchart) {
            self.navChartBrush.clear();
            self.onTop.select(".x2.brush .extent").attr("width", 0);
          }
          // Hides itself
          restview.style("display", "none");
          // Fire event
          if (self.options.onZoomResetEvent !== undefined) {
            self.options.onZoomResetEvent(self.options.container);
          }
        });

      restview.append("rect")
        .attr("x", -23)
        .attr("y", 4)
        .attr("width", 55)
        .attr("height", 21);

      restview.append("text")
        .attr("x", 5)
        .attr("y", 18)
        .style("text-anchor", "middle")
        .text("Reset");
    }

    if (self.options.yLimits.length > 0) {
      self.ylimits = self.semitop.append("g")
        .attr("class", "y limits");
    }

    // Create a legend at the position defined by the user.
    // Now, only bottom position is allowed.
    if (self.options.legend) {
      self.legend = self.svg.append('g');
    }

    self.resize(self.options.width, self.options.height);
    self._updateXDomain();
    self._drawAll();
    self._updateAxis();
  },
  // Gets the color from the global space if the chart has the shared
  // properties or from its one space otherwise
  _getColor: function (key) {
    var self = this;

    if (self.options.shared_colors === true) {
      return RB.Common.color(key);
    } else if (self.options.palette !== undefined && self.options.palette[key] !== undefined) {
      return self.options.palette[key];
    } else {
      return self.color(key);
    }
  },
  // Updates X and Y axises
  _updateAxis: function () {
    var self = this,
      diff;

    if (!self.render) return;

    if (self.options.hide_axises === false) {

      if (self.options.hide_x_axis === false){
        diff = self.x.domain()[1] - self.x.domain()[0];

        if (diff > 259200000 && diff < 1036800000) {
          self.xAxis.ticks(d3.time.days, 1);
        } else if (self.width > 800) {
          self.xAxis.ticks(8);
        } else {
          self.xAxis.ticks(4);
        }

        self.semitop
          .select(".x.axis")
          .transition()
          .duration(500)
          .call(self.xAxis);

        d3.timer(function () {
          self.svg.selectAll(".x.axis g")
          .each(function (d) {
            var parts, X, Y;

            parts = /translate\(\s*([^\s,)]+)[ ,]([^\s,)]+)/.exec(d3.select(this).attr("transform"));

            if (parts !== null) {
              X = parseInt(parts[1]);
              Y = parseInt(parts[2]);

              d3.select(this)
              .selectAll("text")
              .attr("x", function (d) {
                if (X <= 15) return -X + 8;
                else if (X >= self.width - 15) return -(X - self.width) - 8;
                else return 0;
              })
              .style("text-anchor", function (d) {
                if (X <= 15) return "start";
                else if (X >= self.width - 15) return "end";
                else return "middle";
              });
            }
          });
          return true;
        }, 600);

      }

      if (self.options.yLimits.length > 0 && self.elements.length > 0) {
        var limits = self.ylimits.selectAll("line.limit")
                         .data(self.options.yLimits);

        limits.enter()
          .append("line")
          .attr("class", "limit")
          .attr("x1", 0)
          .attr("x2", self.width)
          .attr("y1", function(d) { return self.y(d * self.multiplier); })
          .attr("y2", function(d) { return self.y(d * self.multiplier); })
          .style("stroke", "rgba(255, 0, 0, 0.7)");
      }

      if (self.options.hide_y_axis === false){
        self.semitop
          .select(".y.axis")
          .transition()
          .duration(500)
          .call(self.yAxis);

        self.semitop
          .selectAll(".y.axis text")
          .attr("transform", "translate(15, 0)")
          .style("text-anchor", "start");

        self.semitop
          .selectAll(".y.axis .tick")
          .filter(function (d) { return d === 0; })
          .selectAll("text")
          .remove();
      }

      if (self.options.navchart) {
        self.semitop
          .select(".x2.axis")
          .call(self.xAxis2);
      }
    }
    self._showLastValue();
  },
  // Draws main chart and navchart if options says so
  _drawAll: function(notransition, onDrawingEnd) {
    var self = this,
      finished = 2,
      endDrawingCallback = function () {
        --finished;
        if (finished === 0) {
          if (onDrawingEnd !== undefined) onDrawingEnd();
        }
      };

    if (!self.render) return;

    self._drawLoading();

    self._updateMain(notransition, endDrawingCallback);

    if (self.options.navchart) {
      self._updateNavchart(notransition, endDrawingCallback);
    } else {
      endDrawingCallback();
    }
  },
  // Draws main chart and nav chart
  _updateMain: function (notransition, onDrawingEnd) {
    var self = this,
      areaElements = [],
      lineElements = [],
      barsElements = [],
      finished = -1,
      endDrawingCallback = function () {
        finished--;
        if (finished === 0) {
          if (onDrawingEnd !== undefined) onDrawingEnd();
        }
      };

    self.elements.forEach(function (d) {
      // Choose the area series
      if (d.visible && !d.hidden &&
        (d.showon === "main" || d.showon === "both")) {
        switch(d.type) {
          case "area":
          case "stacked":
            areaElements.push(d);
            break;
          case "line":
            lineElements.push(d);
            break;
          case "bars":
          case "sbars":
            barsElements.push(d);
            break;
        }
      }
    });

    finished = areaElements.length*2 + lineElements.length + barsElements.length;

    if (self.options.predict_area === true) self._updatePredictPosition(notransition);
    self._drawAreas(self.focus, areaElements, true, notransition, endDrawingCallback);
    self._drawLines(self.focus, lineElements, true, notransition, endDrawingCallback);
    self._drawBars(self.focus, barsElements, true, notransition, endDrawingCallback);
  },
  _updateNavchart: function (notransition, onDrawingEnd) {
    var self = this,
      areaElements = [],
      lineElements = [],
      barsElements = [],
      finished = -1,
      endDrawingCallback = function () {
        --finished;
        if (finished === 0) {
          if (onDrawingEnd !== undefined) onDrawingEnd();
        }
      };

    self.elements.forEach(function (d) {
      // Choose the area series
      if (d.visible && !d.hidden &&
        (d.showon === "navchart" || d.showon === "both")) {
        switch(d.type) {
          case "area":
          case "stacked":
            areaElements.push(d);
            break;
          case "line":
            lineElements.push(d);
            break;
          case "bars":
          case "sbars":
            barsElements.push(d);
            break;
        }
      }
    });

    finished = areaElements.length*2 + lineElements.length + barsElements.length;

    self._drawAreas(self.context, areaElements, false, notransition, endDrawingCallback);
    self._drawLines(self.context, lineElements, false, notransition, endDrawingCallback);
    self._drawBars(self.context, barsElements, false, notransition, endDrawingCallback);
  },
  _updatePredictPosition: function (notransition) {
    var self = this;

    self.predict.select("rect")
      .transition()
      .duration(function (d) { return notransition? 0 : 500; })
      .attr("x", self.x(self.created_time));

    self.predict.select("line")
      .transition()
      .duration(function (d) { return notransition? 0 : 500; })
      .attr("x1", self.x(self.created_time))
      .attr("x2", self.x(self.created_time));

    self.predict.select("text")
      .transition()
      .duration(function (d) { return notransition? 0 : 500; })
      .attr("x", self.x(self.created_time) + 10);
  },
  _drawLoading: function () {
    var self = this;

    if (!self.options.loading) {
      self.semitop
        .transition()
        .duration(500)
        .style("opacity", 1);
    } else {
      self.semitop
        .style("opacity", 0.2);
    }
  },
  // Draws every element of the chart
  _drawAreas: function (where, data, mainchart, notransition, onDrawingEnd) {
    var self = this,
      element;

    // Draw areas
    element = where.selectAll("g.area")
      .data(data, function (d) { return d.name; });

    var group = element.enter()
      .append("g")
      .attr("data-name", function (d) { return d.name; })
      .attr("opacity", 0.0001)
      .attr("class", "area");

    group.append("path")
      .attr("class", "area")
      .attr("clip-path", function() {
        if (mainchart) {
          return "url(#rbcharts-clip-" + self.chartid + ")";
        } else {
          return false;
        }
      })
      .attr("fill", function (d) {  return d.color || self._getColor(d.realName); })
      .attr("fill-opacity", 0.75);

    group.append("path")
      .attr("class", "line")
      .attr("clip-path", function() {
        if (mainchart) {
          return "url(#rbcharts-clip-" + self.chartid + ")";
        } else {
          return false;
        }
      })
      .attr("stroke", function (d) { return d3.rgb(d.color || self._getColor(d.realName)).darker(0.7); })
      .attr("stroke-dasharray", function (d) {
        var ret = "0";
        if (d.dashed) {
          ret = "8,12";
        }
        return ret;
      })
      .attr("stroke-width", function (d) {
        var ret = self.options.stroke;
        if (d.stroke) {
          ret = d.stroke;
        }
        return ret;
      })
      .attr("stroke-opacity", 0.8);

    element
      .transition()
      .duration(function (d) { return notransition? 0 : 500; })
      .attr("opacity", 1)
      .selectAll("path")
        .attr("d", function (d) {
          if (d3.select(this).attr("class") == "area") {
            if (mainchart) {
              return self.area(d.values);
            } else {
              return self.area2(d.values);
            }
          } else {
            if (mainchart) {
              return self.line(d.values);
            } else {
              return self.line2(d.values);
            }
          }
        })
        .each("end", onDrawingEnd);

    element.exit()
      .transition()
      .duration(function (d) { return notransition? 0 : 500; })
      .attr("opacity", 0.0001);
  },
  _drawLines: function (where, data, mainchart, notransition, onDrawingEnd) {
    var self = this,
      element;

    // Same here with lines
    element = where.selectAll("g.line")
      .data(data, function (d) { return d.name; });

    element.enter()
      .append("g")
      .attr("data-name", function (d) { return d.name; })
      .attr("opacity", 0.0001)
      .attr("class", "line")
      .append("path")
      .attr("class", "line")
      .attr("clip-path", function() {
        if (mainchart) {
          return "url(#rbcharts-clip-" + self.chartid + ")";
        } else {
          return false;
        }
      })
      .attr("stroke", function (d) { return d3.rgb(d.color || self._getColor(d.realName)).darker(0.7); })
      .attr("stroke-dasharray", function (d) {
        var ret = "0";
        if (d.dashed) {
          ret = "8,12";
        }
        return ret;
      })
      .attr("stroke-width", function (d) {
        var ret = self.options.stroke;
        if (d.stroke) {
          ret = d.stroke;
        }
        return ret;
      });

    element
      .transition()
      .duration(function (d) { return notransition? 0 : 500; })
      .attr("opacity", 0.75)
      .selectAll("path")
        .attr("d", function (d) {
          if (mainchart) {
            return self.line(d.values);
          } else {
            return self.line2(d.values);
          }
        })
        .each("end", onDrawingEnd);

    element.exit()
      .transition()
      .duration(function (d) { return notransition? 0 : 500; })
      .attr("opacity", 0.0001);
  },
  _drawBars: function (where, data, mainchart, notransition, onDrawingEnd) {
    var self = this,
      element, rects,
      step = self.x.domain()[0].getTime() + self.options.step,
      step2 = (self.options.navchart) ? self.x2.domain()[0].getTime() + self.options.step : "";

    // Bars areas
    element = where.selectAll("g.bars")
      .data(data, function (d) { return d.name; });

    element.enter()
      .append("g")
      .attr("class", "bars")
      .attr("clip-path", function() {
        if (mainchart) {
          return "url(#rbcharts-clip-" + self.chartid + ")";
        } else {
          return false;
        }
      })
      .attr("data-name", function (d) { return d.name; })
      .attr("fill", function (d) { return d.color || self._getColor(d.realName); })
      .attr("stroke", function (d) { return d3.rgb(d.color || self._getColor(d.realName)).darker(0.7); })
      .attr("opacity", 0.0001);

    element.transition()
      .duration(function (d) { return notransition? 0 : 500; })
      .attr("opacity", 0.75);

    rects = element.selectAll("rect")
      .data(function (d) {
        return d.values;
      }, function (d2) {
        return d2.timestamp;
      });

    rects.enter()
      .append("rect")
      .attr("stroke-width", function (d) {
        var ret = 1;
        if (d.stroke) {
          ret = d.stroke;
        }
        return ret;
      });

    rects.transition()
      .duration(function (d) { return notransition? 0 : 500; })
      .attr("x", function (d) {
        if (mainchart) {
          return self.x(d.timestamp);
        } else {
          return self.x2(d.timestamp);
        }
      })
      .attr("y", function (d) {
        if (mainchart) {
          if(d.y0 === undefined) {
            return self.y(Math.max(0, d.y * self.multiplier));
          } else {
            return self.y(Math.max(0, (d.y + d.y0) * self.multiplier));
          }
        } else {
          return self.y2(Math.max(0, d.y));
        }
      })
      .attr("width", function () {
        if (mainchart) {
          return self.x(step);
        } else {
          return self.x2(step2);
        }
      })
      .attr("height", function (d) {
        if (mainchart) {
          return Math.abs(self.y(d.y * self.multiplier) - self.y(0));
        } else {
          return Math.abs(self.y2(d.y * self.multiplier) - self.y(0));
        }
      })
      .attr("class", function (d) {
        var value = 0;
        var result = "bars";
        if (mainchart) {
          if(d.y0 === undefined) {
            value = d.y * self.multiplier;
          } else {
            value = (d.y + d.y0) * self.multiplier;
          }
        } else {
          value = d.y;
        }
        if (value < 0) result += " negative";
        return result;
      })
      .each("end", onDrawingEnd);

    element.exit()
      .transition()
      .duration(function (d) { return notransition? 0 : 500; })
      .attr("opacity", 0.0001);
  },
  // Draw the legend in the position. To create the legend set the legend option
  // to true.
  _drawLegend: function(height){
    var self = this;

    // Check the option
    if (!self.options.legend || !self.render) { return; }

    // Create the container if it doesn't exist
    if (self.legend === undefined){
      self.legend = self.svg.append('g');
    }

    // Set the height
    h = height || d3.select("#" + self.options.container).style("height");
    h = parseInt(h, 10);

    self.legend
      .attr("transform", "translate(10, " + (h - 20) + ")").html('');

    legendItems = self.legend.selectAll("rect")
      .data(self.elements, function (d) { return d.name; });

    legendItemsEnter = legendItems.enter();

    // Create the text and lines
    group = legendItemsEnter.append("g");

    group.append("rect")
      .attr("x", function(d, i) { return i*120; })
      .attr("y", 0)
      .attr("rx", 2)
      .attr("ry", 2)
      .attr("width", 15)
      .attr("height", 3)
      .attr("fill", function(d) {
        return self._getColor(d.realName);
      });

    text = group.append("text")
      .attr("class", "rb-legend")
      .attr("x", function(d, i) { return i*120 + 20; })
      .attr("y", 5)
      .text(function (d){
        if(d.name.length > 16){
          return d.name.substring(0, 14) + '...';
        }
        return d.realName || d.name;
      });

    // Delete unused elements
    legendItems.exit().remove();
  },
  _clearLegend: function() {
    this.legend.selectAll("rect").remove();
    this.legend.selectAll("text").remove();
  },

  _drawLegendX: function (i, width) {
    if ((i+1)*120 < width) {
      return i*120;
    } else {
      indent = Math.floor((i+1)*120/width);
      return i*120 - indent*120*Math.floor(width/120);
    }
  },

  _drawLegendY: function (i, width) {
    if ((i+1)*120 < width) {
      return 0;
    } else {
      indent = Math.floor((i+1)*120/width);
      return 20 * indent;
    }
  },
  /* Events */
  // Fired whenever an interval is selected on the main chart
  _onMainBrushEnd: function (context) {
    var self = this,
      extent = self.brush.extent(); // If empty, empty() returns null, idk why

    if (self.brush.empty() !== true && Math.abs(extent[0] - extent[1]) > self.options.step / 2) {
      self.x.domain(extent);

      // If there is a navchart, draw on it brush the
      // area selected on the main chart.
      if (self.options.navchart) {
        var left = self.x2(self.x.domain()[0]),
          right = self.x2(self.x.domain()[1]);

        self.navChartBrush.extent(self.x.domain());
        self.onTop.select(".x2.brush .extent")
          .attr("width", right - left)
          .attr("x", left);
      }

      // Clear the brush
      self.brush.clear();
      self.svg.select(".x.brush .extent").attr("width", 0);
      self.svg.select(".x.brush .resize").remove();

      // Show the reset button
      self.onTop.select(".reset-button")
        .style("display", "block");

      // Draw all
      self._updateYDomain();
      self._updateMain();
      self._updateAxis();

      // Fire zoom event
      if (self.options.onZoomEvent !== undefined) {
        self.options.onZoomEvent(self.options.container);
      }
    } else {
      // Clear the brush
      self.brush.clear();
      self.svg.select(".x.brush .extent").attr("width", 0);
      self.svg.select(".x.brush .resize").remove();
      self._onPathMouseClick(context);
    }
  },
  // Fired whenever an interval is selected on the nav chart
  _onNavChartBrushChange: function () {
    var self = this;

    // If empty, empty() returns null, idk why
    if (self.navChartBrush.empty() !== true) {
      self.x.domain(self.navChartBrush.extent());
    }

    // Show the reset button
    self.onTop.select(".reset-button")
      .style("display", "block");

    // Draw all
    self._updateYDomain();
    self._updateMain();
    self._updateAxis();
  },
  // Draws the tooltip
  _onMouseMove: function (mouse) {
    var self = this, start, stop, d, ind,
      fecha, xcoord, ytooltip, xtooltip,
      drawableSeries, rects, circles,
      infoText, textValue, maxTextWidth,
      rect_width = 0;

    // Fast exit. We cant build a range if we havent got a domain
    if (self.maxDomain[0] === -1 || self.maxDomain[1] === -1) return;

    // Construct the range
    start = self.maxDomain[0];
    stop = new Date(self.maxDomain[1].getTime() + self.options.step);
    d = d3.range(start, stop, self.options.step);
    ind = self.bisect(d, self.x.invert(mouse[0]));

    // Adjust the selected index, choosing the
    // closer index to the mouse position
    if (mouse[0] - self.x(d[ind - 1]) < (self.x(stop) - self.x(stop - self.options.step)) / 2) {
      ind -= 1;
    }

    // Date where the selected index is
    fecha = new Date(d[ind]);

    // Which of the elements must be drawn?
    drawableSeries = self.elements.filter(function (d) {
      var index = d.values.map(function (x) {
        return x.timestamp.getTime();
      }).indexOf(fecha.getTime());

      if (index !== -1 && (d.values[index].y !== 0 ||
        (d.type !== "stacked" && d.type !== "sbars")) &&
        d.visible && !d.hidden && d.values[index].y !== undefined) {
        return true;
      } else return false;
    });

    // X coord of the selected point
    xcoord = self.x(fecha);

    // Calculate tooltip position
    ytooltip = mouse[1] - 45;
    xtooltip = xcoord + 20;

    // Line where the point is
    self.semitop.select("line.tooltip")
      .attr("x1", xcoord)
      .attr("x2", xcoord)
      .attr("y1", 0)
      .attr("y2", this.height);

    // Write the date where the mouse is
    self.tooltip.select("text.date")
      .text(function () {
        if (drawableSeries.length > 0) return self.options.tooltip_date_format(fecha);
      });

    // Draw rectangles
    rects = self.tooltip.selectAll("rect.elements")
      .data(drawableSeries, function (d) {
        return d.name;
      });

    rects.enter()
      .append("rect")
      .attr("x", 12)
      .attr("width", 8)
      .attr("height", 8)
      .attr("rx", 2)
      .attr("ry", 2)
      .attr("fill", function (d) { return (d.color || self._getColor(d.realName)); })
      .attr("class", "elements");

    rects.attr("y", function (d, i) { rect_width = (17 * i) + 29 ; return (17 * i) + 29; });

    rects.exit().remove();

    // Draw texts
    infoText = self.tooltip.selectAll("text.elements")
      .data(drawableSeries, function (d) {
        return d.name;
      });

    infoText.enter()
      .append("text")
      .attr("x", 26)
      .attr("class", "elements");

    infoText
      .attr("y", function (d, i) { return (17  * i) + 37; })
      .text(function (d) {
        var index = d.values.map(function (x) {
          return x.timestamp.getTime();
        }).indexOf(fecha.getTime());

        if(d.realName.length < 16) {
          textValue = d.realName + ": " + Math.round(d.values[index].y * self.multiplier *
            1000) / 1000 + " " + self.multiplierPrefix + self.options.units;
        } else {
          textValue = d.realName.substr(0, 15) + "... : " + Math.round(d.values[index].y *
            self.multiplier * 1000) / 1000 + " " + self.multiplierPrefix + self.options.units;
        }

        return textValue;
      });


    infoText.exit().remove();

    if (self.render) {
      outliers = self._loadSecondaryTooltip(xcoord, rect_width);
    }

    // Draw circles
    circles = self.semitop.selectAll("circle.elements")
      .data(drawableSeries, function (d) {
        return d.name;
      });

    circles.enter()
      .append("circle")
      .attr("r", "4.5")
      .attr("stroke-width", 2.2)
      .attr("stroke", function (d) { return d3.rgb(d.color || self._getColor(d.realName)).darker(0.7); })
      .attr("fill", "white")
      .attr("class", "elements");

    circles
      .attr("cx", xcoord)
      .attr("cy", function (d) {
        var index, ret;

        index = d.values.map(function (x) {
          return x.timestamp.getTime();
        }).indexOf(fecha.getTime());

        ret = self.y(self.multiplier * d.values[index].y);

        if ((d.type === "stacked" || d.type === "sbars") && d.values[index].y0 !== undefined) {
          ret = self.y(self.multiplier * (d.values[index].y + d.values[index].y0));
        }

        return ret;
      });

    circles.exit().remove();

    maxTextWidth = d3.max([(d3.max(infoText[0], function(element) {
      return element === undefined ? 0 : element.getBBox().width;
    }) || 0), (self.tooltip.select("text.date")[0][0].getBBox().width - 15)]);

    // Specify background rectangle width and height
    self.tooltip.select("rect.rect-container")
      .attr("width", maxTextWidth + 40)
      .attr("height", 32 + 17 * (drawableSeries.length + outliers.length));

    // Always contain the tooltip on the chart
    if (xtooltip + maxTextWidth + 80 > self.width) {
      xtooltip -= maxTextWidth + 80;
    }

    if (ytooltip < 10) {
      ytooltip = 10;
    } else if (ytooltip + 42 + 17 * drawableSeries.length > self.height) {
      ytooltip = self.height - (42 + 17 * drawableSeries.length);
    }
  

    if (drawableSeries.length > 0) self.tooltip.attr("transform", "translate(" + xtooltip + ", " + ytooltip + ")");
    else self.tooltip.attr("transform", "translate(-999999, -999999)");
  },
  // Draws the tooltip for outliers points
  _loadSecondaryTooltip: function(xcoord, rect_width) {
    var self = this;
    outliers = self.semitop.selectAll('circle[class^="rb-point-"][cx="' + xcoord + '"]:not([style*="display: none"])')[0];

    // Draw secondary rectangles
    second_rects = self.tooltip.selectAll("rect.rb-rectangles")
      .data(outliers, function (d) {
        return d.attributes.class.value;
      });

    second_rects.enter()
      .append("rect")
      .attr("x", 12)
      .attr("width", 7)
      .attr("height", 7)
      .attr("rx", 2)
      .attr("ry", 2)
      .attr("stroke-width", 2.0)
      .attr("stroke", function (d) { return d.attributes.stroke.value; })
      .attr("fill", "#E17171")
      .attr("class", "rb-rectangles");

    second_rects.attr("y", function (d, i) { return (17 * i) + 20 + rect_width; });

    second_rects.exit().remove();

    // Draw secondary texts
    infoSecondText = self.tooltip.selectAll("text.rb-texts")
      .data(outliers, function (d) {
        return d.attributes.class.value;
      });

    infoSecondText.enter()
      .append("text")
      .attr("x", 26)
      .attr("class", "rb-texts");

    infoSecondText
      .attr("y", function (d, i) { return (17  * i) + 27 + rect_width; })
      .text(function (d) {
        val = Math.round(d.attributes.expected.value *
            self.multiplier * 1000) / 1000 + " " + self.multiplierPrefix + self.options.units;
        return 'Expected: ' + val;
      });

    infoSecondText.exit().remove();

    return outliers;
  },
  // Removes the tooltip
  _onMouseOut: function (mouse) {
    var self = this;

    if (mouse[0] < 0 || mouse[0] > self.width || mouse[1] < 0 || mouse[1] > self.height) {
      self.tooltip.attr("transform", "translate(-999999, -999999)");
      self.semitop.select("line.tooltip").attr("y1", -999999).attr("y2", -999999);
      self.semitop.selectAll("circle.elements").attr("cy", -999999);
    } 
  
    self._showLastValue();
  },

  _showLastValue: function () {
    var self = this;
    if(self.options.show_value_always) {
      
      // height = 10 so timestamp is always visible
      mouse = [self.width, 10];
      self._onMouseMove(mouse);

    }
  },
  // Fired when clicked on a path
  _onPathMouseClick: function (context) {
    var self = this,
      mouse = d3.mouse(context),
      selected, tempElements;

    // Select only visible "stacked" or "area" series
    // We reverse it to select to area nearest to the viewer
    tempElements = self.elements.slice().reverse();
    tempElements.every(function (element) {
      var date, value, serieValue,
        start, stop, d,
        ind, index, fecha;

      if (element.visible && !element.hidden && (element.type === "area" || element.type === "stacked")) {
        // Lets check if X is in self serie's domain
        date = self.x.invert(mouse[0]);
        value = self.y.invert(mouse[1]) / self.multiplier;

        if (date.getTime() >= element.values[0].timestamp.getTime() &&
          date.getTime() <= element.values[element.values.length - 1].timestamp.getTime()) {

          // Construct the range
          start = self.maxDomain[0];
          stop = new Date(self.maxDomain[1].getTime() + self.options.step);
          d = d3.range(start, stop, self.options.step);
          ind = self.bisect(d, self.x.invert(mouse[0]));

          // Adjust the selected index, choosing the
          // closer index to the mouse position
          if (mouse[0] - self.x(d[ind - 1]) < (self.x(stop) - self.x(stop - self.options.step)) / 2) {
            ind -= 1;
          }

          // Date where the selected index is
          fecha = new Date(d[ind]);

          // And now lets see if it is inside the area Y domain too
          index = element.values.map(function (x) {
            return x.timestamp.getTime();
          }).indexOf(fecha.getTime());

          serieValue = element.values[index].y + (element.values[index].y0 || 0);
          if (value <= serieValue && value >= (element.values[index].y0 || 0)) {
            // If it isnt maximized
            if (self.maximized === "") {
              self._maximize(element);
            } else {
              self._unmaximize();
              self._stackAll();
            }
            // If we found one, exit
            return false;
          }
        }
      }

      return true;
    });

    // Draw all
    self._updateXDomain();
    self._updateYDomain();
    self._updateMain();
    self._updateAxis();
  },
  _maximize: function (element) {
    var self = this;

    // Hide all visible elements except this one
    self.elements.forEach(function (elem) {
      if (elem !== element) {
        elem.hidden = true;
      }
    });

    // Put baseline on 0
    element.values.forEach(function (value) {
      value.y0 = 0;
    });

    self.maximized = element.name;
  },
  _unmaximize: function () {
    var self = this;

    // Unhide all visible elements
    self.elements.forEach(function (elem) {
      elem.hidden = false;
    });
    self.maximized = "";
  },
  /* Utils */
  _buildData: function (options) {
    var self = this, name,
      converted = {
        values: []
      };

    // Process, clean and convert the input data
    options.data.forEach(function (d, i) {
      var property, inside;
      converted.values[i] = {};

      // Parse the timestamp into Date object
      if (d.timestamp instanceof Date) {
        if (self.options.timezone === "auto") {
          converted.values[i].timestamp = d.timestamp;
        } else {
          converted.values[i].timestamp = new Date(d.timestamp.getTime() +
            self.options.timezone + new Date().getTimezoneOffset() * 60000);
        }
      } else {
        if (self.options.timezone === "auto") {
          converted.values[i].timestamp = new Date(d.timestamp.replace(/(\d*)-(\d*)-(\d*)/, "$1/$2/$3"));
        } else {
          converted.values[i].timestamp = new Date(new Date(d.timestamp.replace(/(\d*)-(\d*)-(\d*)/, "$1/$2/$3")).getTime() +
            self.options.timezone + new Date().getTimezoneOffset() * 60000);
        }
      }

      // Read the data
      for (property in d) {
        if (d.hasOwnProperty(property)) {
          if (property !== "version" && property !== "timestamp" && property !== "y") {
            if (property == "forecast") {
              converted.values[i].y = d[property];
            } else {
              for (inside in d[property]) {
                if (d[property].hasOwnProperty(inside)) {
                  if (self.options.dataFunction !== undefined) {
                    converted.values[i].y = self.options.dataFunction(d[property][inside]);
                  } else {
                    converted.values[i].y = parseFloat(d[property][inside]);
                  }
                }
              }
            }
          }
        }
      }
    });

    // Set the options
    if (options.displayName !== "" && options.displayName !== null && options.displayName !== undefined) {
      converted.realName = options.displayName;
    } else if (options.name !== "" && options.name !== null && options.name !== undefined) {
      converted.realName = options.name;
    } else {
      converted.realName = "N/A";
    }

    if (options.name !== "" && options.name !== null && options.name !== undefined) {
      name = options.name;
    } else {
      name = "N/A";
    }

    if (options.randomId === true) {
      converted.name = "rbchart" + self.elements.length;
    } else {
      converted.name = RB.util.cleanName(name);
    }

    converted.type     = options.type     || "area";
    converted.dashed   = options.dashed   || false;
    converted.showon   = options.showon   || "both";
    converted.remarked = options.remarked || [];
    converted.visible  = options.visible || true;
    converted.color    = options.color;
    converted.stroke   = options.stroke;
    converted.hidden   = false;

    // Sort the data time-ascending
    if (converted.values.length > 1 &&
      d3.ascending(converted.values[0].timestamp, converted.values[1].timestamp) === 1) {
      converted.values.sort(function (a, b) {
        return d3.ascending(a.timestamp, b.timestamp);
      });
    }

    return converted;
  },
  _stackAll: function () {
    var self = this,
      stackedAreas = [],
      stackedBars = [];

    self.elements.forEach(function (d) {
      if (d.type === "stacked" && d.visible === true) {
        stackedAreas.push(d);
      }
    });

    if (stackedAreas.length !== 0) {
      self.stack(stackedAreas);
    }

    self.elements.forEach(function (d) {
      if (d.type === "sbars" && d.visible === true) {
        stackedBars.push(d);
      }
    });

    if (stackedBars.length !== 0) {
      self.stack(stackedBars);
    }
  },
  _removeRange: function (date1, date2) {
    var self = this;

    self.elements.forEach(function (d) {
      d.values = d.values.filter(function (d2) {
        if (d2.timestamp >= date1 && d2.timestamp <= date2) {
          return false;
        } else {
          return true;
        }
      });
    });
  },
  _updateYDomain: function () {
    var self = this,
      maxWithStacks = 0;

    // Compute the max Y if the series are stacked
    maxWithStacks = (d3.max(self.elements, function (d) {
      if (d.visible && !d.hidden) {
        var range = self.x.domain();

        if(range[1] - range[0] < self.options.step) {
          range[0] = range[0].getTime() - self.options.step;
          range[1] = range[1].getTime() + self.options.step;
        }

        return (d3.max(d.values, function (d2) {
          var ret = 0;
          // Is the point on the visible X domain ?
          if (!self.options.yzoom || (d2.timestamp >= range[0] && d2.timestamp <= range[1])) {
            ret = d2.y;
            if ((d.type === "stacked" || d.type === "sbars") && d2.y0 !== undefined) {
              ret += d2.y0;
            }
          }

          return ret;
        }));
      } else {
        return 0;
      }
    }));

    // Check limits
    if (self.options.yLimits.length > 0) {
      maxWithStacks = d3.max(self.options.yLimits.concat(maxWithStacks));
    }

    // Auto scale the Y domain
    if (self.options.autoscaling === true) {
      // Save multiplier properties
      var scaled = RB.util.scale(maxWithStacks, self.options.multiplier);
      self.multiplier = scaled.multiplier;
      self.multiplierPrefix = scaled.prefix;

      // Append it to axis' ticks
      if (self.options.hide_axises === false && self.options.hide_y_axis === false) {
        self.yAxis.tickFormat(self.tickYFormatter);
      }
    }

    // If the computed Y is 0, use a default value (for empty charts)
    if (maxWithStacks === 0) {
      maxWithStacks = self.y.domain()[1] / self.multiplier;
    }

    var min = d3.min(self.elements, function (d) {
      return d3.min(d.values, function (d2) {
        return d2.y;
      });
    }) || 0;

    if (min > 0) min = 0;

    var max = self.multiplier * maxWithStacks || 0;

    // Use the computed Y domain
    self.y.domain([min, max]);

    // If there is a navigation chart, use the same Y domain on it
    if (self.options.navchart === true) {
      self.y2.domain([min, d3.max(self.elements, function (d) {
        return d3.max(d.values, function (d2) {
          return d2.y;
        });
      })]);
    }

    if (self.render) {
      circles = self.semitop.selectAll('circle[class^="rb-point-"]');
      circles
        .attr("cy", function (d) {
          var index, ret,
            date = new Date(this.attributes.timestamp.value.replace(" ", "T").replace(" ", ""));
          // Make timestamp compatible with different javascript parsers
          var fecha = new Date(date.toISOString());

          index = d.values.map(function (x) {
            return x.timestamp.getTime();
          }).indexOf(fecha.getTime());

          ret = self.y(self.multiplier * d.values[index].y);

          if ((d.type === "stacked" || d.type === "sbars") && d.values[index].y0 !== undefined) {
            ret = self.y(self.multiplier * (d.values[index].y + d.values[index].y0));
          }

          return ret;
        });
    }
  },
  _updateXDomain: function (remove_from) {
    var self = this,
      minAndMax = [-1, -1],
      displayed = 0,
      minute = 60 * 1000,
      n, m, old_value, ret = [];

    // Compute the maximum and minimum timestamp in the elements
    self.elements.forEach(function (d) {
      // Only if self serie is visible
      if (d.visible && !d.hidden) {
        var min = new Date(
            Math.floor(d.values[0].timestamp / self.options.step) * self.options.step
          ),
          max = new Date(
            Math.ceil(d.values[d.values.length - 1].timestamp / self.options.step) * self.options.step
          );

        if (minAndMax[0] === -1 || min < minAndMax[0]) {
          minAndMax[0] = min;
        }

        if (minAndMax[1] === -1 || max > minAndMax[1]) {
          minAndMax[1] = max;
        }
        displayed += 1;
      }
    });

    // If there are visible elements...
    if (minAndMax[0] !== -1 && minAndMax[1] !== -1 && minAndMax[0] !== minAndMax[1]) {
      // If there is only one point of data, extend the domain
      if (minAndMax[0].getTime() === minAndMax[1].getTime()) {
        minAndMax[0] = new Date(minAndMax[0].getTime() - self.options.step);
        minAndMax[1] = new Date(minAndMax[1].getTime() + self.options.step);
      }

      // Now extend self domain if specified in params
      if (self.domain[0] !== -1 || self.domain[1] !== -1) {
        if (self.domain[0] !== -1 && self.domain[0] < minAndMax[0]) {
          n = Math.round((minAndMax[0] - self.domain[0]) / self.options.step);
        } else {
          n = 0;
        }

        if (self.domain[1] !== -1 && minAndMax[1] < self.domain[1]) {
          m = Math.round((self.domain[1] - minAndMax[1]) / self.options.step);
        } else {
          m = 0;
        }

        minAndMax[0] = new Date(minAndMax[0].getTime() - n * self.options.step);
        minAndMax[1] = new Date(minAndMax[1].getTime() + m * self.options.step);

        // If there's too much a difference from the start to the end
        // and we are appending data, return the elements self won't be displayed
        if (minAndMax[1].getTime() - minAndMax[0].getTime() > self.options.step * self.options.maxPoints) {
          if (remove_from === "right") {
            ret[1] = minAndMax[1];
            minAndMax[1] = new Date(minAndMax[0].getTime() + self.options.maxPoints * self.options.step);
            ret[0] = new Date(minAndMax[1].getTime() + self.options.step);
          } else if (remove_from === "left") {
            ret[0] = minAndMax[0];
            minAndMax[0] = new Date(minAndMax[1].getTime() - self.options.maxPoints * self.options.step);
            ret[1] = new Date(minAndMax[0].getTime() - self.options.step);
          }
        }
      }

      // If there is only one serie displayed, the x domain must
      // be the computed maximum and minimum timestamps
      if (self._usesMaxDomain()) {
        self.x.domain(minAndMax);
      }

      // If there is a navchart, always display the maximum domain
      if (self.options.navchart === true) {
        self.x2.domain(minAndMax);
      }

      // Save it for later use
      self.maxDomain = minAndMax;

    // If there is no visible element...
    } else if(self.domain[0] !== -1 && self.domain[1] !== -1) {
      // Use the default domain
      self.x.domain(self.domain);

      if (self.options.navchart === true) {
        self.x2.domain(self.domain);
      }

      // Save it for later use
      self.maxDomain[0] = self.domain[0];
      self.maxDomain[1] = self.domain[1];
    }

    // Show the reset button if necessary
    if (displayed !== 0) {
      if (!self._usesMaxDomain()) {
        self.onTop.select(".reset-button").style("display", "block");
      }
    }

    return ret;
  },
  _usesMaxDomain: function () {
    var self = this,
      ret = false;

    if (self.maxDomain[0] === -1 && self.maxDomain[1] === -1) {
      ret = true;
    } else {
      if (self.x.domain()[0].getTime() === self.maxDomain[0].getTime() &&
        self.x.domain()[1].getTime() === self.maxDomain[1].getTime()) {
        ret = true;
      }
    }

    if (self.domain[0] !== -1 && self.domain[1] !== -1) {
      if (self.x.domain()[0].getTime() === self.domain[0].getTime() &&
        self.x.domain()[1].getTime() === self.domain[1].getTime()) {
        ret = true;
      }
    }

    return ret;
  },
  _fixMissingDataToAll: function () {
    var self = this;

    self.elements.forEach(function (d) {
      d.values = self._fixMissingDataToSerie(d.values);
    });
  },
  // Append zeros on a serie where data is missing
  _fixMissingDataToSerie: function (quien) {
    var self = this,
      domain = [],
      index = -1,
      ret = [],
      i = 0,
      auxArray;

    auxArray = quien.map(function (x) {
      return x.timestamp.getTime();
    });

    domain[0] = d3.min(auxArray);
    domain[1] = d3.max(auxArray);

    if (self.maxDomain[0] !== -1) {
      if (self.maxDomain[0].getTime() < domain[0]) {
        domain[0] = self.maxDomain[0].getTime();
      }
    }

    if (self.domain[0] !== -1) {
      if (self.domain[0].getTime() < domain[0]) {
        domain[0] = self.domain[0].getTime();
      }
    }

    if (self.maxDomain[1] !== -1) {
      if (self.maxDomain[1].getTime() > domain[1]) {
        domain[1] = self.maxDomain[1].getTime();
      }
    }

    if (self.domain[1] !== -1) {
      if (self.domain[1].getTime() > domain[1]) {
        domain[1] = self.domain[1].getTime();
      }
    }

    var start = new Date(domain[0]),
      step = self.options.step,
      stop = new Date(domain[1] + step),
      range = d3.range(start, stop, step),
      max = (stop - start) / step,
      counter = 0;

    for (i = 0; i < max; i += 1) {
      ret[i] = {};
      ret[i].timestamp = new Date(range[i]);

      index = auxArray.indexOf(range[i]);

      if (index !== -1) {
        counter = 0;
        ret[i].y = quien[index].y;
        if (quien[index].y0) {
          ret[i].y0 = quien[index].y0;
        }
      } else {
        if (self.options.fillWith === "previous" && i - 1 >= 0 && counter < 5) {
          ret[i].y = ret[i-1].y;
          counter += 1;
        } else {
          ret[i].y = 0;
          counter = 0;
        }
      }
    }

    return ret;
  }
};

RB.Bars = function (options) {
  // Get/set width and height
  var that = this, minMax;

  // General properties
  this.elements = []; // Elements in the diagram
  this.color = options.color;
  this.sharedColors = RB.util.default(options.shared_colors, true);
  this.palette = RB.util.default(options.palette, undefined);
  this.colorScale = d3.scale.ordinal().range(RB.Common.palette);
  this.max = RB.util.default(options.max, -1);
  this.total = RB.util.default(options.total, -1);
  this.showBiggestAsBarTotal = RB.util.default(options.showBiggestAsBarTotal, false);
  this.showAbsoluteNumbers = RB.util.default(options.showAbsoluteNumbers, false);
  this.units = RB.util.default(options.units, '');
  this.margin = {};
  this.margin.top = RB.util.default((options.margin || {}).top, 10);
  this.margin.bottom = RB.util.default((options.margin || {}).bottom, 10);
  this.margin.right = RB.util.default((options.margin || {}).right, 10);
  this.margin.left = RB.util.default((options.margin || {}).left, 10);
  this.options = options;

  // Draw the svg element
  this.svg = d3.select('#' + options.container)
    .append("svg")
    .attr("class", "rbcharts");

  // Group for paths
  this.group = this.svg
    .append("g");

  // Scale functions
  this.x = d3.scale.linear();
  this.y = d3.scale.ordinal();

  // If data is specified
  if (options.data !== undefined) {
    // Set the data
    this.setData(options.data);
  } else {
    // Set default x domain
    minMax = [0, 100];
    this.x.domain(minMax);
  }

  // Draw the elements
  this.resize();
};

RB.Bars.prototype = {
  resize: function () {
    var that = this, w, h, nelements;

    // Get width defaults
    w = RB.util.default(this.options.width, d3.select("#" + this.options.container).style("width"));
    w = parseInt(w, 10);

    // Set margins
    if (this.max != -1) {
      var max = parseInt(this.max, 10);
      if (max < this.elements.length) {
        nelements = max;
      } else {
        nelements = this.elements.length;
      }
    } else {
      nelements = this.elements.length;
    }

    // Set default nelements to 6
    if (nelements === 0) nelements = 6;

    // Get height from num elements
    h = RB.util.default(this.options.width, d3.select("#" + this.options.container).style("height"));
    h = parseInt(h, 10);

    // Calculate width and height of the svg element
    this.width = w - this.margin.left - this.margin.right;
    this.height = h - this.margin.top - this.margin.bottom;

    // Resize the x scale
    this.x.range([0, this.width]);
    this.y.rangeRoundBands([0, this.height], 0.2);

    // Resize the svg elements
    this.svg
      .attr("width", this.width + this.margin.left + this.margin.right)
      .attr("height", this.height + this.margin.top + this.margin.bottom);

    this.group
      .attr("transform", "translate(" + (this.margin.left) + ", " + (this.margin.top) + ")");

    // Draw the chart
    this.draw();
  },
  setData: function (data) {
    var that = this, minMax, extent;

    this.options.data = data;
    // Populate the data with the needed attributes
    this.elements = this.parseData(data.filter(function (d, i) {
      if (that.max !== -1) {
        return i < that.max;
      } else {
        return true;
      }
    }));

    // Update the x scale
    extent = d3.extent(data, function(d) { return d.events; });
    if (extent[0] > 0 && (extent[1] === extent[0])) {
      this.x.domain([(extent[0] - 1), extent[1]]).nice();
    } else {
      this.x.domain(extent).nice();
    }
    this.y.domain(data.map(function(d) { return d.name; }));

    // Draw the chart
    this.resize();
  },
  setTotal: function(total, draw) {
    this.total = total;

    // Draw the graph?
    if(draw){
      this.resize();
    }
  },
  _getColor: function (key) {
    var self = this;

    if (self.options.color !== undefined) {
      return self.options.color;
    } else if (self.options.sharedColors === true) {
      return RB.Common.color(key);
    } else if (self.options.palette !== undefined && self.options.palette[key] !== undefined) {
      return self.options.palette[key];
    } else {
      return self.colorScale(key);
    }
  },
  draw: function () {
    var that = this, rects, labels;

    this.group.selectAll(".nodata").remove();
    if (this.options.data === undefined || this.options.data.length === 0) {
      this.group.append("text")
          .attr("class", "nodata")
          .text("There is no data")
          .attr("y", (this.height / 2) + "px")
          .style("font-size", "20px");
      return;
    }

    this.group.selectAll(".bar").remove();
    this.group.selectAll(".bar")
      .data(this.options.data)
      .enter().append("rect")
      .attr("class", "bar")
      .attr("fill", function (d) {
        return that._getColor(d.name);
      })
      .attr("fill-opacity", 0.75)
      .attr("stroke", function (d) {
        return d3.rgb(that._getColor(d.name)).darker(0.7);
      })
      .attr("stroke-width", 2)
      .attr("stroke-opacity", 0.8)
      .attr("rx", 3)
      .attr("ry", 3)
      .attr("x", function(d) { return that.x(Math.min(0, d.events)); })
      .attr("y", function(d) { return that.y(d.name); })
      .attr("height", that.y.rangeBand());

    // If only one element is present in the chart, it may fill the graph's width.
    if(this.options.data.length < 2){
      this.group.selectAll(".bar")
        .attr("width", function(d) { return that.width; });
    } else {
      this.group.selectAll(".bar")
        .attr("width", function(d) { return Math.abs(that.x(d.events) - that.x(0)); });
    }

    // Draw elements names
    labels = this.group.selectAll("text.barlabel").remove();
    labels = this.group.selectAll("text.barlabel")
      .data(this.elements, function (d) {
        return d.id;
      });

    // Limit font size to not be less than 10 pixels or more than 50 pixels.
    var max_length_text = 0;

    // Get the label with more characters.
    for (var i = 0; i < this.options.data.length; i++) {
      text_length = this.options.data[i].name.length;

      if(text_length > max_length_text) {
        max_length_text = text_length;
      }
    }

    // Font size depending on bar height
    var bar_height = (that.height / this.options.data.length) / 1.5;

    // Font size depending on bar width
    var bar_width = that.width / (max_length_text * 1.5);

    // Select the more restrictive font size.
    var font_size = Math.min.apply(null, [bar_height, bar_width]);

    if(font_size < 10){
      font_size = 10;
    } else if(font_size > 50){
      font_size = 50;
    }

    labels.enter()
      .append("text")
      .attr("class", "barlabel")
      .style("font-size", font_size + "px")
      .text(function (d) {
        var max_letters = (that.width - that.margin.left - that.margin.right - 60) / 5,
          name = d.name,
          finalName, scaled;

        if (name.length > max_letters) {
          name = name.substr(0, max_letters) + "...";
        }

        if (that.showAbsoluteNumbers) {
          finalName = name + ' : ';
        } else {
          scaled = RB.util.scale(d.events);
          finalName = name + ' : ' + (Math.round(scaled.scaledNum * 100) / 100) + ' ' + scaled.prefix + that.units + ' : ';
        }

        return finalName;
      })
      .attr("y", function (d) { return that.y(d.name) + that.y.rangeBand() / 2 + 3; })
      .style("text-anchor", "end")
      .append("tspan")
      .style("font-weight", "bold")
      .text(function (d) {
        if (that.showAbsoluteNumbers) {
          return d.events;
        } else {
          return d.percentage + '%';
        }
      });

    labels
      .transition()
      .attr("x", this.width - 5);

  },
  parseData: function (data) {
    var that = this, ret = [], scale;
    var sum = 0
    // Configure data and add to ret
    data.forEach(function (d, i) {
      if (d.name === "" || d.name === undefined || d.name === null) {
        d.name = "N/A";
      }

      ret.push(d);
    });

    // Calculate the value that corresponds to 100 percent
    if (this.total !== -1) {
      if(typeof(this.total) != 'object'){
        sum = parseFloat(this.total);
      } else {
        for(var i = 0; i < this.total.length; i++){
          sum += this.total[i].events
        }
      }
    } else {
      sum = d3.sum(ret, function (d) {
        return parseFloat(d.events);
      });
    }

    var minMax = [d3.min(data, function (d) {
        return parseFloat(d.events);
      }), d3.max(data, function (d) {
        return parseFloat(d.events);
      })];
    var norm = sum;

    if (minMax[0] < 0) norm = minMax[1];

    // Make them percentages
    ret.forEach(function (d, i) {
      d.percentage = (100 * parseFloat(d.events)) / norm;
      d.percentage = d.percentage.toFixed(1);
      d.id = i;
    });

    return ret;
  }
};

RB.VerticalBars = function (options) {
  // Get/set width and height
  var self = this, minMax, maxName;

  // General properties
  self.elements = []; // Elements in the diagram
  self.color = RB.util.default(options.color, "#4572a7");
  self.max = RB.util.default(options.max, -1);
  self.total = RB.util.default(options.total, -1);
  self.units = RB.util.default(options.units, '');
  self.margin = {};
  self.margin.top = RB.util.default((options.margin || {}).top, 30);
  self.margin.bottom = RB.util.default((options.margin || {}).bottom, 70);
  self.margin.right = RB.util.default((options.margin || {}).right, 20);
  self.margin.left = RB.util.default((options.margin || {}).left, 20);
  self.maxLetters = RB.util.default(options.maxLetters, 20);
  self.options = options;
  self.bisect = d3.bisector(function (d) { return d; }).right;
  self.multiplier = 1;
  self.multiplierPrefix = "";

  // Draw the svg element
  self.svg = d3.select('#' + options.container).append("svg")
    .attr("class", "rbcharts");

  // Group for paths
  self.group = self.svg.append("g");

  // Group for axises
  self.semitop = self.svg.append("g");

  self.semitop.append("circle")
    .attr("x", 0)
    .attr("y", 0);

  // Draw axises
  self.semitop.append("g").attr("class", "x axis");
  self.semitop.append("g").attr("class", "y axis");

   // Tooltip group
  self.tooltip = self.semitop.append("g")
    .attr("class", "tooltip");

  // Append the container rectangle
  self.tooltip.append("rect")
    .attr("x", 0)
    .attr("y", 0)
    .attr("class", "rect-container");

  // Append the texts and default circle
  self.tooltip.append("text")
    .attr("x", 0)
    .attr("y", 0)
    .attr("class", "name")
    .append("tspan");

  self.onTop = self.svg.append("g")
    .append("rect")
    .attr("x", 0)
    .attr("y", 0)
    .attr("width", self.width)
    .attr("height", self.height)
    .style("opacity", 0)
    .on("mousemove", function () {
      self._onMouseMove(d3.mouse(this));
    })
    .on("mouseout", function () {
      self._onMouseOut(d3.mouse(this));
    });

  // Scale functions
  self.x = d3.scale.ordinal();
  self.y = d3.scale.linear();

  // Axises
  self.xAxis = d3.svg.axis().scale(self.x)
    .orient("bottom")
    .tickFormat(function (d) {
      var name = d;

      if (name.length > self.maxLetters) {
        name = name.substr(0, self.maxLetters - 8) + "..." + name.substr(name.length - 8, name.length);
      }

      return name;
    });

  self.yAxis = d3.svg.axis().scale(self.y)
    .tickFormat(function (d) { return d + " " + self.multiplierPrefix + self.units; })
    .ticks(5)
    .orient("left");

  if (options.data !== undefined) {
    this.setData(options.data);
  } else {
    minMax = [0, 100];
    this.y.domain(minMax);
  }

  this.resize();
};

RB.VerticalBars.prototype = {
  resize: function () {
    var self = this, w, h, nelements;

    // Get width and height defaults
    w = RB.util.default(this.options.width, d3.select("#" + this.options.container).style("width"));
    h = RB.util.default(this.options.height, d3.select("#" + this.options.container).style("height"));
    w = parseInt(w, 10);
    h = parseInt(h, 10);

    // Set margins
    if (self.max != -1) {
      var max = parseInt(self.max, 10);
      if (max < this.elements.length) {
        nelements = max;
      } else {
        nelements = this.elements.length;
      }
    } else {
      nelements = this.elements.length;
    }

    // Set default nelements if this.elements is empty
    if (nelements === 0) nelements = 6;

    // Calculate width and height of the svg element
    self.width = w - self.margin.left - self.margin.right;
    self.height = h - self.margin.top - self.margin.bottom;
    self.padding = self.padding || 150 / nelements;

    // Scale functions
    self.x.rangeRoundBands([0, self.width]);
    self.y.range([self.height, 0]);

    // Update axis tick size
    self.yAxis.tickSize(-self.width, 0, 0);

    // Update SVG elements
    self.svg
      .attr("width", self.width + self.margin.left + self.margin.right)
      .attr("height", self.height + self.margin.top + self.margin.bottom);

    self.group
      .attr("transform", "translate(" + (self.margin.left) + ", " + (self.margin.top) + ")");

    self.semitop
      .attr("transform", "translate(" + (self.margin.left) + ", " + (self.margin.top) + ")");



    // Calculate minMax for range
    minMax = [d3.min(self.elements, function (d) {
      return parseFloat(d.events);
    }), d3.max(self.elements, function (d) {
      return parseFloat(d.events);
    })];

    var xAxisShift = self.height;
    if (minMax[0] <= 0 && minMax[1] > 0) {
      xAxisShift = parseInt(minMax[1]/Math.abs(minMax[0] - minMax[1]) * self.height);
    }
    if (minMax[0] < 0 && minMax[1] <= 0) {
      xAxisShift = -self.margin.top;
    }

    self.semitop.select(".x.axis")
      .attr("transform", "translate(0," + xAxisShift + ")")
      .call(self.xAxis);

    self.semitop.select(".y.axis")
      .attr("class", "y axis")
      .call(self.yAxis);

    self.semitop.selectAll("g text")
      .attr("y", -8)
      .style("text-anchor", "start");

    self.semitop.select(".y.axis g:first-child")
      .style("opacity", 0);

    self.onTop
      .attr("transform", "translate(" + self.margin.left + "," + self.margin.top + ")")
      .select("rect")
      .attr("width", self.width)
      .attr("height", self.height);

    self.semitop.selectAll(".x.axis text")
      .style("text-anchor", "start")
      .attr("transform", function (d) {
        var el =  d3.select(this);
        return "translate(" + el.attr("x") + ", " + (parseInt(el.attr("y")) + 20) + ") rotate(45)";
      })
      .attr("x", 0)
      .attr("y", 0);

    // Draw the elements
    self.draw();
  },
  setData: function (data) {
    var self = this, minMax, scaled;

    // Populate the data with the needed attributes
    self.elements = self.parseData(data.filter(function (d, i) {
      if (self.max !== -1) {
        return i < self.max;
      } else {
        return true;
      }
    })).sort(function (a, b) {
      return parseInt(a.name) - parseInt(b.name);
    });

    self.x.domain(self.elements.map(function (e) {
      return e.name;
    }));

    var elems = self.options.data.map(function(d){return d.events;});
    elems.push(0);
    self.y.domain(d3.extent(elems));

    // Resize and draw
    self.resize();
  },
  draw: function () {
    var self = this, labels;
    var color = null;

    self.group.selectAll("rect").remove();
    self.group.selectAll("rect")
    .data(self.options.data, function (d) {
        return d.id;
    })
    .enter()
    .append("rect")
    .attr("class", function(d) {
        return d.events < 0 ? "rect negative" : "rect positive";
    })
    .attr("fill", function(d) {
        return d.priority ? redBorder.utils.colors.influence[d.priority] : self.color;
    })
    .attr("fill-opacity", 0.75)
    .attr("stroke", function(d) {
        return d3.rgb(d.priority ? redBorder.utils.colors.influence[d.priority] : self.color).darker(0.7);
    })
    .attr("stroke-width", 2)
    .attr("stroke-opacity", 0.8)
      .attr({
        x: function(d){
          return self.x(d.name) + self.padding / 2;
        },
        y: function(d){
          return self.y(Math.max(0, d.events * self.multiplier));
        },
        width: self.x.rangeBand() - self.padding,
        height: function(d){
          return Math.abs(self.y(d.events * self.multiplier) - self.y(0));
        }
      });
  },
  parseData: function (data) {
    var self = this, ret = [],
      sum, scale;

    // Configure data and add to ret
    data.forEach(function (d, i) {
      if (d.name === "" || d.name === undefined || d.name === null) {
        d.name = "N/A";
      }

      ret.push(d);
    });

    // Calculate the value self corresponds to 100 percent
    if (self.total !== -1) {
      sum = parseFloat(self.total);
    } else {
      sum = d3.sum(ret, function (d) {
        return d.events;
      });
    }

    // Make them percentages
    ret.forEach(function (d, i) {
      d.percentage = (100 * d.events) / sum;
      d.percentage = d.percentage.toFixed(1);
      d.id = i;
    });

    return ret;
  },
  // Draws the tooltip
  _onMouseMove: function (mouse) {
    var self = this, elementsPosition, ind,
      xcoord, ytooltip, xtooltip,
      infoText, textValue, maxTextWidth;

    // Select the index
    elementsPosition = self.x.range().map(function (d) { return (d + self.x.rangeBand() * 1.2); });
    ind = self.bisect(elementsPosition, mouse[0]);
    if (ind >= self.elements.length) return;

    // X coord of the selected point
    xcoord = self.x(self.elements[ind].name);

    // Calculate tooltip position
    ytooltip = mouse[1] - 45;
    xtooltip = xcoord + self.x.rangeBand() / 2 + 20;

    // Draw text
    infoText = self.tooltip.select("text.name")
      .attr("x", 10)
      .attr("y", 18)
      .style("font-weight", "bold")
      .text(self.elements[ind].name);

    infoText.append("tspan")
      .style("font-weight", "normal")
      .text(": " + (self.elements[ind].events * self.multiplier) + " " + self.multiplierPrefix);

    maxTextWidth = d3.max(infoText[0], function(element) {
      return element.getBBox().width;
    });

    // Specify background rectangle width and height
    self.tooltip.select("rect.rect-container")
      .attr("width", maxTextWidth + 30)
      .attr("height", 30);

    // Always contain the tooltip on the chart
    if (xtooltip + maxTextWidth + 80 > self.width) {
      xtooltip -= maxTextWidth + 80;
    }

    if (ytooltip < 10) ytooltip = 10;
    else if (ytooltip + 50 > self.height) {
      ytooltip = self.height - 50;
    }

    self.tooltip.attr("transform", "translate(" + xtooltip + ", " + ytooltip + ")");
  },
  // Removes the tooltip
  _onMouseOut: function (mouse) {
    var self = this;

    if (mouse[0] < 0 || mouse[0] > self.width || mouse[1] < 0 || mouse[1] > self.height) {
      self.tooltip.attr("transform", "translate(-999999, -999999)");
      self.semitop.select("line.tooltip").attr("y1", -999999).attr("y2", -999999);
      self.semitop.select("circle").attr("cy", -999999);
    }
  }
};
RB.Pie = function (options) {
  // Get/set width and height
  var that = this, data_to_add;

  // General properties
  this.elements = []; // Elements in the diagram
  this.max = RB.util.default(options.max, -1);
  this.highlighted = undefined;
  this.total = RB.util.default(options.total, -1);
  this.margin = {};
  this.split = false;
  this.full_legend = RB.util.default(options.full_legend, true);
  this.show_highest_percen = RB.util.default(options.show_highest_percen, true);
  this.options = options;
  this.options.margin = {
    top:               RB.util.default((options.margin || {}).top, 15),
    bottom:            RB.util.default((options.margin || {}).bottom, 5),
    horizontal:        RB.util.default((options.margin || {}).right, 15)
  };

  // Draw the svg element
  this.svg = d3.select('#' + options.container)
    .append("svg")
    .attr("class", "rbcharts");

  // Group for paths
  this.group = this.svg.append("g");

  // Value info text
  this.valueText = this.group
    .append("text")
    .attr("class", "infoText")
    .attr("dy", ".3em");

  // Group for info
  this.info = this.svg.append("g");

  // Configure the pie layout
  this.pie = d3.layout.pie()
    .value(function (d) { return d.events; });

  // Set the arc
  this.arc = d3.svg.arc()
    .innerRadius(function (d) { return d.data.innerRadius; });

  // If data is specified, set it
  if (options.data !== undefined) this.setData(options.data);

  // Set default size
  this.resize();
};

RB.Pie.prototype = {
  resize: function (width, height) {
    var that = this, w, h, nelements, h_info;

    // By default we mus't split
    this.split = false;

    // Get w
    if (this.options.width === undefined) {
      w = width  || d3.select("#" + this.options.container).style("width");
    } else {
      w = this.options.width;
    }

    if (this.options.height === undefined) {
      h = height || d3.select("#" + this.options.container).style("height");
    } else {
      h = this.options.height;
    }

    // Remove the "px" at the end of the radius
    this.width = parseInt(w, 10);
    this.height = parseInt(h, 10);

    // If w is "auto", assign it a default value
    if (isNaN(this.width)) {
      this.width = 180;
    }

    if (isNaN(this.height)) {
      this.height = 180;
    }

    // Set margins
    this.margin.top = this.options.margin.top;
    this.margin.horizontal = this.options.margin.horizontal;

    // Set the nelements.
    // Suppose default 3 elements if no data is specified
    if (this.elements.length === 0) nelements = 3;
    else nelements = this.elements.length;

    // Set the height of info group
    h_info = nelements * 15 + (nelements - 1) * 3;

    if (h_info > this.height){
      h_info = h_info/2;
      this.split = true;
    }

    if (this.height > this.width) {
      // Vertical style
      // Set radius based on width or height, depending the relation
      this.radius = (this.height - h_info)/2 - this.options.margin.top - this.options.margin.bottom;
    } else {
      // Horizontal style
      // Set radius based on width or height, depending the relation
      this.radius = this.height/2 - this.margin.top;
    }

    if (this.height > this.width) {
      // Vertical style
      // Set bottom margin bassed on number of elements
      //this.margin.left = this.width/2 - this.radius - this.margin.horizontal
      this.margin.bottom = this.options.margin.bottom + 16 + h_info;
    } else {
      // Horizontal style
      // Set bottom margin bassed on number of elements
      //this.margin.right = this.options.margin.right + 16 + nelements * 15 + (nelements - 1) * 3;
      this.margin.info_top = this.height/2 - h_info/2;
    }

    // Calculate height of the svg element
    //this.height = (2 * this.radius) + this.margin.top + this.margin.bottom;

    // Resize svg
    this.svg
      .attr("width", this.width)
      .attr("height", this.height)
      .style("display", "block");

    // Resize groups
    if (this.height > this.width) {
      // Resize path group
      this.group
        .attr("transform", "translate(" + (this.width/2) + ", " + (this.margin.top + this.radius) + ")");

      // Resize info group
      this.info
        .attr("transform", "translate(" + (this.margin.horizontal - 3) + " , " + (this.radius * 2 + this.margin.top + 20) + ")");
    } else {
      // Resize path group
      this.group
        .attr("transform", "translate(" + (this.radius + this.margin.horizontal) + ", " + (this.margin.top + this.radius) + ")");

      // Resize info group
      this.info
        .attr("transform", "translate(" + (this.radius * 2 + this.margin.top + 25) + " , " + this.margin.info_top + ")");
    }

    // Resize arc
    this.arc.outerRadius(this.radius);

    // Resize radiuses
    this.elements.forEach(function (d) {
      scaled = RB.util.scale(d.value );

      if(that.full_legend){
        dname = d.data.name + ' - ' +(Math.round(scaled.scaledNum * 100) / 100) + ' ' + scaled.prefix +' - ' +d.data.percentage + '%';
      } else {
        dname = d.data.name;
      }


      var t_width = 36 + dname.length * 7.5;

      if (t_width > that.width) {
        var maxLength = (that.width - that.margin.horizontal * 2) / 7.5;
        d.data.displayName = dname.substr(0, maxLength - 1) + "...";
      } else {
        d.data.displayName = dname;
      }

      d.data.innerRadius = that.radius / 1.5;
    });

    // Resize font size
    this.valueText
      .style("font-size", this.radius / 4);

    // Unhighlighting to apply the effect while resizing, cloning and creating
    this.unhighlight();
    this.draw(true);
  },
  setData: function (data) {
    var data_to_add;

    // Parse data
    data_to_add = this.parseData(data);

    // Set the data
    this.elements = this.pie(data_to_add);

    // Update the chart
    this.resize();
  },
  draw: function (notransition) {
    var that = this, text,
      arcs, rects, group, nelements;

    // Check if the highest percentage should always be shown
    this.showHighestPercen();

    // Draw them all
    arcs = this.group.selectAll("path.arc")
      .data(this.elements, function (d) {
        return d.data.id;
      });

    arcs.enter()
      .append("path")
      .attr("class", "arc")
      .attr("fill", function (d) {
        if (d.data.color !== undefined) return d.data.color;
        else return RB.Common.color(d.data.name);
      })
      .on("mouseout", function(e) {
        that.onMouseOut(e, this);
      })
      .on("mouseenter", function(e) {
        that.onMouseIn(e, this);
      });

    arcs.transition()
      .duration(function () { return notransition === true ? 0 : 500; })
      .attr("d", this.arc);

    this.info.selectAll("g").remove();

    rects = this.info.selectAll("rect")
      .data(this.elements, function (d) {
        return d.data.id;
      }).enter();

    // Group container
    group = rects.append("g")
      .on("mouseenter", function(e) {
        that.onMouseIn(e, this);
      })
      .on("mouseout", function(e) {
        that.onMouseOut(e, this);
      });

    if (this.elements.length === 0) nelements = 3;
    else nelements = this.elements.length;

    if (this.split){
      // Draw the rects
      group.append("rect")
        .attr("x", function (d, i) { return (i < nelements/2) ? 0 : 140; })
        .attr("y", function (d, i) { return (i < nelements/2) ? (18 * i) : (18 * (i - nelements/2)); })
        .attr("width", 8)
        .attr("height", 8)
        .attr("rx", 2)
        .attr("ry", 2)
        .attr("fill", function (d) {
          if (d.data.color !== undefined) return d.data.color;
          else return RB.Common.color(d.data.name);
        });

      // Draw the texts
      text = group.append("text")
        .attr("class", "rb-label")
        .attr("x", function (d, i) { return (i < nelements/2) ? 14 : 154; })
        .attr("y", function (d, i) { return (i < nelements/2) ? ((18 * i) + 8) : ((18 * (i - nelements/2)) + 8); })
        .text(function (d) {
          var t = d.data.displayName;
          if (that.options.showQuantityOnLegend) t = t + ' - ';
          return t;
        });
    } else {
      // Draw the rects
      group.append("rect")
        .attr("x", 0)
        .attr("y", function (d, i) { return (18 * i); })
        .attr("width", 8)
        .attr("height", 8)
        .attr("rx", 2)
        .attr("ry", 2)
        .attr("fill", function (d) {
          if (d.data.color !== undefined) return d.data.color;
          else return RB.Common.color(d.data.name);
        });

      // Draw the texts
      text = group.append("text")
        .attr("class", "rb-label")
        .attr("x", 14)
        .attr("y", function (d, i) { return (18 * i) + 8; })
        .text(function (d) {
          var t = d.data.displayName;
          if (that.options.showQuantityOnLegend) t = t + ' - ';
          return t;
        });
    }

    if (that.options.showQuantityOnLegend) {
      text.append("tspan")
        .style("font-weight", "bold")
        .text(function (d) {
          if (that.options.showAbsoluteNumbers) {
            return d.data.events;
          } else {
            return d.data.percentage + '%';
          }
        });
    }
  },
  parseData: function (data) {
    var that = this, ret = [],
      sum, total, scale;

    // Configure data and add to ret
    data.forEach(function (d, i) {
      d.id = d.name;
      d.events = d.events || 0;
      d.events = parseFloat(d.events);
      if (isNaN(d.events)) d.events = 0;

      if (that.max === -1 || i < that.max) {
        d.innerRadius = that.radius / 1.5;

        if (d.name === "" || d.name === undefined || d.name === null) {
          d.name = "N/A";
        }

        var t_width = 36 + d.name.length * 7.5;

        if (t_width > that.width) {
          var maxLength = (that.width - that.margin.horizontal * 2) / 7.5;
          d.displayName = d.name.substr(0, maxLength - 1) + "...";
        } else {
          d.displayName = d.name;
        }

        ret.push(d);
      }
    });

    // Calculate the sum of the data shown
    sum = d3.sum(ret, function (d) {
      return d.events;
    });

    // Calculate the value that represents 100 percent
    if (this.total !== -1) {
      total = this.total;
    } else {
      total = sum;
    }

    // Make an "others" group if total is greater than the sum
    if (total > sum) {
      var others = {};
      others.id = '__others__';
      others.name = others.displayName = "Others";
      others.events = total - sum;
      others.innerRadius = that.radius / 1.5;
      ret.push(others);
    }

    // Make them percentages
    ret.forEach(function (d) {
      d.percentage = (100 * d.events) / total;
      d.percentage = d.percentage.toFixed(1);
    });

    return ret;
  },
  unhighlight: function(){
    var that = this;

    this.elements.forEach(function(element){
      element.data.innerRadius = that.radius / 1.5;
    });

    this.valueText.text("");
    that.highlighted = undefined;
  },
  onMouseIn: function (event, context) {
    var that = this;

    this.unhighlight();

    // Create the effect
    event.data.innerRadius = this.radius / 1.7;
    that.highlighted = event.data.name;

    // Assign value to infoText
    this.valueText
      .text(event.data.percentage + " %");

    // Assign content to text
    this.draw();
  },
  onMouseOut: function (event, context) {
    this.unhighlight();
    this.showHighestPercen();
    this.draw();
  },
  showHighestPercen: function () {
    var that = this;

    if (that.options.show_highest_percen) {
      var highest, percentage, name, value = -1;

      // Look for the highest and get fields
      for (var i = 0; i < that.elements.length; i++) {
        if (that.elements[i].value > value) {
          highest = i;
          percentage = that.elements[i].data.percentage;
          name = that.elements[i].data.name;
          value = that.elements[i].value;
        }
      }

      if (that.elements.length > 0 && (that.valueText[0][0].innerHTML === "" || that.highlighted === undefined)) {
        // Create the effect
        that.elements[highest].data.innerRadius = that.radius / 1.7;
        that.valueText.text(percentage + " %");
        that.highlighted = name;
      }
    }
  }
};
  

RB.Trafficmap = function (options) {
  var that = this;
  var padding = 10;

  this._buildData = function(data) {
    return data;  
  }

  this.margin = {
    top: 0,
    bottom: 15,
    left: 10,
    right: 10
  };

  this.options = options;

  this.chartid = Math.random().toString().substring(2, 7);
  this.data = this._buildData(options.data);

  var container = d3.select("#" + options.container);

   var zoom = d3.behavior.zoom()
  .scaleExtent([1, 10])
  .on("zoom", zoomed);

  function zoomed() {
    var t = d3.event.translate;
    var s = d3.event.scale;

    t[0] = Math.min(width * (s - 1), Math.max(width * (1 - s), t[0]));
    t[1] = Math.min(height * (s - 1), Math.max(height * (1 - s), t[1]));

    zoom.translate(t);
    that.base.attr("transform", "translate(" + t + ")scale(" + s + ")");
  }

  container.style("height", "1000px");
  var width = container.node().getBoundingClientRect().width;
  var height = container.node().getBoundingClientRect().height;

  var nodes = {}, links = [];

   this.data.forEach(function(d) {
  if (!nodes[d.lan_ip]) {
    nodes[d.lan_ip] = { id: d.lan_ip, private: true, totalBytes: d.value, types: ['lan_ip'] };
  } else {
    nodes[d.lan_ip].totalBytes += d.value;
    if (!nodes[d.lan_ip].types.includes('lan_ip')) {
      nodes[d.lan_ip].types.push('lan_ip');
    }
  }
  if (!nodes[d.wan_ip]) {
    nodes[d.wan_ip] = { id: d.wan_ip, private: true, totalBytes: d.value, types: ['wan_ip'] };
  } else {
    nodes[d.wan_ip].totalBytes += d.value;
    if (!nodes[d.wan_ip].types.includes('wan_ip')) {
      nodes[d.wan_ip].types.push('wan_ip');
    }
  }
  links.push({ source: nodes[d.lan_ip], target: nodes[d.wan_ip], value: d.value });
   })
  
    var graph = { nodes: Object.values(nodes), links: links };

  var numNodos = graph.nodes.length;
  var alturaPorNodo = 4;

  var altura = numNodos * alturaPorNodo;

  altura = Math.min(Math.max(altura, 500), 1350);



  container.style("height", altura + "px");

  var width = container.node().getBoundingClientRect().width;
  var height = container.node().getBoundingClientRect().height;


  var minBytes = d3.min(graph.nodes, function(d) { return d.totalBytes; });
  var maxBytes = d3.max(graph.nodes, function(d) { return d.totalBytes; });

  var nodeScale = d3.scale.sqrt().domain([minBytes, maxBytes]).range([3, 20]);

  graph.nodes.sort(function(a, b) { return b.totalBytes - a.totalBytes; });

  this.svg = container.append("svg")
    .attr("class", "rbcharts rbnetworkgraph")
    .attr("width", width)
    .attr("height", height)
    .call(zoom);

   this.defs = this.svg.append("defs");
   this.base = this.svg.append("g");

   this.defs.append("clipPath")
    .attr("id", "rbcharts-clip-" + this.chartid)
    .append("rect")


   var force = d3.layout.force()
    .nodes(graph.nodes)
    .links(graph.links)
    .size([width, height])
    .linkDistance(150)
    .charge(-400)
    .gravity(0.4)
    .on("tick", ticked);

  force.start();
    setTimeout(function() {
    force.stop();
  }, 2500);

  function customCharge(d) {
    return -Math.max(width, height) * force.charge() * (d.x / width);
  }

  force.charge(customCharge);

  var link = this.base.selectAll(".trafficmapLink")
    .data(graph.links)
    .enter().append("line")
    .attr("class", "trafficmapLink highlighted")
    .style("opacity", 1);

  function formatBytes(bytes, decimals) {
    if (bytes === 0) return '0 Bytes';
    if (!decimals) decimals = 2;

    const k = 1024;
    const dm = decimals < 0 ? 0 : decimals;
    const sizes = ['Bytes', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'];

    const i = Math.floor(Math.log(bytes) / Math.log(k));

    return parseFloat((bytes / Math.pow(k, i)).toFixed(dm)) + ' ' + sizes[i];
  }

  var drag = force.drag()
    .on("dragstart", function(d) {
      dragstart(d);  
      d3.event.sourceEvent.stopPropagation();
      d.fixed = true;
    })
    .on("drag", function(d) {
      dragged(d);  
    })
    .on("dragend", function(d) {
      dragend(d);  
      d.fixed = true;
    });

 function dragstart(d) {
  force.stop();

  d.xStart = d.x;
  d.yStart = d.y;

  d.fixed = true;
}

function dragged(d) {
  d.px = d3.event.x;
  d.py = d3.event.y;

  d3.select(this).attr("transform", "translate(" + d.px + "," + d.py + ")");

  graph.links.forEach(function(link) {
    if (link.source === d) {
      link.source.x = d.px;
      link.source.y = d.py;
    } else if (link.target === d) {
      link.target.x = d.px;
      link.target.y = d.py;
    }
  });
}

function dragend(d) {
  d.fixed = true;
}


var node = this.base.selectAll(".trafficmapNode")
  .data(graph.nodes)
  .enter().append("g")
  .attr("class", "trafficmapNode highlighted")
  .style("opacity", 1) 
  .call(drag)
  .on("mousedown", function() { d3.event.stopPropagation(); });

  node.append("circle")
  .attr("r", function(d) { return nodeScale(d.totalBytes + 1); })
  .style("fill", function(d) {
    if (d.types.includes('lan_ip') && d.types.includes('wan_ip')) {
      return 'purple'; 
    } else if (d.types.includes('lan_ip')) {
      return 'blue';  
    } else if (d.types.includes('wan_ip')) {
      return 'green'; 
    } else {
      return 'gray';  
    }
  })

  node.append("text")
    .attr("dx", 12)
    .attr("dy", ".35em")
    .style("font-size", "1em")
    .text(function(d) { return d.id; })
    .attr("textLength", "40")
    .attr("lengthAdjust", "spacingAndGlyphs");

  node.append("title")
    .text(function(d) {
        var connectedLinks = graph.links.filter(function(link) {
            return link.source.id === d.id || link.target.id === d.id;
        });
        var linkValues = connectedLinks.map(function(link) {
            return "Link value: " + formatBytes(link.value);
        }).join(", ");
        return "IP: " + d.id + ", " + linkValues;
    });

function ticked() {
  link.attr("x1", function(d) { return Math.max(padding, Math.min(width - padding, d.source.x)); })
      .attr("y1", function(d) { return Math.max(padding, Math.min(height - padding, d.source.y)); })
      .attr("x2", function(d) { return Math.max(padding, Math.min(width - padding, d.target.x)); })
      .attr("y2", function(d) { return Math.max(padding, Math.min(height - padding, d.target.y)); });

  node.attr("transform", function(d) {
    d.x = Math.max(padding, Math.min(width - padding, d.x));
    d.y = Math.max(padding, Math.min(height - padding, d.y));
    return "translate(" + d.x + "," + d.y + ")";
  });
}


this.resize = function() {
  width = container.node().getBoundingClientRect().width;
  height = container.node().getBoundingClientRect().height;

  that.svg.attr("width", width)
    .attr("height", height);

  force.size([width, height]).resume();
};

var colors = ['blue', 'green', 'purple'];
var descriptions = ['LAN IP', 'WAN IP', 'Bidirectional'];

var legend = this.svg.selectAll(".legend")
  .data(colors)
  .enter().append("g")
  .attr("class", "legend")
  .attr("transform", function(d, i) { return "translate(0," + (height - 30 - i * 30) + ")"; });

legend.append("rect")
  .attr("x", width - 18)
  .attr("width", 28)
  .attr("height", 28)
  .style("fill", function(d, i) { return colors[i]; });

legend.append("text")
  .attr("x", width - 34)
  .attr("y", 15)
  .attr("dy", ".35em")
  .style("text-anchor", "end")
  .text(function(d, i) { return descriptions[i]; });



this.selectedItems = {};

  this.highlight = function(lan_ip, wan_ip) {
    this.selectedItems[lan_ip + "-" + wan_ip] = {lan_ip: lan_ip, wan_ip: wan_ip};
    this.highlightAll();
  };

  this.unhighlight = function(lan_ip, wan_ip) {
    delete this.selectedItems[lan_ip + "-" + wan_ip];
    this.highlightAll();
  };

  this.highlightAll = function() {
    node.classed("highlighted", false);
    link.classed("highlighted", false);

    const connectedNodes = new Set();

    for (var key in this.selectedItems) {
      const item = this.selectedItems[key];

      link.each(function(d) {
        if (d.source.id === item.lan_ip || d.target.id === item.lan_ip ||
            d.source.id === item.wan_ip || d.target.id === item.wan_ip) {
          connectedNodes.add(d.source.id);
          connectedNodes.add(d.target.id);
        }
      });
    }

    node.classed("highlighted", function(d) {
      return connectedNodes.has(d.id);
    });

    link.classed("highlighted", function(d) {
      return connectedNodes.has(d.source.id) && connectedNodes.has(d.target.id);
    });

    if(Object.keys(this.selectedItems).length === 0) {
      node.classed("highlighted", true);
      link.classed("highlighted", true);
    }
  };
};

RB.Trafficmap.prototype = {
  updateGraph: function () {
    var that = this;

    this.simulation
      .nodes(this.data.nodes)
      .on("tick", ticked);

    this.simulation.force("link")
      .links(this.data.links);

    var links = this.networkGraph.selectAll(".link")
      .data(this.data.links);

    links.enter().append("line")
      .attr("class", "link");

    links.exit().remove();

    var nodes = this.networkGraph.selectAll(".node")
      .data(this.data.nodes, function(d) { return d.id; });

    var nodesEnter = nodes.enter().append("g")
      .attr("class", "node");

    
    nodesEnter.append("circle")
     .attr("r", 5)
     .attr("id", function(d) { return "node-" + d.id; });  // Agrega un id único a cada nodo

   
    nodesEnter.append("text")
      .attr("dx", 12)
      .attr("dy", ".35em")
      .text(function(d) { return d.id; });

    nodes.exit().remove();

    nodes.call(d3.drag()
      .on("start", dragstarted)
      .on("drag", dragged)
      .on("end", dragended));

    function ticked() {
      links
        .attr("x1", function(d) { return d.source.x; })
        .attr("y1", function(d) { return d.source.y; })
        .attr("x2", function(d) { return d.target.x; })
        .attr("y2", function(d) { return d.target.y; });

      nodes
        .attr("transform", function(d) { return "translate(" + d.x + "," + d.y + ")"; });
    }

    function dragstarted(d) {
      if (!d3.event.active) that.simulation.alphaTarget(0.3).restart();
      d.fx = d.x;
      d.fy = d.y;
    }

    function dragged(d) {
      d.fx = d3.event.x;
      d.fy = d3.event.y;
    }

    function dragended(d) {
      if (!d3.event.active) that.simulation.alphaTarget(0);
      d.fx = null;
      d.fy = null;
    }
  },

 highlight: function(id) {
    // Selecciona el nodo y cambia su color
    d3.select("#node-" + id)
      .style("fill", "yellow");  // Cambia el color a amarillo
  },

  unhighlight: function(id) {
    // Selecciona el nodo y cambia su color de vuelta
    d3.select("#node-" + id)
      .style("fill", "blue");  // Cambia el color de vuelta a azul
  }

};

RB.Scatterplot = function (options) {
  hook_scatterplot_filter()
  // Get width and height
  var that = this, restview;

  // Margins
  this.margin = {
    top: 0,
    bottom: 15,
    left: 10,
    right: 10
  };

  // Save them for later
  this.options = options;

  // Other important properties
  this.chartid = Math.random().toString().substring(2, 7);
  this.data = this._buildData(options.data);

  this.max = d3.max(this.data, function (d) { return parseInt(d.value); });
  this.min = d3.min(this.data, function (d) { return parseInt(d.value); });

  // General SVG element
  this.svg = d3.select("#" + options.container)
    .append("svg")
    .attr("class", "rbcharts rbscatterplot");

  this.defs = this.svg.append("defs");

  this.defs.append("clipPath")
    .attr("id", "rbcharts-clip-" + this.chartid)
    .append("rect");

  this.defs.append("clipPath")
    .attr("id", "rbcharts-leftbar-clip-" + this.chartid)
    .append("rect")
      .attr("width", 10);

  this.defs.append("clipPath")
    .attr("id", "rbcharts-bottombar-clip-" + this.chartid)
    .append("rect")
      .attr("height", 10);

  this.base = this.svg.append("g");

  this.tooltip_layer = this.svg.append("g")
    .attr("class", "rbtooltip");

  this.datalayer = this.svg.append("g");

  this.circles_group = this.datalayer.append("g")
    .attr("clip-path", "url(#rbcharts-clip-" + this.chartid + ")");

  this.leftbar_group = this.datalayer.append("g")
    .attr("clip-path", "url(#rbcharts-leftbar-clip-" + this.chartid + ")");

  this.bottombar_group = this.datalayer.append("g")
    .attr("clip-path", "url(#rbcharts-bottombar-clip-" + this.chartid + ")");

  this.onTop = this.svg.append("g");

  // Restart view rectangle
  restview = this.onTop.append("g")
    .attr("class", "reset-button")
    .style("display", "none");

  var reset = restview.append("g")
    .on("click", function () {
      // Restart domain
      that.x.domain([0, 4294967295]);
      that.y.domain([0, 4294967295]);

      // Update the chart
      that._autoZoom();

      // Hides itself
      restview.style("display", "none");
    });

  var zoom = restview.append("g")
    .on("click", function () {
      // Auto-zoom
      that._autoZoom();
    });

  reset.append("rect")
    .attr("x", -5)
    .attr("y", 4)
    .attr("width", 55)
    .attr("height", 21);

  reset.append("text")
    .attr("x", 23)
    .attr("y", 18)
    .style("text-anchor", "middle")
    .text("Reset");

  zoom.append("rect")
    .attr("x", -55)
    .attr("y", 4)
    .attr("width", 50)
    .attr("height", 21);

  zoom.append("text")
    .attr("x", -29)
    .attr("y", 18)
    .style("text-anchor", "middle")
    .text("Auto");

  // Scale functions
  this.x = d3.scale.linear().domain([0, 4294967295]);
  this.y = d3.scale.linear().domain([0, 4294967295]);
  this.rad = d3.scale.linear().range([2, 20]).domain([this.min, this.max]);

  // Axises
  this.xAxis = d3.svg.axis().scale(this.x)
    .orient("bottom")
    .tickFormat(function (d) {
      return RB.util.numToIP(Math.round(d));
    });

  this.yAxis = d3.svg.axis().scale(this.y)
    .orient("left")
    .tickFormat(function (d) {
      return RB.util.numToIP(Math.round(d));
    });

  // Brush
  this.brush = d3.svg.brush()
    .x(this.x)
    .y(this.y)
    .on("brushend", function () {
      that._onMainBrushEnd(this);
    });

  this.base.append("g")
    .attr("class", "x axis")
    .call(this.xAxis)
    .append("g")
    .append("text")
    .attr("x", -50)
    .attr("y", 20)
    .text("Destination");

  this.base.append("g")
    .attr("class", "y axis")
    .call(this.yAxis)
    .append("g")
    .attr("transform", "rotate(90)translate(10, 15)")
    .append("text")
    .text("Origin");

  this.base.select(".x.axis text")
    .style("opacity", 1)
    .style("font-size", 11);

  this.base.select(".y.axis text")
    .style("opacity", 0);

  // Style fixes
  this.base.selectAll("g.x.axis .tick text")
    .attr("x", 8)
    .attr("y", -15);

  this.base.selectAll("g.y.axis .tick text")
    .attr("x", 8)
    .attr("y", -10);

  this.base.append("g")
    .attr("class", "brush")
    .call(this.brush);

  this.resize(this.options.width, this.options.height);
};

RB.Scatterplot.prototype = {
  resize: function (width, height) {
    var that = this, w, h;

    // Hide the SVG
    this.svg.style("display", "none");

    w = width || that.options.width || d3.select("#" + this.options.container).style("width");
    h = height || that.options.height || d3.select("#" + this.options.container).style("height");
    w = parseInt(w, 10);
    h = parseInt(h, 10);

    // If w or h is "auto", assign it a default value
    if (isNaN(w)) {
      w = 300;
    }

    if (isNaN(h) || h === 0) {
      h = w/4;

      if (h < 200) {
        h = 200;
      }
    }

    this.width = parseInt(w) - this.margin.left - this.margin.right;
    this.height = parseInt(h) - this.margin.top - this.margin.bottom;

    this.svg
      .attr("width", this.width + this.margin.left + this.margin.right)
      .attr("height", this.height + this.margin.top + this.margin.bottom)
      .style("display", "block");

    this.defs.select("#rbcharts-clip-" + this.chartid + " rect")
      .attr("width", this.width)
      .attr("height", this.height)
      .attr("transform", "translate(0, " + this.margin.top + ")");

    this.defs.select("#rbcharts-leftbar-clip-" + this.chartid + " rect")
      .attr("height", this.height)
      .attr("transform", "translate(-10, " + this.margin.top + ")");

    this.defs.select("#rbcharts-bottombar-clip-" + this.chartid + " rect")
        .attr("width", this.width)
        .attr("transform", "translate(0, " + (this.margin.top + this.height) + ")");

    this.base.attr("transform", "translate(" + (this.margin.left + 10) + ",0)");

    this.datalayer.attr("transform", "translate(" + (this.margin.left + 10) + ",0)");

    // Restart view rectangle
    this.onTop.select("g.reset-button")
      .attr("transform", "translate(" + (this.width - 45) + ", " + (this.margin.top + 1) + ")");

    // Scale functions
    this.x.range([0, this.width]);
    this.y.range([this.height, 0]);

    // Axises
    this.xAxis.tickSize(-this.height, 0, 0);
    this.yAxis.tickSize(-this.width, 0, 0);

    this.base.select("g.x.axis")
      .attr("transform", "translate(0," + (this.height + this.margin.top) + ")")
      .call(this.xAxis)
      .select("g:last-child")
      .attr("transform", "translate(" + (this.width + this.margin.left) + ", -7)");

    this.base.select("g.y.axis")
      .attr("transform", "translate(0," + (this.margin.top) + ")")
      .call(this.yAxis);

    this.base.select("g.brush")
      .attr("transform", "translate(0, " + this.margin.top + ")")
      .call(this.brush);

    this._updateAxis();

    if (this.options.autozoom) {
      this._autoZoom();
    }

    this._draw(true);
  },
  add: function (data) {
    if (data.length === 0) return;
    this.data = this.data.concat(this._buildData(data));
    this.max = d3.max(this.data, function (d) { return parseInt(d.value); });
    this.min = d3.min(this.data, function (d) { return parseInt(d.value); });
    this.rad = d3.scale.linear().range([2, 20]).domain([this.min, this.max]);
    this._draw();
    hook_scatterplot_filter()
  },
  highlight: function (src, dst) {
    var datum, el;

    this.data.forEach(function (d) {
      if (d.x === dst && d.y === src) {
        datum = d;
      }
    });

    if (datum === undefined) {
      throw new Error("That point doesn't exist on the scatterplot");
    }

    el = this.datalayer.selectAll("circle")
      .filter(function (d) {
        if (d === datum) {
          return d;
        }
       })
      .attr("class", "highlighted");
  },
  unhighlight: function (src, dst) {
    var datum, el;

    this.data.forEach(function (d) {
      if (d.x === dst && d.y === src) {
        datum = d;
      }
    });

    if (datum === undefined) {
      throw new Error("That point doesn't exist on the scatterplot");
    }

    el = this.datalayer.selectAll("circle")
      .filter(function (d) {
        if (d === datum) {
          return d;
        }
       })
      .attr("class", "");
  },
  _draw: function (notransition) {
    var that = this,
      circles, leftbar, bottombar;

    circles = this.circles_group.selectAll("circle")
      .data(this.data);

    circles.enter()
      .append("circle")
      .attr("fill-opacity", 0.5)
      .on("mouseenter", function (e) {
        that._onMouseIn(d3.mouse(this), this);
      })
      .on("mouseout", function (e) {
        that._onMouseOut(d3.mouse(this), this);
      });

    circles
      .transition().duration(function (d) { return notransition? 0 : 500; })
      .attr("cx", function (d) { return that.x(RB.util.IPToNum(d.x)); })
      .attr("cy", function (d) { return that.y(RB.util.IPToNum(d.y)) + that.margin.top; })
      .attr("r", function (d) { return that.rad(d.value); });

    leftbar = this.leftbar_group.selectAll("line")
      .data(this.data);

    leftbar.enter()
      .append("line")
      .attr("class", "horizontal");

    leftbar
      .transition().duration(function (d) { return notransition? 0 : 500; })
      .attr("x1", -10)
      .attr("x2", 0)
      .attr("y1", function (d) { return that.y(RB.util.IPToNum(d.y)) + that.margin.top; })
      .attr("y2", function (d) { return that.y(RB.util.IPToNum(d.y)) + that.margin.top; });

    bottombar = this.bottombar_group.selectAll("line")
      .data(this.data);

    bottombar.enter()
      .append("line")
      .attr("class", "vertical");

    bottombar
      .transition().duration(function (d) { return notransition? 0 : 500; })
      .attr("x1", function (d) { return that.x(RB.util.IPToNum(d.x)); })
      .attr("x2", function (d) { return that.x(RB.util.IPToNum(d.x)); })
      .attr("y1", this.margin.top + this.height)
      .attr("y2", this.margin.top + this.height + 10);
  },
  _updateAxis: function () {
    this.base.select(".x.axis")
      .call(this.xAxis);

    this.base.select(".y.axis")
      .call(this.yAxis);

    this.base.selectAll("g.x.axis .tick text")
      .attr("x", 8)
      .attr("y", -15);

    this.base.selectAll("g.y.axis .tick text")
      .attr("x", 8)
      .attr("y", -10);
  },
  _autoZoom: function () {
    var xdom = [-1, -1],
      ydom = [-1, -1],
      actualXDom = this.x.domain(),
      actualYDom = this.y.domain();

    this.data.forEach(function (d) {
      var x = RB.util.IPToNum(d.x),
        y = RB.util.IPToNum(d.y);

      if (x > actualXDom[0] && x < actualXDom[1] &&
        y > actualYDom[0] && y < actualYDom[1]) {
        if (xdom[0] === -1 || x < xdom[0]) xdom[0] = x;
        if (xdom[1] === -1 || x > xdom[1]) xdom[1] = x;
        if (ydom[0] === -1 || y < ydom[0]) ydom[0] = y;
        if (ydom[1] === -1 || y > ydom[1]) ydom[1] = y;
      }
    });

    if(xdom[0] !== -1 && xdom[1] !== -1 &&
      ydom[0] !== -1 && ydom[1] !== -1) {
      var marginx = Math.round((xdom[1] - xdom[0]) / 10);
      var marginy = Math.round((ydom[1] - ydom[0]) / 12);

      if (marginx === 0) marginx = 5;
      if (marginy === 0) marginy = 5;

      this.x.domain([(xdom[0] - marginx), (xdom[1] + marginx)]);
      this.y.domain([(ydom[0] - marginy), (ydom[1] + marginy)]);
      this._updateAxis();
      this._draw();
    }
  },
  _buildData: function (data) {
    var dataret = [],
      that = this,
      pattern = /^([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])$/,
      ret;

    data.forEach(function (d) {
      if (d.dst !== null && d.src !== null && pattern.test(d.dst) && pattern.test(d.src)) {
        ret = {
          x: d.dst,
          y: d.src,
          value: 0
        };

        for (var prop_1 in d) {
          if (d.hasOwnProperty(prop_1)) {
            if (prop_1 !== "timestamp" && prop_1 !== "src" && prop_1 !== "dst" &&
              prop_1 !== "dst_name" && prop_1 !== "src_name") {
              if (d[prop_1] !== null) {
                ret.value = d[prop_1];
              }
            }
          }
        }

        dataret.push(ret);
      } else if (d.lan_ip !== null && d.wan_ip !== null && pattern.test(d.lan_ip) && pattern.test(d.wan_ip)) {
        ret = {
          y: d.lan_ip,
          x: d.wan_ip,
          value: 0
        };

        for (var prop2 in d) {
          if (d.hasOwnProperty(prop2)) {
            if (prop2 !== "timestamp" && prop2 !== "lan_ip" && prop2 !== "wan_ip" &&
              prop2 !== "lan_ip_name" && prop2 !== "wan_ip_name") {
              if (d[prop2] !== null) {
                ret.value = d[prop2];
              }
            }
          }
        }

        dataret.push(ret);
      }
    });

    return dataret;
  },
  _onMainBrushEnd: function (context) {
    // If empty, empty() returns null, idk why
    if (this.brush.empty() !== true) {
      if ((this.brush.extent()[1][0] - this.brush.extent()[0][0]) >= 1) {
        this.x.domain([Math.round(this.brush.extent()[0][0]), Math.round(this.brush.extent()[1][0])]);
      }

      if ((this.brush.extent()[1][1] - this.brush.extent()[0][1]) >= 1) {
        this.y.domain([Math.round(this.brush.extent()[0][1]), Math.round(this.brush.extent()[1][1])]);
      }

      // Clear the brush
      this.brush.clear();
      this.svg.select(".brush .extent").attr("width", 0);

      // Show the reset button
      this.onTop.select(".reset-button")
        .style("display", "block");

      // Draw all
      this._updateAxis();
      this._draw();
    }
  },
  _onMouseIn: function (mouse, e) {
    var that = this, layer,
      node, ipx, ipy;

    node = d3.select(e).data();
    ipx = RB.util.IPToNum(node[0].x);
    ipy = RB.util.IPToNum(node[0].y);

    this.tooltip_layer.append("line")
      .attr("x1", function (d) {
        return that.margin.left + that.x(ipx);
      })
      .attr("x2", function (d) {
        return that.margin.left + that.x(ipx);
      })
      .attr("y1", that.margin.top)
      .attr("y2", that.margin.top + that.height);

    this.tooltip_layer.append("line")
      .attr("x1", this.margin.left)
      .attr("x2", this.margin.left + this.width)
      .attr("y1", function (d) {
        return that.y(ipy) + that.margin.top;
      })
      .attr("y2", function (d) {
        return that.y(ipy) + that.margin.top;
      });

    this.tooltip_layer.append("text")
      .text(RB.util.numToIP(ipy))
      .attr("x", this.margin.left + that.width)
      .attr("y", function (d) {
        return that.y(ipy) - 5 + that.margin.top;
      })
      .style("text-anchor", "end");

    this.tooltip_layer.append("text")
      .text(RB.util.numToIP(ipx))
      .attr("y", this.margin.top + 10)
      .attr("x", function (d) {
        return that.x(ipx) + that.margin.left + 5;
      });
  },
  _onMouseOut: function (mouse, context) {
    this.tooltip_layer.selectAll("*").remove();
  }
};

RB.Sankey = function (options) {
    var that = this;

    this.options = options;
    this.margin = RB.util.default(options.margin, {top: 10, right: 10, bottom: 10, left: 10});
    this.formatNumber = d3.format(",.0f");
    this.linksName = RB.util.default(options.linksName, "events");
    this.format = function(d) { return that.formatNumber(d) + " " + that.linksName; };

    this.palette = [
      "#638bdf", "#e86b58", "#89a54e", "#e89c4f", "#946acc",
      "#1758b5", "#c74531", "#3e812d", "#d87e34", "#734ca2",
      "#63a0e7", "#f4988a", "#a2bb77", "#edbc6c", "#be90fd",
      "#0f3742", "#783234", "#55633f", "#fdc52b", "#371c39",
      "#38606b", "#996062", "#6f795f", "#878787", "#8a3b8f",
      "#5d8d9a", "#ae9596", "#a8b298", "#bcbcbc", "#eb80f1"
    ];

    this.color = d3.scale.ordinal().range(this.palette);
    this.nodes = this.originalNodes = [];
    this.links = this.originalLinks = [];
    this.value = RB.util.default(options.value, "events");
    this.maxElementsInColumn = RB.util.default(options.maxElementsInColumn, 10);
    this.onClick = options.onClick;
    this.onContextMenu = options.onContextMenu;
    this.linksToLighten = [];

    this.svg = d3.select("#" + options.container).append("svg")
        .attr("class", "rbsankey");

    this.linksGroup = this.svg.append("g");
    this.nodesGroup = this.svg.append("g");

    this.sankey = d3.sankey()
        .nodeWidth(15)
        .nodePadding(10);

    this.resize(this.options.width, this.options.height);

    this.path = this.sankey.link();
};

RB.Sankey.prototype = {
    resize: function (width, height) {
        var that = this, w, h;

        this.svg.style("display", "none");

        // Get width and height defaults
        w = width || that.options.width || d3.select("#" + that.options.container).style("width");
        h = height || that.options.height || d3.select("#" + that.options.container).style("height");

        // Remove the "px" at the end of both height and width
        w = parseInt(w, 10);
        h = parseInt(h, 10);

        // Default height is width / 3, if not specified in CSS
        if (isNaN(h) || h === 0) {
            if (this.navchart) {
                h = (w / 3) + 60 + this.margin.top;
            } else {
                h = (w / 3);
            }
        }

        // Calculate width and height
        this.width  = w - this.margin.left - this.margin.right;
        this.height = h - this.margin.top - this.margin.bottom;

        this.svg
            .attr("width", this.width + this.margin.left + this.margin.right)
            .attr("height", this.height + this.margin.top + this.margin.bottom);

        this.linksGroup
            .attr("transform", "translate(" + this.margin.left + "," + this.margin.top + ")");

        this.nodesGroup
            .attr("transform", "translate(" + this.margin.left + "," + this.margin.top + ")");

        this.svg.selectAll("text.no-data")
            .attr("x", this.width/2)
            .attr("y", this.height/2 + 20);

        this.sankey.size([this.width, this.height]);

        this.svg.style("display", "block");

        this.draw();
    },
    add: function (data, columns) {
        var that = this;

        if (columns.length > 1) {
            data.forEach(function (element) {
                if (element[that.value] === 0 || element[that.value] === undefined || element[that.value] === null ) return;
                var i, elementsOnOthers = [];

                // Create nodes
                for(i = 0; i < columns.length; i++) {
                    var name, index = -1;

                    if (element[columns[i]] === "" || element[columns[i]] === undefined || element[columns[i]] === null) {
                        element[columns[i]] = "N/A";
                    }

                    if (element[columns[i]].length > 20) {
                        name = element[columns[i]].substr(0, 20) + "...";
                    } else {
                        name = element[columns[i]];
                    }

                    // Search for the name
                    that.originalNodes.forEach(function (d, ielem) {
                        if (d.id == element[columns[i]] && d.column == columns[i]) {
                            index = ielem;
                        }
                    });

                    // Add column if it don't exists
                    if (index === -1) {
                        // Check if we already have the max allowed columns
                        var numColumns = that.originalNodes.filter(function (d) {
                            return d.column === columns[i];
                        }).length;

                        if (numColumns < that.maxElementsInColumn - 1) {
                            // Add the new column
                            index = that.originalNodes.length;

                            that.originalNodes.push({
                                id: element[columns[i]],
                                column: columns[i],
                                name: name
                            });
                        } else {
                            // Does an "Others" element exists in the column?
                            that.originalNodes.forEach(function (d, ielem) {
                                if (d.id == "others" && d.column == columns[i]) {
                                    index = ielem;
                                }
                            });

                            // Add a new others node if it doesnt exist
                            if (index === -1) {
                                index = that.originalNodes.length;

                                that.originalNodes.push({
                                    id: "others",
                                    column: columns[i],
                                    name: "Others"
                                });
                            }

                            // Save the element to avoid it when adding links
                            elementsOnOthers.push({
                                id: index,
                                column: columns[i]
                            });
                        }
                    }

                    // Replace column for its index in the originalNodes array
                    element[columns[i]] = index;
                }

                // Create links
                for(i = 0; i < columns.length - 1; i++) {
                    var foundLink = -1;

                    // Is this element in the Others group?
                    elementsOnOthers.forEach(function (d) {
                        if (d.id === element[columns[i]] && d.column === columns[i]) {
                            element[columns[i]] = d.id;
                        }
                        if (d.id === element[columns[i + 1]] && d.column === columns[i + 1]) {
                            element[columns[i + 1]] = d.id;
                        }
                    });

                    // Does a link with the same src and dst already exist?
                    that.originalLinks.forEach(function (d, index) {
                        if (d.source === element[columns[i]] && d.target === element[columns[i + 1]]) {
                            foundLink = index;
                        }
                    });

                    if (foundLink === -1) {
                        // If not, add it
                        that.originalLinks.push({
                            source: element[columns[i]],
                            target: element[columns[i + 1]],
                            value: element[that.value]
                        });
                    } else {
                        // If it exists, append the extra new events
                        that.originalLinks[foundLink].value += element[that.value];
                    }
                }
            });
        }

        this.draw();
    },
    draw: function () {
        var that = this;

        if (!this.originalNodes.length) {
            this.svg.selectAll("text.no-data")
                .data(["unique"])
                .enter()
                .append("text")
                .attr("class", "no-data")
                .text("No enough data to display")
                .style("text-anchor", "middle")
                .style("font-size", "32px")
                .style("fill", "#4A4A4A")
                .attr("x", this.width/2)
                .attr("y", this.height/2 + 20);

            this.linksGroup.selectAll("*").remove();
            this.nodesGroup.selectAll("*").remove();
        } else {
            // Copying arrays
            this.nodes = RB.util.copyHashArray(this.originalNodes);
            this.links = RB.util.copyHashArray(this.originalLinks);

            this.sankey
                .nodes(this.nodes)
                .links(this.links)
                .layout(4);

            // Remove the no-data text if exists
            this.svg.selectAll("text.no-data")
                .remove();

            var link = this.linksGroup.selectAll(".link")
                .data(this.links, function (d) {
                    return [d.source.id, d.source.column, d.target.id, d.target.column];
                });

            var newLink = link.enter()
                .append("path")
                    .on("mouseenter", function () {
                        var thisElement = this;
                        d3.select(this).attr("stroke-opacity", 0.4);
                    })
                    .on("mouseout", function () {
                        var flag = true,
                            thisElement = d3.select(this).data();

                        that.linksToLighten.forEach(function (d) {
                            if (d.source.id === thisElement[0].source.id &&
                                d.source.column === thisElement[0].source.column &&
                                d.target.id === thisElement[0].target.id &&
                                d.target.column === thisElement[0].target.column) {
                                flag = false;
                            }
                        });

                        if (flag) d3.select(this).attr("stroke-opacity", 0.1);
                    })
                    .on("click", function () {
                        var flag = false,
                            thisElement = d3.select(this).data(),
                            linksToDarken = that.getPath(thisElement);

                        that.linksToLighten.forEach(function (d) {
                            if (d.source.id === thisElement[0].source.id &&
                                d.source.column === thisElement[0].source.column &&
                                d.target.id === thisElement[0].target.id &&
                                d.target.column === thisElement[0].target.column) {
                                flag = true;
                            }
                        });

                        if (that.linksToLighten.length !== 0) {
                            that.linksGroup.selectAll(".link")
                                .data(that.linksToLighten, function (d) {
                                    return [d.source.id, d.source.column, d.target.id, d.target.column];
                                }).transition()
                                .duration(500)
                                .attr("stroke-opacity", 0.1);
                            that.linksToLighten = [];
                        }

                        if (!flag) {
                            that.linksGroup.selectAll(".link")
                                .data(linksToDarken, function (d) {
                                    return [d.source.id, d.source.column, d.target.id, d.target.column];
                                }).transition()
                                .duration(500)
                                .attr("stroke-opacity", 0.4);

                            that.linksToLighten = linksToDarken;
                        }

                        if (that.onClick !== undefined) {
                            that.onClick(thisElement, flag);
                        }
                    })
                    .attr("class", "link")
                    .attr("stroke", "black")
                    .attr("fill", "none")
                    .attr("stroke-opacity", 0.1)
                    .append("title");

            link.attr("d", this.path)
                .attr("stroke-width", function(d) { return Math.max(1, d.dy); })
                .sort(function(a, b) { return b.dy - a.dy; })
                .select("title")
                    .text(function(d) { return d.source.name + " -> " + d.target.name + "\n" + that.format(d.value); });

            link.exit().remove();

            var node = this.nodesGroup.selectAll(".node")
                .data(this.nodes, function (d) {
                    return [d.id, d.column];
                });

            var newNode = node.enter()
                .append("g")
                .attr("class", "node");

            node.attr("transform", function(d) { return "translate(" + d.x + "," + d.y + ")"; })
                .call(d3.behavior.drag()
                        .origin(function(d) { return d; })
                        .on("dragstart", function() { this.parentNode.appendChild(this); })
                        .on("drag", function (d) {
                            d3.select(this).attr("transform", "translate(" + d.x + "," +
                                (d.y = Math.max(0, Math.min(that.height - d.dy, d3.event.y))) + ")");
                            that.sankey.relayout();
                            link.attr("d", that.path)
                                .attr("stroke-width", function(d) { return Math.max(1, d.dy); })
                                .sort(function(a, b) { return b.dy - a.dy; })
                                .select("title")
                                    .text(function(d) { return d.source.name + " → " + d.target.name +
                                        "\n" + that.format(d.value); });
                        })
                    );

            newNode.append("rect")
                .attr("width", this.sankey.nodeWidth())
                .attr("fill", function(d) {
                    d.color = that.color(d.name.replace(/ .*/, ""));
                    return d.color;
                })
                .attr("stroke", function(d) { return d3.rgb(d.color).darker(2); })
                .attr("rx", 3)
                .attr("ry", 3)
                .on('contextmenu', function () {
                    var thisElement = d3.select(this).data();
                    if (that.onContextMenu !== undefined) {
                        that.onContextMenu(thisElement[0]);
                    }
                    d3.event.preventDefault();
                })
                .append("title");

            node.select("rect")
                .attr("height", function(d) { return d.dy; })
                .select("title")
                    .text(function(d) { return d.name + "\n" + that.format(d.value); });

            newNode.append("text")
                .attr("x", -6)
                .attr("dy", ".35em")
                .attr("text-anchor", "end")
                .attr("transform", null)
                .text(function(d) { return d.name; })
                .filter(function(d) { return d.x < that.width / 2; })
                .attr("x", 6 + this.sankey.nodeWidth())
                .attr("text-anchor", "start");

            node.select("text")
                .attr("y", function(d) { return d.dy / 2; });

            node.exit().remove();
        }
    },
    restart: function () {
        this.originalNodes = [];
        this.originalLinks = [];
    },
    getPath: function (element) {
        var collectedData = [], that = this;

        for (var x in element) {
            if (element[x].target) {
                collectedData.push(element[x]);
            }

            if (element[x].target && element[x].target.sourceLinks.length > 0) {
                collectedData = collectedData.concat(that.getPath(element[x].target.sourceLinks));
            }
        }

        return collectedData;
    }
};
RB.Ball = function (options) {
  var that = this;

  // General properties
  this.options = options;
  this.elements = []; // Elements in the diagram
  //this.color = d3.scale.linear().range(["white", "red"]);
  this.max = RB.util.default(options.max, 300);
  this.margin = RB.util.default(options.margin, { top: 5, horizontal: 5, bottom: 5 });
  this.value = RB.util.default(options.value, 10);
  this.width = options.width;
  this.height = options.height;
  this.invertColors = options.invertColors || false;

  // Color range
  var colors = ["#5DB262", "#7BC255", "#99CF3F", "#B2D62A", "#D4D62A",
                "#E7D820", "#E7B820", "#E78920", "#D95A1D", "#C61E33"];
  if (this.invertColors === true) colors = colors.reverse();

  var domain = [0, this.max*0.11, this.max*0.22, this.max*0.33, this.max*0.44,
                this.max*0.55, this.max*0.66, this.max*0.77, this.max*0.88, this.max];

  // Scale functions
  this.color = d3.scale.linear().domain(domain).range(colors);

  // Draw the svg element and the circle
  this.svg = d3.select('#' + options.container)
    .append("svg")
    .attr("class", "rbcharts")
    .on("mouseenter", function(e) {
      that.onMouseIn(e, this);
    })
    .on("mouseleave", function(e) {
      that.onMouseOut(e, this);
    });

  // Append the circle
  this.circle = this.svg.data([this.value]).append("circle");

  // And the text with the data
  this.valueText = this.svg
    .append("text")
    .attr("class", "infoText")
    .attr("dy", ".3em")
    .attr("fill", "#fff");

  // Draw the elements
  this.resize();
};

RB.Ball.prototype = {
  resize: function (width, height) {
    var that = this, w, h;

    // Get w and h
    w = width  || RB.util.default(this.options.width, d3.select("#" + this.options.container).style("width"));
    h = height || RB.util.default(this.options.height, d3.select("#" + this.options.container).style("height"));
    w = parseInt(w, 10);
    h = parseInt(h, 10);

    // Default width of 100
    if (isNaN(w) && isNaN(h)) {
      w = 100;
    } else if (isNaN(w) && !isNaN(h)){
      w = 0;
    } else if (isNaN(h) && !isNaN(w)){
      h = 0;
    }

    // Save the width
    that.width = w;
    that.height = h;

    // Set radius based on the smaller length
    if(w < h){
      this.radius = w / 2 - this.margin.horizontal;
    } else {
      this.radius = h / 2 - this.margin.top - this.margin.bottom;
    }

    this.svg
      .attr("width", that.width)
      .attr("height", that.height);

    this.valueText
      .style("font-size", this.radius / 3 + "px")
      .attr("transform", "translate(" + (that.width/2) + ", " + (this.height/2) + ")");

    // Draw the elements
    this.draw();
  },
  setValue: function (value) {
    this.value = value;
    this.draw();
  },
  draw: function () {
    var that = this, selection;

    selection = this.svg.selectAll("circle")
      .data([parseFloat(this.value)]);

    selection.enter()
      .append("circle");

    selection.attr("r", this.radius)
      .attr("cx", that.width/2)
      .attr("cy", that.height/2 - that.margin.top)
      .attr("fill", function (d) {
        if (d > that.max) {
          d = that.max;
        }

        return that.color(d);
      });

    if (that.options.alwaysShowQuantity && that.options.alwaysShowQuantity != "0") {
      that.onMouseIn();
    }
  },
  onMouseIn: function (event, context) {
    var value = RB.util.scale(this.value);

    // Assign value to infoText
    this.valueText.text((Math.round(value.scaledNum * 100) / 100) + " " + value.prefix);
  },
  onMouseOut: function (event, context) {
    if(!this.options.alwaysShowQuantity || this.options.alwaysShowQuantity == "0"){
      this.valueText.text("");
    }
  }
};

RB.util = {
  // Sets a default value for a object property
  default: function (where, default_value) {
    return (where !== undefined && where !== "") ? where : default_value;
  },
  // Find the minimum difference in times
  findStep: function (element) {
    var i = 0,
      max = 0,
      min = -1,
      data = [];

    data = element.values.map(function (d) {
      return d.timestamp;
    });

    max = data.length;
    if (max > 1) {
      for (i = 0; i < max; i += 1) {
        if (i === 1) {
          if (Math.abs(data[1] - data[0]) !== 0) min = Math.abs(data[1] - data[0]);
        } else if (i > 1) {
          if (Math.abs(data[i] - data[i - 1]) < min) {
            if (Math.abs(data[i] - data[i - 1]) !== 0) min = Math.abs(data[i] - data[i - 1]);
          }
        }
      }
    } else {
      min = -1;
    }

    return min;
  },
  // Clean the name param.
  cleanName: function (name) {
    name = name.replace(/ /g, "_");
    name = name.replace(/\.|\,/g, "-");
    name = name.replace(/:|\[|\]/g, "");

    return escape(name);
  },
  numToIP: function (num) {
    var d = num % 256;

    for (var i = 3; i > 0; i--) {
      num = Math.floor(num / 256);
      d = num % 256 + '.' + d;
    }

    return d;
  },
  IPToNum: function (ip) {
    var ret, a;

    a = ip.split(".");
    a.forEach(function (d, i) {
      a[i] = parseInt(d, 10);
    });

    ret = a[3] + a[2] * 256 +
      a[1] * 256 * 256 +
      a[0] * 256 * 256 * 256;

    return ret;
  },
  copyHashArray: function (array) {
    var newArray = [];

    array.forEach(function (d) {
      var hash = {};

      for (var attr in d) {
        if (d.hasOwnProperty(attr)) hash[attr] = d[attr];
      }

      newArray.push(hash);
    });

    return newArray;
  },
  scale: function (num, multiplier) {
    var values = [1, ""];

    if (typeof multiplier === 'undefined'){
      var base = 10;

      if (num >= Math.pow(base, 3) && num < Math.pow(base, 6)) {
        values[0] = Math.pow(base, -3);
        values[1] = "K";
      } else if (num >= Math.pow(base, 6) && num < Math.pow(base, 9)) {
        values[0] = Math.pow(base, -6);
        values[1] = "M";
      } else if (num >= Math.pow(base, 9) && num < Math.pow(base, 12)) {
        values[0] = Math.pow(base, -9);
        values[1] = "G";
      } else if (num >= Math.pow(base, 12)) {
        values[0] = Math.pow(base, -12);
        values[1] = "T";
      }
    } else if (typeof(multiplier) === 'function') {
      values = multiplier(num);
    }

    return {
      multiplier: values[0],
      prefix: values[1],
      scaledNum: (num * values[0]),
      num: num
    };
  },
  transform: function (n, offset, reference) {
    return 100 * (n - offset) / (reference - offset);
  }
};

RB.Balls = function (options) {
  var that = this;

  // General properties
  this.options = options;
  this.elements = RB.util.default(options.elements, []);
  this.margin = RB.util.default(options.margin, { top: 5, horizontal: 5, bottom: 5 });
  this.total = options.total;
  this.width = options.width;
  this.height = options.height;
  this.textHeight = 20;
  // If you need reference to use this inside callbacks
  this.object = RB.util.default(options.object, undefined);
  // Max number of balls before create "Others"
  this.maxBalls = RB.util.default(options.maxBalls, 4);
  this.othersTitle = RB.util.default(options.othersTitle, 'Others');
  this.callback = RB.util.default(options.callback, undefined);

  // Calc total:
  if(this.total === undefined && this.elements.length > 0){
    this._calcTotal();
  }

  // Order elements
  this._orderElements();

  // Draw the svg element and the circle
  this.svg = d3.select('#' + options.container)
    .append("svg")
    .attr("class", "rbcharts rb-balls");

  // Draw the elements
  this.resize();
};

// Proptotype functions
RB.Balls.prototype = {
  resize: function (width, height) {
    var that = this, w, h;

    // Get w and h
    w = width  || RB.util.default(this.options.width, d3.select("#" + this.options.container).style("width"));
    h = height || RB.util.default(this.options.height, d3.select("#" + this.options.container).style("height"));
    w = parseInt(w, 10);
    h = parseInt(h, 10);

    // Default width of 100
    if (isNaN(w) && isNaN(h)) {
      w = 100;
    } else if (isNaN(w) && !isNaN(h)){
      w = 0;
    } else if (isNaN(h) && !isNaN(w)){
      h = 0;
    }

    // Save the width
    that.width = w;
    that.height = h;

    // Set radius based on the smaller length
    if(w < h){
      this.radius = w / 2 - this.margin.horizontal;
    } else {
      this.radius = h / 2 - this.margin.top - this.margin.bottom;
    }

    // Max radius and separation for elements
    this.maxRadius = (this.height - this.margin.top - this.textHeight - this.margin.bottom) / 2;
    if (this.elements.length > 1){
      this.separation = (this.width - this.maxRadius * 2) / (this.elements.length - 1);
    } else {
      this.separation = this.width / 2;
    }

    this.svg
      .attr("width", that.width)
      .attr("height", that.height);

    /*this.valueText
      .style("font-size", this.radius / 3 + "px")
      .attr("transform", "translate(" + (that.width/2) + ", " + (this.height/2) + ")");*/

    // Draw the elements
    this.draw();
  },
  setValue: function (value) {
    this.value = value;
    this.draw();
  },
  setElements: function(elements) {
    this.elements = elements;
    this._calcTotal();
    this._orderElements();
    this.draw();
  },
  // Calculate total
  _calcTotal: function() {
    this.total = 0;
    // Iterate over elements to calc the total
    for(var i = 0; i < this.elements.length; i++){
      this.total += this.elements[i].value;
    }
  },
  // Order elements and check the limit
  _orderElements: function() {
    this.elements = this.elements.sort(function(a, b){
      return b.value - a.value;
    });

    // Check limit to group
    if (this.elements.length > this.maxBalls){
      // Group
      grouped = [];
      for(var i = 0; i < this.elements.length; i++){
        if (i < (this.maxBalls - 1)){
          grouped.push(this.elements[i]);
        } else if (i == this.maxBalls) {
          grouped.push({
            name: this.othersTitle,
            value: this.elements[i].value
          });
        } else {
          grouped[grouped.length].value += this.elements[i].value;
        }
      }

      this.elements = grouped;
    }
  },
  draw: function () {
    var that = this, selection;

    select = this.svg.selectAll("g.element")
      .data(that.elements);

    var elEnter = select.enter()
      .append('g')
      .attr("class", function(d, i){
        return i === 0 ? "element active" : "element";
      });

    var circle = elEnter.append('circle')
      .attr("class", "rb-circle")
      .attr("cx", function(d, i){
        if (that.elements.length > 1) {
          return i * that.separation + that.maxRadius;
        } else {
          return that.separation;
        }
      })
      .attr("cy", function(d){
        return that.maxRadius + that.margin.top + that.textHeight;
      })
      .attr("r", function(d){
        var rad = (d.value * that.maxRadius) / that.total;
        return rad < 15 ? 15 : rad;
      })
      .on("click", function(e) {
        that.onClickElement(e, this);
      });

    elEnter.append('text')
      .attr("dx", function(d, i){
        if (that.elements.length > 1){
          return i * that.separation + that.maxRadius;
        } else {
          return that.separation;
        }
      })
      .attr("dy", function(d){
        return that.maxRadius + that.margin.top + that.textHeight + 5;
      })
      .attr("class", "rb-value")
      .text(function(d) {
        return d.value;
      })
      .on("click", function(e) {
        that.onClickElement(e, this);
      });

    elEnter
      .append('text')
      .text(function(d) {
        return d.name === "" ? "Undefined" : d.name;
      })
      .attr("class", "rb-title")
      .attr("dx", function(d, i){
        if (that.elements.length > 1){
          return i * that.separation + that.maxRadius;
        } else {
          return that.separation;
        }
      })
      .attr("dy", function(d){
        // Add some margin to show all text
        return that.margin.top * 2 + 3;
      });
  },
  // Set an element as active
  setActive: function(name){
    var group = this.svg.selectAll('g').filter(function(data){
      if (data.name === name){
        return true;
      }
    });
    // Check if there are a group with this name
    if (group.length === 0){
      return;
    }

    // Check if it's active
    if (!group[0][0].classList.contains("active")) {
      // Add active and remove it from other
      this.svg.select('g.active')[0][0].classList.remove('active');
      group[0][0].classList.add("active");
    }
  },
  onClickElement: function(event, context) {
    var name;
    var group = context.parentNode;

    if (!group.classList.contains("active")) {
      // Add active and remove it from other
      this.svg.select('g.active')[0][0].classList.remove('active');
      group.classList.add("active");
    }
    // Get name
    name = this.svg.select('g.active text.rb-title')[0][0].innerHTML;

    // Call on click if it's defined
    if (typeof this.callback === 'function') {
      /*
          Use the "call" method so that inside of the onComplete
          callback function the "this" keyword refers to the
          specific DOM node that called the plugin.

          More: https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Function/call
      */
      this.callback.call(this.object, name, group);
    }
  }
};

RB.Nodes = function (options) {
  var that = this;
  // General properties
  this.options = options;
  // Elements to draw
  this.nodes = RB.util.default(options.nodes, []);
  this.alignFirstNode = RB.util.default(options.alignFirstNode, false);
  this.links = RB.util.default(options.links, []);
  this.unify = RB.util.default(options.unify, true);
  this.sources = [];
  // Margin and style
  this.margin = RB.util.default(options.margin, { top: 5, horizontal: 5, bottom: 5 });
  this.sourceColor = RB.util.default(options.sourceColor, undefined);
  this.strokeRange = RB.util.default(options.strokeRange, [1, 2]);
  this.colorRange = RB.util.default(options.colorRange, ['#bfbfbf', '#262626']);
  this.startColor = RB.util.default(options.startColor, "#cbcbcb");
  this.nodeColor = RB.util.default(options.nodeColor, "#cbcbcb");
  this.activeColor = RB.util.default(options.activeColor, "#EC8B84");
  this.activeLinkColor = RB.util.default(options.activeLinkColor, "#F86A6A");
  this.width = options.width;
  this.height = options.height;
  // If you need reference to use this inside callbacks
  this.object = RB.util.default(options.object, undefined);
  // Function to call on click elements
  this.clickCallback = RB.util.default(options.clickCallback, undefined);
  // Highlight interactions
  this.highlightInteraction = RB.util.default(options.highlightInteraction, true);
  // To control drag and mouseover event
  this.dragging = false;

  // Check if we have data
  if (this.nodes.length > 0) {
    // Unify links
    if (this.unify) { this._unifyLinks(); }
    // Calculate sources if we want to color them
    if (this.sourceColor !== undefined) { this.sources = this._getSources(); }
    // Calculate domain
    this._calcDomain();
  }

  // Draw the svg element
  this.svg = d3.select(options.container)
    .append("svg")
    .attr("class", "rbcharts rb-nodes");

  // Draw the elements
  this.resize();
};

// Proptotype functions
RB.Nodes.prototype = {
  // Resize the SVG and draw elements
  resize: function (width, height) {
    var that = this, w, h;

    // Get w and h
    w = width  || RB.util.default(this.options.width, d3.select(this.options.container).style("width"));
    h = height || RB.util.default(this.options.height, d3.select(this.options.container).style("height"));
    w = parseInt(w, 10);
    h = parseInt(h, 10);

    // Default width of 100
    if (isNaN(w) && isNaN(h)) {
      w = 100;
    } else if (isNaN(w) && !isNaN(h)){
      w = 0;
    } else if (isNaN(h) && !isNaN(w)){
      h = 0;
    }

    // Save the width
    that.width = w;
    that.height = h;

    this.svg
      .attr("width", that.width)
      .attr("height", that.height);

    // Align first element
    if (this.alignFirstNode) {
      that.nodes.forEach(function(node){
        if (node.start) {
          node.fixed = true;
          node.x = (that.width / 2) - that.nodes.length * 11;
          node.y = that.height / 2;
        }
      });
    }

    // Draw the elements
    if (this.nodes.length > 0) {
      this.draw();
    } else {
      this.drawNoData();
    }
  },
  // Draw elements
  draw: function() {
    var that = this;
    // Initialize the layout
    this.force = d3.layout.force()
      .gravity(0.05)
      .distance(100)
      .charge(-100)
      .linkStrength(function(l) { return that.stregthScale(l.value); })
      .size([that.width, that.height])
      .nodes(that.nodes)
      .links(that.links)
      .start();

    // Build arrows. Code from: http://bl.ocks.org/d3noob/5141278
    this.svg.append("svg:defs").selectAll("marker")
      .data(["end"])      // Different link/path types can be defined here
      .enter().append("svg:marker")    // This section adds in the arrows
        .attr("id", String)
        .attr("viewBox", "0 -5 10 10")
        .attr("refX", 15)
        .attr("refY", -1.5)
        .attr("markerWidth", 6)
        .attr("markerHeight", 6)
        .attr("orient", "auto")
        .append("svg:path")
          .attr("d", "M0,-5L10,0L0,5");

    // Path of the links
    var path = this.svg.append("svg:g").selectAll("path")
      .data(that.links)
      .enter().append("svg:path")
      //.attr("class", function(d) { return "link " + d.type; })
        .attr("class", "link")
        .attr("stroke-width", function(l) { return that.strokeScale(l.value); })
        .attr("stroke", function(l) { return l.color || that.colorScale(l.value); })
        .attr("marker-end", "url(#end)");

    // Draw nodes
    var node = this.svg.selectAll(".node")
                .data(that.nodes)
                .enter().append("g")
                  .attr("class", "node")
                  .call(that.force.drag);
    // Prevent over interaction when dragging
    that.force.drag()
      .on("dragstart", function() {
        that.dragging = true;
      })
      .on("dragend", function() {
        that.dragging = false;
      });

    // Circle and textx
    node.append("circle")
      .attr("r", 6)
      .attr("fill", function(d, i) { return that._getColor(that, d, i); })
      .attr("cx", 0)
      .attr("cy", 0)
      .on("click", function(d, i) {
        // Check if the event is a click
        if (d3.event.defaultPrevented) return;
        that.svg.selectAll('circle.active').attr('class', '');
        this.classList.add('active');
        // Check callback
        // Call on click if it's defined
        if (typeof that.clickCallback === 'function') {
          that.clickCallback.call(that.object, d, i);
        }
      })
      .on("mouseover", function(d, i) {
        if (that.highlightInteraction && !that.dragging) { that.highlight(d.name); }
      })
      .on("mouseout", function(d, i) {
        if (that.highlightInteraction && !that.dragging) { that.unhighlight(); }
      });
    // Text
    node.append("text")
      .attr("dx", 12)
      .attr("dy", ".35em")
      .text(function(d) { return d.name; });

    // Tick event.
    that.force.on("tick", function() {
      path.attr("d", function(d) {
        var dx = d.target.x - d.source.x,
            dy = d.target.y - d.source.y,
            dr = Math.sqrt(dx * dx + dy * dy);
        return "M" +
            d.source.x + "," +
            d.source.y + "A" +
            dr + "," + dr + " 0 0,1 " +
            d.target.x + "," +
            d.target.y;
      });

      node
        .attr("transform", function(d) {
        return "translate(" + d.x + "," + d.y + ")"; });
    });
  },
  // Draw a box and a text indicating there are not events
  drawNoData: function() {
    var that = this;
    // Append the group with the elements
    noData = this.svg.append("g").attr("class", "no-data");
    noData.append('rect')
      .attr('height', 150)
      .attr('width', 350)
      .attr('x', (that.width / 2) - 175)
      .attr('y', (that.height / 2) - 75);
    noData.append('text')
      .attr('x', that.width / 2)
      .attr('y', (that.height / 2) + 8)
      .text('There is no data');
  },
  // Calculate domain and scales
  _calcDomain: function() {
    this.domain = d3.extent(this.links, function(el) { return +el.value; });
    this.stregthScale = d3.scale.linear().domain(this.domain).range([0, 1]);
    this.colorScale = d3.scale.linear().domain(this.domain).range(this.colorRange);
    this.strokeScale = d3.scale.linear().domain(this.domain).range(this.strokeRange);
  },
  // Group equal links in one and sum values
  _unifyLinks: function() {
    var unified_links = [];
    // Iterate hover the links
    this.links.forEach(function(element, index, arr){
      var unify_index;

      unified_links.forEach(function(uni_el, uni_index, uni_arr){
        if(uni_el.source == element.source && uni_el.target == element.target){
          unify_index = uni_index;
        }
      });

      if(unify_index !== undefined) {
        unified_links[unify_index].value += element.value;
      } else {
        unified_links.push(element);
      }
    });
    // Set the links
    this.links = unified_links;
  },
  // Return an array with nodes that are sources
  _getSources: function() {
    var that = this;
    // Initialize sources as nodes
    var sources = this.nodes.map(function(node) {
      return node.name;
    });
    // Get target elements
    var targets = this.links.map(function(link) {
      return that.nodes[link.target].name;
    });
    // Unique targets
    targets = this._unique(targets);
    // Return source elements
    return this._arrayDiff(sources, targets);
  },
  // Disable listeners on Nodes
  disableHighlightInteraction: function() {
    this.highlightInteraction = false;
  },
  // Enable listeners on Nodes
  enableHighlightInteraction: function() {
    this.highlightInteraction = true;
  },
  // Highlight nodes and links related to a name.
  highlight: function(name) {
    var actives = this._highlightLinks(name);
    this._highlightNodes(actives);
  },
  // Unhighlight nodes and links
  unhighlight: function() {
    this._unhighlightNodes();
    this._unhighlightLinks();
  },
  // Highlight nodes based on an array of names
  _highlightNodes: function(highlightNodes) {
    var that = this;
    // Nodes
    that.nodes.forEach(function(node) {
      if(highlightNodes.indexOf(node.name) > -1){
        node.activeColor = that.activeColor;
      } else {
        node.activeColor = undefined;
      }
    });
    // Apply changes
    that.svg.selectAll("g.node circle")
      .attr("fill", function(d, i) { return that._getColor(that, d, i); })
      .attr("opacity", function(d) { return d.activeColor === undefined ? 0.4 : 1; });
    that.svg.selectAll("g.node text")
      .attr("opacity", function(d) { return d.activeColor === undefined ? 0.4 : 1; });
  },
  // Unhighlight nodes
  _unhighlightNodes: function() {
    var that = this;
    this.nodes.forEach(function(node) { node.activeColor = undefined; });
    this.svg.selectAll("g.node circle")
      .attr("fill", function(d, i) { return that._getColor(that, d, i); })
      .attr("opacity", 1);
    this.svg.selectAll("g.node text").attr("opacity", 1);
  },
  // Highlight links based on a name of a node. The links that has the element
  // in target or source will be highlighted.
  _highlightLinks: function(name) {
    var that = this;
    var actives = [];
    // Get links
    that.links.forEach(function(link) {
      if (link.source.name.indexOf(name) > -1 || link.target.name.indexOf(name) > -1 ) {
        link.color = that.activeLinkColor;
        actives.push(link.source.name);
        actives.push(link.target.name);
      } else {
        link.color = undefined;
      }
    });
    // Apply changes to links
    that.svg.selectAll("path")
      .attr("stroke", function(l) { return l.color || that.colorScale(l.value); })
      .attr("opacity", function(l) { return l.color === undefined ? 0.4 : 1; });
    // Return active nodes.
    return actives;
  },
  // Unhighlight links
  _unhighlightLinks: function() {
    var that = this;
    this.links.forEach(function(link) { link.color = undefined; });
    // Apply changes
    that.svg.selectAll("path")
      .attr("stroke", function(l) { return l.color || that.colorScale(l.value); })
      .attr('opacity', 1);
  },
  // Select the color for a node based on given options
  _getColor: function(that, d, i) {
    if (d.start) {
      return that.startColor;
    } else if (that.sourceColor !== undefined && that.sources.indexOf(d.name) > -1) {
      return that.sourceColor;
    } else if (d.activeColor !== undefined) {
      return d.activeColor;
    } else {
      return that.nodeColor;
    }
  },
  // Get the index of the given node
  _getNodeIndex: function(nodes, name) {
    var index;
    // Itearate over nodes
    nodes.forEach(function(node, i) {
      if (node.name === name) { index = i; }
    });
    // Return the index
    return index;
  },
  // Return an array as Unique.
  _unique: function(array) {
    return array.filter(function(value, index, arr){
      return array.indexOf(value) === index;
    });
  },
  // Return the diff of the Array
  _arrayDiff: function(array, array_diff) {
    return array.filter(function(value, index, arr) {
      return array_diff.indexOf(value) < 0;
    });
  }
};
RB.RelationsGraph = function (options) {
  var that = this;
  // General properties
  this.options = options;
  // Elements to draw
  this.nodes = RB.util.default(options.nodes, []);
  this.alignFirstNode = RB.util.default(options.alignFirstNode, false);
  this.links = RB.util.default(options.links, []);
  this.unify = RB.util.default(options.unify, true);
  this.sources = [];
  // Margin and style
  this.margin = RB.util.default(options.margin, {
    top: 5,
    horizontal: 5,
    bottom: 5,
  });
  this.sourceColor = RB.util.default(options.sourceColor, undefined);
  this.strokeRange = RB.util.default(options.strokeRange, [1, 2]);
  this.colorRange = RB.util.default(options.colorRange, [
    "#bfbfbf",
    "#262626",
  ]);
  this.startColor = RB.util.default(options.startColor, "#cbcbcb");
  this.nodeColor = RB.util.default(options.nodeColor, "#cbcbcb");
  this.activeColor = RB.util.default(options.activeColor, "#EC8B84");
  this.activeLinkColor = RB.util.default(options.activeLinkColor, "#F86A6A");
  this.width = options.width;
  this.height = options.height;
  // If you need reference to use this inside callbacks
  this.object = RB.util.default(options.object, undefined);
  // Function to call on click elements
  this.clickCallback = RB.util.default(options.clickCallback, undefined);
  // Highlight interactions
  this.highlightInteraction = RB.util.default(
    options.highlightInteraction,
    true
  );
  // To control drag and mouseover event
  this.dragging = false;

  // Check if we have data
  if (this.nodes.length > 0) {
    // Unify links
    if (this.unify) {
      this._unifyLinks();
    }
    // Calculate sources if we want to color them
    if (this.sourceColor !== undefined) {
      this.sources = this._getSources();
    }
    // Calculate domain
    this._calcDomain();
  }

  // Draw the svg element
  this.svg = d3
    .select(options.container)
    .append("svg")
    .attr("class", "rbcharts rb-nodes");

  // Add zoom behavior
  this.zoom = d3.behavior
    .zoom()
    .scaleExtent([0.5, 5])
    .on("zoom", function () {
      that.svg
        .select("g")
        .attr(
          "transform",
          "translate(" + d3.event.translate + ")scale(" + d3.event.scale + ")"
        );
    });

  this.svg.call(this.zoom);

  // Create a group for all elements to support zooming
  this.containerGroup = this.svg.append("g");

  // Draw the elements
  this.resize();

  // Handle window resize
  window.addEventListener("resize", function () {
    that.resize();
  });

  // If there is elements
  if (this.nodes.length > 0) {
    this.draw();
  }
};

// Proptotype functions
RB.RelationsGraph.prototype = {
  // Resize the SVG and draw elements
  resize: function (width, height) {
    var that = this,
      w,
      h;

    // Get w and h
    w =
      width ||
      RB.util.default(
        this.options.width,
        d3.select(this.options.container).node().clientWidth
      );
    h =
      height ||
      RB.util.default(
        this.options.height,
        d3.select(this.options.container).node().clientHeight
      );
    w = parseInt(w, 10);
    h = parseInt(h, 10);

    // Default width of 100
    if (isNaN(w) && isNaN(h)) {
      w = 100;
    } else if (isNaN(w) && !isNaN(h)) {
      w = 0;
    } else if (isNaN(h) && !isNaN(w)) {
      h = 0;
    }

    // Save the width
    that.width = w;
    that.height = h;

    this.svg.attr("width", that.width).attr("height", that.height);

    // Align first element
    if (this.alignFirstNode) {
      that.nodes.forEach(function (node) {
        if (node.start) {
          node.fixed = true;
          node.x = that.width / 2 - that.nodes.length * 11;
          node.y = that.height / 2;
        }
      });
    }

    // If there is no elements
    if (this.nodes.length <= 0) {
      this.drawNoData();
    }
  },
  // Draw elements
  draw: function () {
    var that = this;
    // Offset curve
    var curve = d3.svg.line().interpolate("cardinal-closed").tension(1);
    var off = 65, // cluster hull offset
      expand = {}, // collapsed clusters
      data = {
        nodes: that.nodes,
        links: that.links,
      },
      net,
      force,
      hullg,
      linkg,
      nodeg,
      imagesg,
      textg;

    // Initial call to setup the graph
    setTimeout(function () {
      for (var i = 0; i < data.links.length; ++i) {
        var o = data.links[i];
        o.source = data.nodes[o.source];
        o.target = data.nodes[o.target];
      }

      hullg = that.containerGroup.append("g");
      linkg = that.containerGroup.append("g");
      nodeg = that.containerGroup.append("g");
      imagesg = that.containerGroup.append("g");
      textg = that.containerGroup.append("g");

      // Create nodes and group nodes info
      init();
      that._createInfoToGroupNodes(
        nodeg.selectAll(".node").data(net.nodes, nodeid)
      );
    }, 0);

    function nodeid(n) {
      return n.size ? "_g_" + n.type : n.name;
    }

    function linkid(l) {
      var u = nodeid(l.source),
        v = nodeid(l.target);
      return u < v ? u + "|" + v : v + "|" + u;
    }

    function getType(n) {
      return n.type;
    }

    // constructs the network to visualize
    function network(data, prev, index, expand) {
      expand = expand || {};
      var gm = {}, // group map
        nm = {}, // node map
        lm = {}, // link map
        gn = {}, // previous group nodes
        gc = {}, // previous group centroids
        nodes = [], // output nodes
        links = []; // output links

      // process previous nodes for reuse or centroid calculation
      if (prev) {
        prev.nodes.forEach(function (n) {
          var i = index(n),
            o;
          if (n.size > 0) {
            gn[i] = n;
            n.size = 0;
          } else {
            o = gc[i] || (gc[i] = { x: 0, y: 0, count: 0 });
            o.x += n.x;
            o.y += n.y;
            o.count += 1;
          }
        });
      }

      // determine nodes
      for (var k = 0; k < data.nodes.length; ++k) {
        var n = data.nodes[k],
          i = index(n),
          l =
            gm[i] ||
            (gm[i] = gn[i]) ||
            (gm[i] = { type: i, size: 0, nodes: [] });

        if (expand[i]) {
          // the node should be directly visible
          nm[n.name] = nodes.length;
          nodes.push(n);
          if (gn[i]) {
            // place new nodes at cluster location (plus jitter)
            n.x = gn[i].x + Math.random();
            n.y = gn[i].y + Math.random();
          }
        } else {
          // the node is part of a collapsed cluster
          if (l.size == 0) {
            // if new cluster, add to set and position at centroid of leaf nodes
            nm[i] = nodes.length;
            nodes.push(l);
            if (gc[i]) {
              l.x = gc[i].x / gc[i].count;
              l.y = gc[i].y / gc[i].count;
            }
          }
          l.nodes.push(n);
        }
        // always count group size as we also use it to tweak the force graph strengths/distances
        l.size += 1;
        n.group_data = l;
      }

      for (i in gm) {
        gm[i].link_count = 0;
      }

      // determine links
      for (k = 0; k < data.links.length; ++k) {
        var e = data.links[k],
          u = index(e.source),
          v = index(e.target);
        if (u != v) {
          gm[u].link_count++;
          gm[v].link_count++;
        }
        u = expand[u] ? nm[e.source.name] : nm[u];
        v = expand[v] ? nm[e.target.name] : nm[v];
        var i = u < v ? u + "|" + v : v + "|" + u,
          l = lm[i] || (lm[i] = { source: u, target: v, size: 0 });
        l.size += 1;
      }
      for (i in lm) {
        links.push(lm[i]);
      }

      return { nodes: nodes, links: links };
    }

    function rectangularHull(nodes, index, offset) {
      var hulls = {};

      // Create point sets and calculate bounding boxes
      for (var k = 0; k < nodes.length; ++k) {
        var n = nodes[k];
        if (n.size) continue;
        var i = index(n),
          l =
            hulls[i] ||
            (hulls[i] = {
              minX: Infinity,
              maxX: -Infinity,
              minY: Infinity,
              maxY: -Infinity,
            });

        // Update bounding box coordinates
        l.minX = Math.min(l.minX, n.x);
        l.maxX = Math.max(l.maxX, n.x);
        l.minY = Math.min(l.minY, n.y);
        l.maxY = Math.max(l.maxY, n.y);
      }

      // Finalize bounding box and create rectangle paths
      var hullset = [];
      for (var i in hulls) {
        var hull = hulls[i];
        // Adjust bounding box with offset
        var minX = hull.minX - offset;
        var maxX = hull.maxX + offset;
        var minY = hull.minY - offset;
        var maxY = hull.maxY + offset;

        // Generate rectangle corners
        var rectPath = [
          [minX, minY],
          [maxX, minY],
          [maxX, maxY],
          [minX, maxY],
          [minX, minY], // Closing the rectangle path
        ];

        hullset.push({ type: i, path: rectPath });
      }

      return hullset;
    }

    function drawCluster(d) {
      return curve(d.path);
    }

    function clickNode(d) {
      expand[d.type] = !expand[d.type];

      // Group node info
      if (expand[d.type]) {
        init();
        that._updateInfoToGroupNodes(
          "",
          nodeg.selectAll(".node").data(net.nodes, nodeid),
          expand
        );
      } else {
        // Normal node info
        const simulatedClick = new MouseEvent("click", {
          clientX: event.clientX,
          clientY: event.clientY,
          bubbles: true,
          cancelable: true,
          view: window,
        });

        event.target.dispatchEvent(simulatedClick);

        observable = {
          image: d.image,
          name: d.name,
          type: d.type,
        };

        showObservableInfo(observable);
      }
    }

    function init() {
      if (force) force.stop();
      net = network(data, net, getType, expand);
      force = d3.layout
        .force()
        .linkStrength(function () {
          return 1;
        })
        .gravity(0.01)
        .charge(-2500)
        .friction(0.5)
        .nodes(net.nodes)
        .links(net.links)
        .size([that.width, that.height])
        .linkDistance(function (l) {
          var n1 = l.source,
            n2 = l.target;
          return (
            150 + // Further increased base distance
            Math.min(
              90 * // Further increased size multiplier
                Math.min(
                  n1.size || (n1.type !== n2.type ? n1.group_data.size : 0),
                  n2.size || (n1.type !== n2.type ? n2.group_data.size : 0)
                ),
              20 + // Further adjusted to allow for more positive offset
                90 * // Further increased link count multiplier
                  Math.min(
                    n1.link_count ||
                      (n1.type !== n2.type ? n1.group_data.link_count : 0),
                    n2.link_count ||
                      (n1.type !== n2.type ? n2.group_data.link_count : 0)
                  ),
              400 // Further increased maximum cap
            )
          );
        })
        .start();

      // Render offset of the grouping nodes
      var hullNode = hullg
        .selectAll("path.hull")
        .data(rectangularHull(net.nodes, getType, off));
      hullNode.exit().remove();

      hullNode
        .enter()
        .append("path")
        .attr("class", "hull")
        .style("fill", "#fff")
        .attr("stroke-dasharray", "24")
        .attr("d", drawCluster)
        .on("mousedown", function (d) {
          var dragging = 0;

          const mouseMoveHandler = function () {
            dragging++;
          };

          // Add the mousemove event listener to handle dragging
          window.addEventListener("mousemove", mouseMoveHandler);

          // Remove the mousemove event listener on mouseup
          window.addEventListener("mouseup", function mouseUpHandler() {
            window.removeEventListener("mousemove", mouseMoveHandler); // Stop dragging
            window.removeEventListener("mouseup", mouseUpHandler); // Clean up the mouseup listener

            // Click
            if (dragging <= 3) {
              expand[d.type] = false;
              init();
              that._updateInfoToGroupNodes(
                "show",
                nodeg.selectAll(".node").data(net.nodes, nodeid),
                expand
              );
            }
          });
        });

      // Define arrow markers for graph links
      linkg
        .append("svg:defs")
        .selectAll("marker")
        .data(["end"])
        .enter()
        .append("svg:marker")
        .attr("id", String)
        .attr("viewBox", "0 -5 10 10")
        .attr("refX", 0)
        .attr("refY", 0)
        .attr("markerWidth", 8)
        .attr("markerHeight", 8)
        .attr("orient", "auto")
        .append("svg:path")
        .attr("d", "M0,-5L10,0L0,5");

      // Select link elements and bind data
      var link = linkg.selectAll("path.link").data(net.links, linkid);
      link.exit().remove();
      link
        .enter()
        .append("path")
        .attr("class", "link")
        .attr("stroke-width", function (l) {
          return that.strokeScale(l.value);
        })
        .attr("stroke", "#bfbfbf")
        .attr("marker-end", "url(#end)");

      // Append circle for each node
      var nodeEnter = nodeg
        .selectAll(".node")
        .data(net.nodes, nodeid)
        .call(force.drag);
      nodeEnter.exit().remove();

      nodeEnter
        .enter()
        .append("circle")
        .attr("class", function (d) {
          return "node" + (d.size ? "" : " leaf");
        })
        .attr("r", 32)
        .attr("cx", function (d) {
          return d.x;
        })
        .attr("cy", function (d) {
          return d.y;
        })
        .style("fill", "#fff")
        .attr("stroke", function (d) {
          return that._getColor(that, d);
        })
        .attr("stroke-width", 1)
        .attr("stroke-dasharray", "4 1")
        .on("click", clickNode);

      // Append circle for each node
      var nodeImageEnter = imagesg
        .selectAll(".image")
        .data(net.nodes, nodeid)
        .call(force.drag);
      nodeImageEnter.exit().remove();

      nodeImageEnter
        .enter()
        .append("image")
        .attr("class", function (d) {
          return "node image" + (d.size ? "" : " leaf");
        })
        .attr("xlink:href", function (d) {
          return d.image;
        })
        .attr("width", 42)
        .attr("height", 42)
        .attr("x", function (d) {
          return d.x;
        })
        .attr("y", function (d) {
          return d.y;
        })
        .on("click", clickNode);

      // Append text for each node
      var nodeTextEnter = textg
        .selectAll(".node")
        .data(net.nodes, nodeid)
        .call(force.drag);
      nodeTextEnter.exit().remove();

      nodeTextEnter
        .enter()
        .append("text")
        .attr("class", function (d) {
          return "node" + (d.size ? "" : " leaf");
        })
        .text(function (d) {
          return d.name;
        })
        .attr("dx", function (d) {
          return d.x;
        })
        .attr("dy", function (d) {
          return d.y;
        })
        .attr("text-anchor", "middle")
        .style("font-size", "10px")
        .style("font-weight", "bold")
        .style("fill", function (d) {
          return that._getColor(that, d);
        });

      // Add mouse interaction
      that._mouseInteraction([nodeEnter, nodeImageEnter]);

      // Realtime method
      force.on("tick", function () {
        if (!hullNode.empty()) {
          hullNode
            .data(rectangularHull(net.nodes, getType, off))
            .attr("d", drawCluster);
        }

        // Update position of links
        link.attr("d", function (d) {
          var dx = d.target.x - d.source.x,
            dy = d.target.y - d.source.y,
            dr = Math.sqrt(dx * dx + dy * dy);
          return (
            "M" +
            d.source.x +
            "," +
            d.source.y +
            "A" +
            dr +
            "," +
            dr +
            " 0 0,1 " +
            d.target.x +
            "," +
            d.target.y
          );
        });

        // Update position of arrows
        link.attr("d", function (d) {
          const markerSize = 16.97;
          var pl = this.getTotalLength(),
            r = d.target.weight * 8 + markerSize,
            m = this.getPointAtLength(pl - r);

          var dx = m.x - d.source.x,
            dy = m.y - d.source.y,
            dr = Math.sqrt(dx * dx + dy * dy);

          return (
            "M" +
            d.source.x +
            "," +
            d.source.y +
            "A" +
            dr +
            "," +
            dr +
            " 0 0,1 " +
            m.x +
            "," +
            m.y
          );
        });

        // Update position of nodes
        nodeEnter
          .attr("cx", function (d) {
            return d.x;
          })
          .attr("cy", function (d) {
            return d.y;
          });

        // Update position of images
        nodeImageEnter
          .attr("x", function (d) {
            return d.x - 20;
          })
          .attr("y", function (d) {
            return d.y - 21;
          });

        // Update position of text
        nodeTextEnter
          .attr("dx", function (d) {
            return d.x;
          })
          .attr("dy", function (d) {
            return d.y + 45;
          });

        // Update circle, text and image positions of Group Nodes
        that.nodeCircleElements.forEach(function (circle, index) {
          const d = that.nodeData[index];
          circle.setAttribute("cx", d.x + 25);
          circle.setAttribute("cy", d.y - 25);
        });

        that.nodeTextElements.forEach(function (text, index) {
          const d = that.nodeData[index];
          text.setAttribute("dx", d.x + 25);
          text.setAttribute("dy", d.y - 21);
        });

        that.nodeTypeElements.forEach(function (type, index) {
          const d = that.nodeData[index];
          type.setAttribute("dx", d.x);
          type.setAttribute("dy", d.y + 45);
        });

        that.nodeImageElements.forEach(function (image, index) {
          const d = that.nodeData[index];
          image.setAttribute("x", d.x - 20);
          image.setAttribute("y", d.y - 21);
        });
      });

      return that.containerGroup.node();
    }
  },
  // Draw a box and a text indicating there are not events
  drawNoData: function () {
    var that = this;
    // Append the group with the elements
    noData = this.containerGroup.append("g").attr("class", "no-data");
    noData
      .append("rect")
      .attr("height", 150)
      .attr("width", 350)
      .attr("x", that.width / 2 - 175)
      .attr("y", that.height / 2 - 75);
    noData
      .append("text")
      .attr("x", that.width / 2)
      .attr("y", that.height / 2 + 8)
      .text("There is no data");
  },
  _getColor: function (that, d) {
    if (!d) return;

    if (d.start) return that.startColor;

    if (that.sourceColor && that.sources.includes(d.name)) {
      return that.sourceColor;
    }

    if (d.activeColor) return d.activeColor;

    return that.nodeColor;
  },
  // Calculate domain and scales
  _calcDomain: function () {
    this.domain = d3.extent(this.links, function (el) {
      return +el.value;
    });
    this.stregthScale = d3.scale.linear().domain(this.domain).range([0, 1]);
    this.colorScale = d3.scale
      .linear()
      .domain(this.domain)
      .range(this.colorRange);
    this.strokeScale = d3.scale
      .linear()
      .domain(this.domain)
      .range(this.strokeRange);
  },
  // Group equal links in one and sum values
  _unifyLinks: function () {
    var unified_links = [];
    // Iterate hover the links
    this.links.forEach(function (element) {
      var unify_index;

      unified_links.forEach(function (uni_el, uni_index) {
        if (
          uni_el.source == element.source &&
          uni_el.target == element.target
        ) {
          unify_index = uni_index;
        }
      });

      if (unify_index !== undefined) {
        unified_links[unify_index].value += element.value;
      } else {
        unified_links.push(element);
      }
    });
    // Set the links
    this.links = unified_links;
  },
  // Return an array with nodes that are sources
  _getSources: function () {
    var that = this;
    // Initialize sources as nodes
    var sources = this.nodes.map(function (node) {
      return node.name;
    });
    // Get target elements
    var targets = this.links.map(function (link) {
      return that.nodes[link.target].name;
    });
    // Unique targets
    targets = this._unique(targets);
    // Return source elements
    return this._arrayDiff(sources, targets);
  },
  // Add mouse interaction to nodes
  _mouseInteraction: function (nodes) {
    var that = this;

    nodes.forEach(function (node) {
      node.on("mouseover", function (d) {
        if (
          that.highlightInteraction &&
          !that.dragging &&
          d.name !== undefined
        ) {
          that.highlight(d.name);
        }
      });

      node.on("mouseout", function (d) {
        if (
          that.highlightInteraction &&
          !that.dragging &&
          d.name !== undefined
        ) {
          that.unhighlight();
        }
      });
    });
  },
  // Hide or create info from groupNodes
  _updateInfoToGroupNodes: function (action, groupNodes, type) {
    var that = this;

    if (action === "show") {
      that._createInfoToGroupNodes(groupNodes);
    } else {
      Object.keys(type).forEach(function (key) {
        if (type[key]) {
          // Generate the ID for the g element
          const gChildNodeId = "group-info-" + key;

          // Check if an element with the same ID exists
          const existingGChildNode = document.getElementById(gChildNodeId);
          if (existingGChildNode) {
            existingGChildNode.remove();
          }
        }
      });
    }
  },
  // Create images and text of group nodes
  _createInfoToGroupNodes: function (groupNodes) {
    var that = this;
    const circlesData = groupNodes[0];
    const gParentNode = document.querySelector(".rbcharts > g:nth-child(1)");

    // Store references to elements and their data
    that.nodeCircleElements = [];
    that.nodeTextElements = [];
    that.nodeTypeElements = [];
    that.nodeImageElements = [];
    that.nodeData = [];

    circlesData.forEach(function (circle) {
      if (circle.getAttribute("class") === "node") {
        const d = d3.select(circle).datum();

        // Generate the ID for the new g element
        const gChildNodeId = "group-info-" + d.type;

        // Check if an element with the same ID already exists
        const existingGChildNode = document.getElementById(gChildNodeId);
        if (existingGChildNode) existingGChildNode.remove();

        // Create a new g element to contain text and image
        const gChildNode = document.createElementNS(
          "http://www.w3.org/2000/svg",
          "g"
        );
        gChildNode.setAttribute("id", gChildNodeId);

        // Create circle node
        const groupCircle = document.createElementNS(
          "http://www.w3.org/2000/svg",
          "circle"
        );
        that.create_circle_node(groupCircle);

        // Create text node
        const text = document.createElementNS(
          "http://www.w3.org/2000/svg",
          "text"
        );
        that.create_text_node(text, d);

        // Create type node
        const type = document.createElementNS(
          "http://www.w3.org/2000/svg",
          "text"
        );
        that.create_type_node(type, d);

        // Create image node
        const image = document.createElementNS(
          "http://www.w3.org/2000/svg",
          "image"
        );
        that.create_image_node(image, d);

        // Flag to prevent recursive clicks
        var isSimulatedClick = false;

        // We hide the image to expand the node (z-index does not work with SVGs HTML tags)
        image.addEventListener("click", function (event) {
          if (isSimulatedClick) {
            // Reset the flag and exit if this is a simulated click
            isSimulatedClick = false;
            return;
          }

          // Hide the image as part of the action
          image.style.visibility = "hidden";

          // Set the flag before dispatching the event
          isSimulatedClick = true;

          // Simulate the click
          const simulatedClick = new MouseEvent("click", {
            clientX: event.clientX,
            clientY: event.clientY,
            bubbles: true,
            cancelable: true,
            view: window,
          });

          circle.dispatchEvent(simulatedClick);
          image.parentNode.remove();
        });

        // Append circle, text and image to the new g element
        gChildNode.appendChild(groupCircle);
        gChildNode.appendChild(text);
        gChildNode.appendChild(type);
        gChildNode.appendChild(image);

        // Append the new g element to the parent g node
        gParentNode.appendChild(gChildNode);

        // Store references to update the position in real-time
        that.nodeCircleElements.push(groupCircle);
        that.nodeTextElements.push(text);
        that.nodeTypeElements.push(type);
        that.nodeImageElements.push(image);
        that.nodeData.push(d);
      }
    });
  },
  create_image_node: function (imageNode, d) {
    imageNode.setAttribute("class", "group-image");
    imageNode.setAttribute("width", "42");
    imageNode.setAttribute("height", "42");
    imageNode.setAttribute("href", d.nodes[0].image);
  },
  create_text_node: function (textNode, d) {
    textNode.setAttribute("class", "group-text");
    textNode.textContent = d.size;
  },
  create_type_node: function (typeNode, d) {
    typeNode.setAttribute("class", "group-type");
    typeNode.textContent = d.type;
  },
  create_circle_node: function (circleNode) {
    circleNode.setAttribute("class", "group-circle");
    circleNode.setAttribute("r", 10);
  },
  // Disable listeners on Nodes
  disableHighlightInteraction: function () {
    this.highlightInteraction = false;
  },
  // Enable listeners on Nodes
  enableHighlightInteraction: function () {
    this.highlightInteraction = true;
  },
  // Highlight nodes and links related to a name.
  highlight: function (name) {
    var actives = this._highlightLinks(name);
    this._highlightNodes(actives);
  },
  unhighlight: function () {
    this._unhighlightNodes();
    this._unhighlightLinks();
  },
  // Highlight nodes based on an array of names
  _highlightNodes: function (highlightNodes) {
    var that = this;

    that.nodes.forEach(function (node) {
      if (highlightNodes.includes(node.name)) {
        node.activeColor = "#F86A6A";
      } else {
        node.activeColor = undefined;
      }
    });

    that.svg
      .selectAll("circle")
      .attr("fill", "white")
      .attr("stroke", function (d) {
        if (d !== undefined) {
          return d.activeColor || that.colorScale(d.value);
        }
      })
      .attr("stroke-width", 1)
      .attr("stroke-dasharray", "4 1")
      .attr("opacity", function (d) {
        if (d !== undefined) {
          return d.activeColor === undefined ? 0.4 : 1;
        }
      });

    that.svg.selectAll("text.node").attr("opacity", function (d) {
      return d.activeColor === undefined ? 0.4 : 1;
    });
  },
  _unhighlightNodes: function () {
    var that = this;
    this.nodes.forEach(function (node) {
      node.activeColor = undefined;
    });

    this.svg
      .selectAll("circle")
      .attr("fill", "white")
      .attr("stroke", function (d) {
        return that._getColor(that, d);
      })
      .attr("stroke-width", 1)
      .attr("stroke-dasharray", "4 1")
      .attr("opacity", 1);

    this.svg.selectAll("text.node").attr("opacity", 1);
  },
  _highlightLinks: function (name) {
    var that = this;
    var actives = [];

    // Collect active links based on the provided name
    that.links.forEach(function (link) {
      if (
        link.source.name.indexOf(name) > -1 ||
        link.target.name.indexOf(name) > -1
      ) {
        actives.push(link.source.name);
        actives.push(link.target.name);
      }
    });

    // Select all path links and apply stroke and opacity attributes
    that.svg.selectAll("path.link").each(function (l) {
      // Calculate if the link is active
      var isActive =
        l.size === 1 &&
        l.source.name !== undefined &&
        l.target.name !== undefined &&
        actives.some(function (activeName) {
          return l.source.name.includes(activeName);
        }) &&
        actives.some(function (activeName) {
          return l.target.name.includes(activeName);
        });

      // Set the color based on the active status
      l.color = isActive ? that.activeLinkColor : undefined;

      // Apply stroke color and opacity
      d3.select(this).attr("stroke", isActive ? "#F86A6A" : "#bfbfbf");
      d3.select(this).attr("opacity", isActive ? 1 : 0.4);
    });

    return actives;
  },
  _unhighlightLinks: function () {
    this.links.forEach(function (link) {
      link.color = undefined;
    });
    this.svg
      .selectAll("path.link")
      .attr("stroke", "#bfbfbf")
      .attr("opacity", 1);
  },
  // Get the index of the given node
  _getNodeIndex: function (nodes, name) {
    var index;
    // Itearate over nodes
    nodes.forEach(function (node, i) {
      if (node.name === name) {
        index = i;
      }
    });
    // Return the index
    return index;
  },
  // Return an array as Unique.
  _unique: function (array) {
    return array.filter(function (value, index, arr) {
      return array.indexOf(value) === index;
    });
  },
  // Return the diff of the Array
  _arrayDiff: function (array, array_diff) {
    return array.filter(function (value, index, arr) {
      return array_diff.indexOf(value) < 0;
    });
  },
  // Zoom functions
  zoomIn: function () {
    var scaleFactor = 1.2;
    var currentScale = this.zoom.scale();
    var newScale = this.zoom.scale() * scaleFactor;

    // Calculate the center of the current viewport
    var translate = this.zoom.translate();
    var x0 = translate[0];
    var y0 = translate[1];
    var w = this.svg.attr("width");
    var h = this.svg.attr("height");
    var centerX = w / 2;
    var centerY = h / 2;

    // Calculate the new translation to keep the center in place
    var newX = x0 - centerX * (newScale - currentScale);
    var newY = y0 - centerY * (newScale - currentScale);

    this.zoom.translate([newX, newY]);
    this.zoom.scale(newScale);
    this.zoom.event(this.svg);
  },
  zoomOut: function () {
    var scaleFactor = 1.2;
    var currentScale = this.zoom.scale();
    var newScale = currentScale / scaleFactor;

    // Calculate the center of the current viewport
    var translate = this.zoom.translate();
    var x0 = translate[0];
    var y0 = translate[1];
    var w = this.svg.attr("width");
    var h = this.svg.attr("height");
    var centerX = w / 2;
    var centerY = h / 2;

    // Calculate the new translation to keep the center in place
    var newX = x0 + centerX * (currentScale - newScale);
    var newY = y0 + centerY * (currentScale - newScale);

    this.zoom.translate([newX, newY]);
    this.zoom.scale(newScale);
    this.zoom.event(this.svg);
  },
  resetZoom: function () {
    this.zoom.scale(1);
    this.zoom.translate([0, 0]);
    this.zoom.event(this.svg);
  },
  // Fullscreen function
  fullScreen: function () {
    const elem = document.querySelector("#incident-chart-container");

    // Check if the document is currently in fullscreen mode
    if (
      !document.fullscreenElement &&
      !document.mozFullScreenElement &&
      !document.webkitFullscreenElement &&
      !document.msFullscreenElement
    ) {
      // If not in fullscreen, enter fullscreen mode
      if (elem.requestFullscreen) {
        elem.requestFullscreen();
      } else if (elem.mozRequestFullScreen) {
        elem.mozRequestFullScreen(); // Firefox
      } else if (elem.webkitRequestFullscreen) {
        elem.webkitRequestFullscreen(); // Chrome, Safari and Opera
      } else if (elem.msRequestFullscreen) {
        elem.msRequestFullscreen(); // IE/Edge
      }
    } else {
      // If in fullscreen, exit fullscreen mode
      if (document.exitFullscreen) {
        document.exitFullscreen();
      } else if (document.mozCancelFullScreen) {
        document.mozCancelFullScreen(); // Firefox
      } else if (document.webkitExitFullscreen) {
        document.webkitExitFullscreen(); // Chrome, Safari and Opera
      } else if (document.msExitFullscreen) {
        document.msExitFullscreen(); // IE/Edge
      }
    }
  },
};
RB.BallSerie = function (options) {
  var self = this;
  // General properties
  self.options = {
    container:           options.container,
    width:               options.width,
    height:              options.height,
    margin: {
      top:               RB.util.default((options.margin || {}).top, 2),
      bottom:            RB.util.default((options.margin || {}).bottom, 2),
      right:             RB.util.default((options.margin || {}).right, 2),
      left:              RB.util.default((options.margin || {}).left, 2)
    },
    timezone:            RB.util.default(options.timezone, "auto"),
    tooltip:             RB.util.default(options.tooltip, true),
    tooltip_date_format: d3.time.format(options.tooltip_date_format || "%a, %H:%M - %d/%m/%Y"),
    hoverColor:          RB.util.default(options.hoverColor, "#79B2F8"),
    defaultColor:        RB.util.default(options.defaultColor, RB.Common.palette[0]),
    mouseoverCallback:   options.mouseoverCallback,
    clickCallback:       options.clickCallback,
    mouseoutCallback:    options.mouseoutCallback
  };

  // Elements
  self.elements         = options.elements;
  self.render           = RB.util.default(options.render, true);
  self.chartid          = Math.random().toString().substring(2, 7);
  self.maxEvents        = 0;

  // Init domain
  if (options.domain) {
    self.domain = [];
    if (self.options.timezone === "auto") {
      self.domain[0] = new Date(options.domain[0] * 1000);
      self.domain[1] = new Date(options.domain[1] * 1000);
    } else {
      self.domain[0] = new Date(new Date(options.domain[0] * 1000).getTime() +
        self.options.timezone + new Date().getTimezoneOffset() * 60000);
      self.domain[1] = new Date(new Date(options.domain[1] * 1000).getTime() +
        self.options.timezone + new Date().getTimezoneOffset() * 60000);
    }
  } else {
    self.domain = [-1, -1];
  }

  // Scale functions
  self.x = d3.time.scale().domain(self.domain);
  self.radiusScale = d3.scale.linear();

  // Init Xaxis
  self.xAxis = d3.svg.axis()
    .scale(self.x)
    .orient("bottom")
    .tickFormat(function(d) {
      var diff = self.x.domain()[1] - self.x.domain()[0],
      using = d3.time.format,
      format;

      if (diff < 259200000) {
        format = using("%H:%M")(d);
      } else if (diff < 1036800000) {
        format = using("%a")(d);
      } else if (diff < 3456000000) {
        if (self.options.genericXAxis) {
          format = using("Day %d")(d);
        } else {
          format = using("%d %b")(d);
        }
      } else {
        format = using("%b")(d);
      }

      return format;
    });

  // Draw the svg element
  self.svg = d3.select(self.options.container)
    .append("svg")
    .attr("class", "rbcharts rb-ballserie");

  // Semi-top group (tooltip, etc)
  self.semitop = self.svg.append("g")
    .attr("id", "rbcharts-" + self.chartid + "-semitop");

  // Data
  self.board = self.svg.append("g")
    .attr("id", "rbcharts-" + self.chartid + "-board");
  // Draw the elements
  self.resize();
};

// Proptotype functions
RB.BallSerie.prototype = {
  // Resize the SVG and draw elements
  resize: function (width, height) {
    var self = this, w, h;

    // Get w and h
    w = width  || RB.util.default(self.options.width, d3.select(self.options.container).style("width"));
    h = height || RB.util.default(self.options.height, d3.select(self.options.container).style("height"));
    w = parseInt(w, 10);
    h = parseInt(h, 10);

    // Default width of 100
    if (isNaN(w) && isNaN(h)) {
      w = 100;
    } else if (isNaN(w) && !isNaN(h)){
      w = 0;
    } else if (isNaN(h) && !isNaN(w)){
      h = 0;
    }

    // Save the width
    self.width = w - self.options.margin.left - self.options.margin.right;
    self.height = h - self.options.margin.top - self.options.margin.bottom;

    // Set size of board. -20 due to the axis
    self.boardHeight = self.height - 20;
    self.options.margin.left += self.boardHeight / 2;
    self.options.margin.right += self.boardHeight / 2;
    self.board
      .attr("width", self.width - self.boardHeight)
      .attr("height", self.boardHeight)
      .attr("transform",
            "translate(" + self.options.margin.left + "," + self.options.margin.top + ")");

    // Resize scale functions
    self.x.range([0, self.width - self.boardHeight]);

    // Set size of SVG
    self.svg
      .attr("width", self.width)
      .attr("height", self.height);

    // Semi-top group (tooltip, etc)
    self.semitop
      .attr("width", self.width - self.boardHeight)
      .attr("transform",
            "translate(" + self.options.margin.left + "," + self.options.margin.top + ")");

    // Draw axis
    self._drawAxis();

    // Draw the elements
    if (self.elements.length > 0) {
      // Load timestamp based on timezone
      self.elements = self._buildData(self.elements);
      // Draw!!!
      self._calcRadiusScale();
      self.draw();
    } else {
      self.drawNoData();
    }
  },
  // Draw the graph
  draw: function() {
    var self = this,
        circles;

    // Draw circles
    circles = self.board.selectAll("circle")
      .data(self.elements);

    circles.enter()
      .append("circle")
      .attr("cx", function(d) { return self.x(d.timestamp); })
      .attr("cy", self.boardHeight / 2)
      .attr("r", function(d) { return self.radiusScale(d.y); })
      .attr("fill", function(d) { return self._getColor(self.options.defaultColor, d); })
      .on("click", function(d, i) {
        if (typeof self.options.clickCallback === 'function') {
          self.options.clickCallback.call(self, d, i);
        }
      })
      .on("mouseover", function(d, i) {
        d3.select(this).transition().attr('fill', self.options.hoverColor);
        if (typeof self.options.mouseoverCallback === 'function') {
          self.options.mouseoverCallback.call(self, d, i);
        }
      })
      .on("mouseout", function(d, i) {
        d3.select(this).transition().attr('fill', function(d) { return self._getColor(self.options.defaultColor, d); });
        if (typeof self.options.mouseoutCallback === 'function') {
          self.options.mouseoutCallback.call(self, d, i);
        }
      });
  },
  drawNoData: function() {

  },
  // Highlight a ball based on the timestamp
  highlight: function(timestamp) {
    var self = this;

    if (typeof timestamp !== 'number') {
      timestamp = timestamp.getTime();
    }

    // Highlight a circle
    self.board.selectAll("circle")
      .transition()
      .attr("fill", function(d) {
        if (d.timestamp.getTime() === timestamp){
          return self.options.hoverColor;
        } else {
          return self._getColor(self.options.defaultColor, d);
        }
      });
  },
  unhighlight: function() {
    var self = this;

    self.board.selectAll("circle")
      .transition()
      .attr("fill", function(d) { return self._getColor(self.options.defaultColor, d); });
  },
  // Calculate the radius scale
  _calcRadiusScale: function() {
    var self = this;
    self.maxRadius = (self.boardHeight - (self.boardHeight * 0.2)) / 2;
    // Calc max events
    self.maxEvents = d3.max(self.elements, function(el) { return el.y; });
    // Get the scale
    self.radiusScale.domain([0, self.maxEvents]).range([2.5, self.maxRadius]);
  },
  // Get the color of a node
  _getColor: function(defaultColor, d) {
    return d.color || defaultColor;
  },
  // Draw xAxis
  _drawAxis: function() {
    var self = this;
    // Append the axis
    self.semitop.append("g")
      .attr("class", "x axis")
      .attr("transform", "translate(0," + (self.height - 20) + ")");

    // Get the size of domain
    diff = self.x.domain()[1] - self.x.domain()[0];
    // Set the number of ticks
    if (diff > 259200000 && diff < 1036800000) {
      self.xAxis.ticks(d3.time.days, 1);
    } else if (self.width > 800) {
      self.xAxis.ticks(8);
    } else {
      self.xAxis.ticks(4);
    }
    // Add the axis to semitip
    self.semitop
      .select(".x.axis")
      .transition()
      .duration(500)
      .call(self.xAxis);
    // After set the elements, move the first and last ticks
    d3.timer(function () {
      self.svg.selectAll(".x.axis g")
      .each(function (d) {
        var parts, X, Y;

        parts = /translate\(\s*([^\s,)]+)[ ,]([^\s,)]+)/.exec(d3.select(this).attr("transform"));

        if (parts !== null) {
          X = parseInt(parts[1]);
          Y = parseInt(parts[2]);

          d3.select(this)
          .selectAll("text")
          .attr("x", function (d) {
            if (X <= 15) return -X + 8;
            else if (X >= self.width - 15) return -(X - self.width) - 8;
            else return 0;
          })
          .style("text-anchor", function (d) {
            if (X <= 15) return "start";
            else if (X >= self.width - 15) return "end";
            else return "middle";
          });
        }
      });
      return true;
    }, 600);
  },
  /* Utils */
  _buildData: function (elements) {
    var self = this, name,
      converted = [];

    // Process, clean and convert the input data
    elements.forEach(function (d, i) {
      var property, inside;
      converted[i] = {};

      // Parse the timestamp into Date object
      if (d.timestamp instanceof Date) {
        if (self.options.timezone === "auto") {
          converted[i].timestamp = d.timestamp;
        } else {
          converted[i].timestamp = new Date(d.timestamp.getTime() +
            self.options.timezone + new Date().getTimezoneOffset() * 60000);
        }
      } else {
        if (self.options.timezone === "auto") {
          converted[i].timestamp = new Date(d.timestamp.replace(/(\d*)-(\d*)-(\d*)/, "$1/$2/$3"));
        } else {
          converted[i].timestamp = new Date(new Date(d.timestamp.replace(/(\d*)-(\d*)-(\d*)/, "$1/$2/$3")).getTime() +
            self.options.timezone + new Date().getTimezoneOffset() * 60000);
        }
      }

      // Read the data
      for (property in d) {
        if (d.hasOwnProperty(property)) {
          if (property !== "version" && property !== "timestamp" && property !== "y") {
            for (inside in d[property]) {
              if (d[property].hasOwnProperty(inside)) {
                name = inside == 'events' ? 'y' : inside;
                if (self.options.dataFunction !== undefined) {
                  converted[i][name] = self.options.dataFunction(d[property][inside]);
                } else if(name == 'color') {
                  converted[i][name] = d[property][inside];
                } else {
                  converted[i][name] = parseFloat(d[property][inside]);
                }
              }
            }
          }
        }
      }
    });

    // Sort the data time-ascending
    if (converted.length > 1 &&
      d3.ascending(converted[0].timestamp, converted[1].timestamp) === 1) {
      converted.sort(function (a, b) {
        return d3.ascending(a.timestamp, b.timestamp);
      });
    }

    return converted;
  }
};

RB.Chords = function (opts) {

  // Defaults
  opts = opts || {};
  opts.size = opts.size || 600;
  opts.matrix = opts.matrix || [];
  opts.neighs = opts.neighs || [];
  opts.container = opts.container || "#chord-container";

  // Variables
  var matrix = opts.matrix,
    neighs = opts.neighs,
    total = 0;

  for (var i = 0; i < matrix.length; i++) {
    for (var j = 0; j < matrix.length; j++) {
      total += matrix[i][j];
    }
  }

  (function drawChord() {
    var w = opts.size,
      h = opts.size,
      r0 = Math.min(w, h) * 0.35,
      r1 = r0 * 1.15;

    var chord,
      fill,
      tooltip,
      svg;

    // Utility Functions

    function flowValue(value) {
      return Math.round(value) + " - " + (value * 100 / total).toFixed(2) + "%";
    }

    function fade(opacity) {
      return function(g, i) {
        svg.selectAll("g.chord path")
            .filter(function(d) {
              return d.source.index != i && d.target.index != i;
            })
          .transition()
            .style("opacity", opacity);
      };
    }

    function groupTicks(d) {
      var k = (d.endAngle - d.startAngle) / d.value;
      return [{
        angle: d.value * k / 2 + d.startAngle,
        label: neighs[d.index],
        percent: (d.value * 100 / total).toFixed(2)
      }];
    }

    function drawTicks() {
      var ticks = svg.append("g")
        .attr("class", "ticks")
        .attr("opacity", 0.1)
        .selectAll("g")
          .data(chord.groups)
        .enter().append("g")
        .selectAll("g")
          .data(groupTicks)
        .enter().append("g")
          .attr("transform", function(d) {
            return "rotate(" + (d.angle * 180 / Math.PI - 90) + ")" +
              "translate(" + r1 + ",0)";
          });
      ticks.append("line")
          .attr("x1", 1)
          .attr("y1", 0)
          .attr("x2", 5)
          .attr("y2", 0)
          .style("stroke", "#000");
      ticks.append("text")
          .attr("x", 8)
          .attr("dy", ".35em")
          .attr("text-anchor", function(d) {
            return d.angle > Math.PI ? "end" : null;
          })
          .attr("transform", function(d) {
            return d.angle > Math.PI ? "rotate(180)translate(-16)" : null;
          })
          .text(function(d) { return d.label.substring(0,7) + (d.label.length > 7 ? "." : ""); })
            .style("cursor", "default");

      svg.selectAll(".ticks").transition()
        .duration(340)
        .attr("opacity", 1);

      ticks
        .on("mouseover", function(d) {
        tooltip.style("visibility", "visible");
        tooltip.select(".src")
          .html(d.label);
        tooltip.select(".dst")
          .html(d.percent + "% of total");
      })
        .on("mouseout", function() {
          tooltip.style("visibility", "hidden");
          tooltip.select(".src").html("");
          tooltip.select(".dst").html("");
        })
        .on("mousemove", function() {
          return tooltip.style("top", (event.pageY-10)+"px")
                        .style("left",(event.pageX+10)+"px");
        });
    }

    // Render Chords

    chord = d3.layout.chord()
      .padding(0.05)
      .sortSubgroups(d3.descending)
      .matrix(matrix);

    fill = d3.scale.category20c().range(RB.Common.palette);

    svg = d3.select(opts.container)
      .append("svg:svg")
        .attr("width", w)
        .attr("height", h)
      .append("svg:g")
        .attr("transform", "translate(" + w / 2 + "," + h / 2 + ")");

    svg.append("svg:g")
      .attr("class", "neigh")
      .selectAll("path")
        .data(chord.groups)
      .enter().append("svg:path")
        .attr("data-neigh-index", function(d) { return d.index; })
        .style("fill", function(d) { return fill(d.index); })
        .style("stroke", function(d) { return fill(d.index); })
        .attr("d", d3.svg.arc().innerRadius(r0).outerRadius(r1))
        .on("mouseover", fade(0.1))
        .on("mouseout", fade(0.9));

    tooltip = d3.select("body")
      .append("div")
        .attr("class", "chord-tooltip")
        .style("visibility", "hidden");

    tooltip.append("div")
      .attr("class", "src");

    tooltip.append("div")
      .attr("class", "dst");

    svg.append("g")
      .attr("class", "chord")
      .selectAll("path")
        .data(chord.chords)
      .enter().append("path")
        .style("fill", function(d) { return fill(d.source.index); })
        .style("fill-opacity", 0.6)
        .style("stroke", "#000")
        .style("stroke-width", ".15px")
        .attr("d", d3.svg.chord().radius(r0))
        .on("mouseover", function(d) {
          tooltip.style("visibility", "visible");
          tooltip.select(".src")
            .html(
              neighs[d.source.index] + " → " + neighs[d.target.index] + "</br>" +
              flowValue(d.source.value)
            );
          if (neighs[d.source.index] !== neighs[d.target.index] &&
              d.target.value !== 0) {
            tooltip.select(".dst")
              .html(
                neighs[d.target.index] + " → " + neighs[d.source.index] + "</br>" +
                flowValue(d.target.value)
              );
          }
        })
        .on("mousemove", function() {
          return tooltip.style("top", (event.pageY-10)+"px")
                        .style("left",(event.pageX+10)+"px");
        })
        .on("mouseout", function() {
          tooltip.style("visibility", "hidden");
          tooltip.select(".src").html("");
          tooltip.select(".dst").html("");
        });

    drawTicks();
  })();
};

})(window, undefined);
