/*!
 * Copyright (c) 2014 ENEO Tecnología S.L.
 * This file is part of redBorder.
 * redBorder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * redBorder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with redBorder. If not, see <http://www.gnu.org/licenses/>.
 */

if (redBorder.malware === undefined){
  redBorder.malware = {}
}

/*
 * Loader settings library. This library calculate weight of the elements
 * to prevent errors.
 */
redBorder.malware.LoaderSettings = function() {

  // Initialize
  initialize();

  // Initialize the library
  function initialize() {
    callbacks();
    // Trigger some callbacks to initialize
    $('.loader-tab tbody .loader-group:first-child .loader-global input').trigger('input');
    $('.loader-tab tbody .loader-group .loader-values .form-group:first-child input').trigger('input');
  }

  /*
   * Add Callbacks to the elements
   */
  function callbacks(){
    $(document).on('input', '.loader-global input', function(event){
      var $el = $(this),
          $parent = $el.parents('.loader-tab'),
          $result = $('.loader-global-sum[data-sum="global-sum-' + $parent.data('loader-type') + '"] span'),
          value = parseFloat($el.val()),
          total = 0;
      // Clear values
      if (value > 1) {
        $el.val(1);
      } else if (value < 0) {
        $el.val(0);
      }
      // Calculate total
      $parent.find('.loader-global input').each(function(){
        var other_value = $(this).val() == '' ? 0 : parseFloat($(this).val());
        total += other_value;
      });
      total = Math.round(total * 100) / 100;
      // Display result
      showResult($result, total);
    });

    $(document).on('input', '.loader-values input', function(event){
      var $el = $(this),
          type = $el.parents('.loader-tab').data('loader-type'),
          $group = $el.parents('.loader-group'),
          group = $group.data('group'),
          value = parseFloat($el.val()),
          $result = $('.loader-sum[data-sum="sum-' + type + '-' + group + '"] span'),
          total = 0;
      // Clear values
      if (value > 1) {
        $el.val(1);
      } else if (value < 0) {
        $el.val(0);
      }
      // Calculate total
      $group.find('.loader-values input').each(function(){
        var other_value = $(this).val() == '' ? 0 : parseFloat($(this).val());
        total += other_value;
      });
      total = Math.round(total * 100) / 100;
      // Display result
      showResult($result, total);
    });

    // Prevent submit before check loaders
    $(document).on('submit', '#malware-settings form', function(event){
      // Check if its valid
      if(isValid()){
        blockWebUI();
      } else {
        // Prevent
        event.preventDefault();
        // Return button to normal mode
        setTimeout(function(){
          redBorder.forms.enableButton(
            '#malware-settings form .btn-success',
            'Update settings');
        }, 1000);
        // show a modal
        redBorder.modals.showModalWith(
          'Loader weight',
          'There is a <b>problem with the configured weight of loaders</b>, '+
            'the sum of every section must be 1. '+
            'Please, check problems before continue.',
          redBorder.modals.dismissButton('Back'),
          '#default_modal');

        return false;
      }
    });

    // List elements (Black and white list)
    $(document).on('click', '.destroy-list-element', function(event){
      $(this).parents('.list-element').addClass("removing");
    });


    // Search elements
    $(document).on('input', '.list-elements-search', function(e) {
      var $el = $(this),
          value = $el.val(),
          $parent = $el.parents('.list-elements-tab'),
          $list = $parent.find('.list-elements'),
          type = $parent.data('type'),
          list_type = $parent.data('list-type');

      if(value.length < 3){
        // Need more text
        $list.find('.list-element.hidden').removeClass('hidden');
      } else {
        $list.find('.list-element').each(function(i){
          if($(this).data('value').indexOf(value) == -1){
            $(this).addClass('hidden');
          }
        });
      }
    });

    $(document).on('keypress', 'form.settings', function(e) {
      if (e.keyCode == 13) { return false }
    });
  }

  /*
   * Show result and add CSS classes
   */
  function showResult(el, value){
    el.html(value);
    el.removeClass('ok error');
    if (value != 1) {
      el.addClass('error');
    } else {
      el.addClass('ok');
    }
  }

  /*
   * Destroy the callbacks generated by the library
   */
  function destroyCallbacks() {
    $(document).off('input', '.loader-global input');
    $(document).off('input', '.loader-values input');
    $(document).off('click', '#malware-settings form .btn-success');
    $(document).off('input', '.list-elements-search');
    $(document).off('click', '.destroy-list-element');
  }

  /*
   * Check all sums are 1
   */
  function isValid() {
    var valid = true;
    $('.loader-global-sum span').each(function(){
      var value = $(this).text() == '' ? 1 : parseFloat($(this).text());
      if (value != 1) {
        valid = false;
      }
    });

    $('.loader-sum span').each(function(){
      var value = $(this).text() == '' ? 1 : parseFloat($(this).text());
      if (value != 1) {
        valid = false;
      }
    });

    return valid;
  }

  return {
    destroyCallbacks: destroyCallbacks,
    isValid: isValid
  }
}

/*
 * GRR Functionality
 */
 redBorder.malware.GRRSettings = function() {
   var $el = $('.grr-status'),
       status,
       configured;

   // Initialize
   initialize();

   // Initialize the library
   function initialize() {
     reload();
     // Build callbacks
     callbacks();
   }

   // Attach callbacks
   function callbacks() {
     $(document).on('click', 'a.grr-start-services', function(event){
       event.preventDefault();
       if(!configured) {
         alertNotConfigured();
         return;
       }
       // Check status
       if(isOnline()){
         redBorder.modals.showModalWith(
           'GRR is already running',
           'GRR services are running now.',
           redBorder.modals.dismissButton('Back'),
           '#default_modal');
       } else {
         setLoading();
         $.ajax({
           url: '/malware/settings/grr',
           method: 'POST',
           dataType: 'json'
         }).then(update);
       }
     });

     $(document).on('click', 'a.grr-stop-services', function(event){
       event.preventDefault();
       if(!configured) {
         alertNotConfigured();
         return;
       }
       // Check status
       if(!isOnline()){
         redBorder.modals.showModalWith(
           'GRR is already stopped',
          'GRR is already stopped',
           redBorder.modals.dismissButton('Back'),
           '#default_modal');

       } else {
         setLoading();
         $.ajax({
           url: '/malware/settings/grr',
           method: 'DELETE',
           dataType: 'json'
         }).then(update);
       }
     });
   }

   // Destroy callbacks
   function destroyCallbacks() {
     $(document).off('click', 'a.grr-start-services');
     $(document).off('click', 'a.grr-stop-services');
   }

   // Reload the table
   function reload() {
     setLoading();
     $.ajax({
       url: '/malware/settings/grr',
       dataType: 'json'
     }).then(update);
   }

   // Show the data
   function update(data) {
     // Store last status
     status = data.status;
     configured = data.configured;

     if(!data.configured){
        // GRR Server is not configured!
        setNotConfigured();
     } else {
       // GRR was configured so show the data
       for (var service in data.status) {
         var $service = $el.find('tr[data-service="' + service + '"]');
         // Get formatted status
         var statusText = '<span class="red">Stopped</span>';
         if(data.status[service].running) {
           statusText = '<span class="green">Running</span>';
         }
         var pid = data.status[service].pid;
         pid = pid === -1 ? '-' : pid;
         // Update content
         $service.find('.service-status').html(statusText);
         $service.find('.service-pid').html(pid);
       }
     }
   }

   // Set current data as Loading
   function setLoading() {
     $el.find('table tr').each(function(i){
       $(this).find('.service-status').html(loadingText());
       $(this).find('.service-pid').html(loadingText());
     });
   }

   // Set not configured
   function setNotConfigured() {
     $el.find('table tr').each(function(i){
       $(this).find('.service-status').html(notConfiguredText());
       $(this).find('.service-pid').html(notConfiguredText());
     });
   }

   // Show an alert of configuration
   function alertNotConfigured() {
     redBorder.modals.showModalWith(
       'GRR is not configured',
       'Please set the IP, username and password of GRR server and save settings.',
       redBorder.modals.dismissButton('Back'),
       '#default_modal');
   }

   // Not configured
   function notConfiguredText() {
     return '<span class="dark-gray">GRR is not configured</span>';
   }

   // Loading text
   function loadingText() {
     return '<i class="fa fa-spin fa-spinner">';
   }

   // Check if the service is online
   function isOnline(){
     if(status === undefined || Object.keys(status).length === 0) { return false; }
     var online = true;

     for (var service in status) {
       if (!status[service].running) {
         online = false;
         break;
       }
     }

     return online;
   }

   return {
     reload: reload,
     destroyCallbacks: destroyCallbacks
   }
};

/* Listen changes on Relay_domain */
$(document).on('input', 'input[name="settings[mail[relay_domain]]"]', function() {
  var $el = $(this),
      original = $el.data('original-value'),
      current = $el.val(),
      $checkbox = $('.relay-domain-apply');

  if (original === current && $checkbox.hasClass('active')) {
    $checkbox.fadeOut(100);
    $checkbox.removeClass('active');
  } else if (original !== current && !$checkbox.hasClass('active')) {
    $checkbox.fadeIn(100);
    $checkbox.addClass('active');
  }
});
