redBorder.widgets.graphs.malware_fast_search = function (container, widget) {
  var graph,
      wtype,
      $container = $('#' + container),
      path = '.widget[data-widget-id="' + widget.id + '"] ';

  // The widget manager calls this method when the graph should start
  function init(_wtype) {
    wtype = _wtype;
  }

  function render(){
    // Bind events of the widget
    bindEvents();
    // This widget only need to call draw!
    draw();
  }

  // Draws the widget. This data comes directly from the widget
  // type selected. You should prepare the data to be used if needed.
  function draw() {
    redBorder.widgets.util.reloadDone($container.parents('.widget'));
  }

  // Draw response after receive data of the element
  function drawResponse(data) {
    // Check if we found data.
    if(data.found) {
      // Times
      var first_timestamp = new Date(data.times[0]).getTime() + 600 * 1000;
      var last_timestamp = new Date(data.times[1]).getTime() - 600 * 1000;
      var first_time = formatTime(new Date(first_timestamp));
      var last_time = formatTime(new Date(last_timestamp));
      // Show data
      var html =
        "<table class='table-icons'>" +
          "<thead>" +
            "<th><i class='fa fa-star-o'></i> Score</th>" +
            "<th><i class='fa fa-clock-o'></i> First view</th>" +
            "<th><i class='fa fa-clock-o'></i> Last view</th>" +
          "</thead>" +
          "<tbody>" +
            "<tr>" +
              "<td>" + data.data.score + "</td>" +
              "<td>" + first_time + "</td>" +
              "<td>" + last_time + "</td>" +
            "</tr>" +
          "</tbody>" +
        "</table>";
      // Add buttons
      var uri_value = encodeURIComponent(data.value);
      html +=
        "<div class='text-center margin-l'>" +
          "<a class='btn btn-secondary' target='_blank' href='/malware/" + uri_value +
            "/detail'><i class='fa fa-search'></i> Analysis</a>" +
          "<a class='btn btn-secondary' target='_blank' href='/malware/" + uri_value +
            "/evolution'><i class='fa fa-bar-chart'></i> Evolution</a>" +
          "<a class='btn btn-secondary' target='_blank' href='/malware/" + uri_value +
            "/propagation/ip'><i class='fa fa-code-fork'></i> Propagation</a>" +
          "<a href='#' class='btn btn-default fast-search-back'>Back</a>" +
        "</div>";
      // Show result
      $(path + '.fast-search-result-data').html(html);
      // Show it
      $(path + '.fast-search-wrap').fadeOut(100, function() {
        $(path + '.fast-search-result').fadeIn(100);
      });
      $(path + '.fast-search-result-data').show();
    } else {
      // Not found!
      notFound();
      $(path + '.fast-search-result-data').hide();
    }
  }

  function bindEvents(){
    // List search button click
    $(document).on('click', path + '.btn-success', function(event) {
      // Prevent click
      event.preventDefault();
      // Get value and search!
      var value = $(path + '#search_element_' + widget.id).val();
      // Only search if the length of the text is greater than 3
      if (value.length >= 3) {
        // Set the button as reloading
        $(this).addClass('disabled').html('<i class="fa fa-spin fa-spinner"></i>');
        $(path + '.not-found-value').text(value);
        wtype.queryElement(value).then(drawResponse);
      }
    });
    // Go back!
    $(document).on('click', path + '.fast-search-back', function(event) {
      // Prevent
      event.preventDefault();
      // Enable button
      $(path + '.btn-success').removeClass('disabled').html('Search');
      // Fade int search box
      $(path + '.fast-search-result').fadeOut(100, function() {
        $(path + '.fast-search-not-found').hide();
        $(path + '.fast-search-wrap').fadeIn(100);
      });
    });
  }

  // Clear input
  function clearInput(){
    $(path + '#search_element_' + widget.id).val('');
  }

  // Show a not found message
  function notFound(){
    $(path + '.fast-search-wrap').fadeOut(100, function(){
      clearInput();
      $(path + '.fast-search-not-found').show();
      $(path + '.fast-search-result').fadeIn(100);
      $(path + '.fast-search-result-data').hide();
    });
  }

  // Format time Output
  function formatTime(time){
    var month = (time.getMonth() + 1) < 10 ? "0" + (time.getMonth() + 1) : (time.getMonth() + 1);
    var day = time.getDate() < 10 ? "0" + time.getDate() : time.getDate();
    var hour = time.getHours() < 10 ? "0" + time.getHours() : time.getHours();
    var minute = time.getMinutes() < 10 ? "0" + time.getMinutes() : time.getMinutes();
    return time.getFullYear() + "-" + day + "-" + month +
        " " + hour + ":" + minute;
  }

  // Reload function
  function reload(){
    redBorder.widgets.util.reloadDone($container.parents('.widget'));
  }

  function resize() {
    // Todo
  }

  // Hide the main element when resizing
  function hide() {
    $container.fadeOut(100);
  }

  // Show the main element
  function show() {
    $container.fadeIn(100);
  }

  // Returns the graph object for debug purposes
  function getGraph() {
    return graph;
  }

  // Returns the widget
  function getWidget() {
    return widget;
  }

  return {
    init: init,
    resize: resize,
    getGraph: getGraph,
    getWidget: getWidget,
    hide: hide,
    show: show,
    draw: draw,
    reload: reload,
    render: render
  }
}
