/*!
 * Copyright (c) 2014 ENEO Tecnología S.L.
 * This file is part of redBorder.
 * redBorder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * redBorder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with redBorder. If not, see <http://www.gnu.org/licenses/>.
 */

redBorder.widgets.graphs.recent_malware = function (container, widget) {
  var graph,
      wtype,
      $container = $('#' + container),
      rendered = false,
      path = '.widget[data-widget-id="' + widget.id + '"] ',
      $table = $(path + 'table tbody'),
      malwareShowed = [],
      newMalwareAdded = false,
      threshold = 0;

  // The widget manager calls this method when the graph should start
  function init(_wtype) {
    wtype = _wtype;
  }

  function render(){
    // Bind events of the widget
    bindEvents();
    // This widget only need to call draw!
    wtype.queryRecent().then(draw);
  }

  // Draws the widget. This data comes directly from the widget
  // type selected. You should prepare the data to be used if needed.
  function draw(data) {
    if (data.malware.length == 0) {
      noRecent();
    } else {
      // Show list
      $(path + '.recent-malware-emtpy').fadeOut(50, function() {
        $(path + '.recent-malware-list').fadeIn(50);
      });
      // Store threshold
      threshold = data.threshold;
      // We don't add now any malware
      newMalwareAdded = false;
      // Build and add the data
      data.malware.forEach(function(element, index, array){
        if (!isShowed(element)) {
          var html = buildRow(data.sensors, element);
          $table.prepend(html);
          malwareShowed.unshift(element);
          // Check if we must remove another element.
          if(malwareShowed.length > data.entries){
            $(path + 'table tbody tr:last-child').remove();
            malwareShowed.pop();
          }
          // new malware!
          newMalwareAdded = true;
        }
      });
      // Draw malware icon
      if (rendered && newMalwareAdded) {
        newMalware();
        setTimeout(function(){
          $(path + '.table tr.new').removeClass('new');
        }, 4000);
      }
      // Clip it
      if(newMalwareAdded) { add_clipit_function(); }
    }
    // Store first render
    if (!rendered) { rendered = true; }
    initStickyHeader(widget.id);
    // Finish load
    redBorder.widgets.util.reloadDone($container.parents('.widget'));
  }

  function bindEvents(){
    // Todo...
  }

  function buildRow(sensors, malware){
    var css = rendered ? 'new' : '';
    return "<tr class='" + css + "'>" +
             "<td>" + formatTime(new Date(malware.timestamp)) + "</td>" +
             "<td>" + getSensor(sensors, malware.sensor_uuid, malware.sensor_name) + "</td>" +
             "<td class='row-center'>" + getScore(malware) + "</td>" +
             "<td>" + buildMenu(getValue(malware)) + "</td>" +
             "<td>" + value(malware.src) + "</td>" +
             "<td>" + value(malware.dst) + "</td>" +
             "<td>" + getMailId(malware.email_id, malware.timestamp) + "</td>" +
           "</tr>";
  }

  // Build a menu for a given value
  function buildMenu(value) {
    var uri_value = encodeURIComponent(value);
    return '<span class="prelative">' +
             '<a class="dropdown-toggle" data-toggle="dropdown" href="#">' + value + '</a>' +
             '<ul class="dropdown-menu">' +
               "<li>" +
                 '<a class="js-copy" href="#" data-clipboard-text="' + value + '"><i class="fa fa-clipboard"></i> Copy To Clipboard</a>' +
               "</li>" +
               '<li class="divider"></li>' +
               "<li><a target='_blank' href='/malware/" + uri_value + "/detail'>" +
                 "<i class='fa fa-search'></i> Analysis</a></li>" +
               "<li><a target='_blank' href='/malware/" + uri_value + "/evolution'>" +
                 "<i class='fa fa-bar-chart'></i> Evolution</a></li>" +
               "<li><a target='_blank' href='/malware/" + uri_value + "/propagation/ip'>" +
                 "<i class='fa fa-code-fork'></i> Propagation</a></li>" +
             '</ul>' +
           '</span>';
  }

  // Show a recent malware message
  function noRecent() {
    $(path + '.recent-malware-list').fadeOut(50, function() {
      $(path + '.recent-malware-emtpy').fadeIn(50);
    });
  }

  // New malware
  function newMalware() {
    $(path + '.recent-malware-new').addClass('fade-pulse');
  }

  // Check if the element is already showed
  function isShowed(element) {
    var exists = malwareShowed.some(function(malware){
        if (getValue(malware) === getValue(element) &&
            getScore(malware) === getScore(element) &&
            malware.src_name === element.src_name &&
            malware.dst_name === element.dst_name &&
            malware.timestamp === element.timestamp) {
              return true;
            }
    });
    // Return
    return exists;
  }

  // Return Sensor
  function getSensor(sensors, uuid, sensor_name) {
    // Fetch sensor
    sobject = sensors[uuid]
    if (sobject === undefined){
      if (uuid === "ICAP"){
        return '<span>' + uuid + '</span>';
      } else if (sensor_name !== undefined) {
        return '<span>' + sensor_name + '</span>';
      } else {
        return '<span class="gray">Not found</span>';
      }
    }

    if (sobject.icon_type === 'image'){
      html = '<img class="custom-icon" src="' + sobject.icon_asset + '"/>';
    } else {
      html = '<i class="fa fa-' + sobject.icon + '"></i>';
    }
    // Return HTML and the name
    return html + ' ' + sobject.name;
  }

  // Return the value or '-'
  function value(value) {
    var ret = '-'

    if(value !== undefined && value !== '' && value !== null){
      ret = value;
    }

    return ret;
  }

  // Get the mail ID and a button
  function getMailId(email_id, time) {
    if (email_id === '' || email_id === undefined || email_id === null) {
      return '-';
    }
    // Format time
    time = encodeURIComponent(time);
    var button = '';
    // Build button
    button = '<a href="/malware/' + email_id + '/mail?time=' + time + '&range=last_24" ' +
             'class="btn btn-small" data-toggle="modal" data-target="#default_large">' +
             '<i class="fa fa-info"></i></a>';

    return email_id + ' ' + button;
  }

  // Get the value of an element
  function getValue(malware) {
    return 'hash' in malware ? malware.hash : malware.url;
  }

  // Get the score of an element
  function getScore(malware) {
    var val = 'hash_score' in malware ? malware.hash_score : malware.url_score,
        html = '';
    // Force int
    val = parseInt(val);

    if(val == -1) {
      html = '<span class="gray">Undefined</span>';
    } else if (val >= threshold) {
      html = '<span class="red">Malware (' + val + ')</span>';
    } else {
      html = '<span class="green">Clean (' + val + ')</span>';
    }
    // Return HTML
    return html;
  }

  // Format time Output
  function formatTime(time){
    var months = ["Jan", "Feb", "Mar", "Apr", "May", "Jun",
      "Jul", "Aug", "Sept", "Oct", "Nov", "Dec"];

    var day = time.getDate();
    var month = months[time.getMonth()];

    var hours = time.getHours() < 10 ? "0" + time.getHours() : time.getHours();
    var minutes = time.getMinutes() < 10 ? "0" + time.getMinutes() : time.getMinutes();
    return day + ' ' + month + ' at ' + hours + ":" + minutes;
  }

  // Reload function
  function reload(){
    // Remove animation
    $(path + '.recent-malware-new').removeClass('fade-pulse');
    // Get data
    wtype.queryRecent().then(draw);
  }

  function resize() {
    var height = (widget.name.length !== 0) ? 43 : 0;
    $container.css('height', ($container.parents('.widget').height() - height) + 'px');
  }

  // Hide the main element when resizing
  function hide() {
    $container.fadeOut(100);
  }

  // Show the main element
  function show() {
    $container.fadeIn(100, function() { initStickyHeader(widget.id) });
  }

  // Returns the graph object for debug purposes
  function getGraph() {
    return graph;
  }

  // Returns the widget
  function getWidget() {
    return widget;
  }

  return {
    init: init,
    resize: resize,
    getGraph: getGraph,
    getWidget: getWidget,
    hide: hide,
    show: show,
    draw: draw,
    reload: reload,
    render: render
  }
}
